@@ -2,13 +2,14 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006-2007 Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -250,7 +251,7 @@
             MPI_Fint attr_val = translate_to_fortran_mpi1(attribute); \
             (*((keyval_obj->delete_attr_fn).attr_mpi1_fortran_delete_fn)) \
                 (&(((ompi_##type##_t *)object)->attr_##type##_f), \
-                 &f_key, &attr_val, keyval_obj->extra_state, &f_err); \
+                 &f_key, &attr_val, (int*)keyval_obj->extra_state, &f_err); \
             if (MPI_SUCCESS != OMPI_FINT_2_INT(f_err)) { \
                 if (need_lock) { \
                     OPAL_THREAD_UNLOCK(&alock); \
@@ -263,7 +264,7 @@
             MPI_Aint attr_val = translate_to_fortran_mpi2(attribute); \
             (*((keyval_obj->delete_attr_fn).attr_mpi2_fortran_delete_fn)) \
                 (&(((ompi_##type##_t *)object)->attr_##type##_f), \
-                 &f_key, &attr_val, keyval_obj->extra_state, &f_err); \
+                 &f_key, (int*)&attr_val, (int*)keyval_obj->extra_state, &f_err); \
             if (MPI_SUCCESS != OMPI_FINT_2_INT(f_err)) { \
                 if (need_lock) { \
                     OPAL_THREAD_UNLOCK(&alock); \
@@ -277,12 +278,12 @@
         void *attr_val = translate_to_c(attribute); \
         if ((err = (*((keyval_obj->delete_attr_fn).attr_##type##_delete_fn)) \
                             ((ompi_##type##_t *)object, \
-			    key, attr_val, \
-			    keyval_obj->extra_state)) != MPI_SUCCESS) {\
+                            key, attr_val, \
+                            keyval_obj->extra_state)) != MPI_SUCCESS) {\
             if (need_lock) { \
                 OPAL_THREAD_UNLOCK(&alock); \
             } \
-	    return err;\
+            return err;\
         } \
     }
 
@@ -300,7 +301,7 @@
             in = translate_to_fortran_mpi1(in_attr); \
             (*((keyval_obj->copy_attr_fn).attr_mpi1_fortran_copy_fn)) \
                 (&(((ompi_##type##_t *)old_object)->attr_##type##_f), \
-                 &f_key, keyval_obj->extra_state, \
+                 &f_key, (int*)keyval_obj->extra_state, \
                  &in, &out, &f_flag, &f_err); \
             if (MPI_SUCCESS != OMPI_FINT_2_INT(f_err)) { \
                 OPAL_THREAD_UNLOCK(&alock); \
@@ -366,13 +367,13 @@ typedef struct attribute_value_t {
  * Local functions
  */
 static void attribute_value_construct(attribute_value_t *item);
-static void ompi_attrkey_item_construct(ompi_attrkey_item_t *item);
-static void ompi_attrkey_item_destruct(ompi_attrkey_item_t *item);
+static void ompi_attribute_keyval_construct(ompi_attribute_keyval_t *keyval);
+static void ompi_attribute_keyval_destruct(ompi_attribute_keyval_t *keyval);
 static int set_value(ompi_attribute_type_t type, void *object, 
-                     opal_hash_table_t **keyhash, int key, 
+                     opal_hash_table_t **attr_hash, int key, 
                      attribute_value_t *new_attr,
                      bool predefined, bool need_lock);
-static int get_value(opal_hash_table_t *keyhash, int key, 
+static int get_value(opal_hash_table_t *attr_hash, int key, 
                      attribute_value_t **attribute, int *flag);
 static void *translate_to_c(attribute_value_t *val);
 static MPI_Fint translate_to_fortran_mpi1(attribute_value_t *val);
@@ -391,10 +392,10 @@ static OBJ_CLASS_INSTANCE(attribute_value_t,
 /*
  * ompi_attribute_entry_t classes
  */
-static OBJ_CLASS_INSTANCE(ompi_attrkey_item_t, 
+static OBJ_CLASS_INSTANCE(ompi_attribute_keyval_t, 
                           opal_object_t,
-                          ompi_attrkey_item_construct,
-                          ompi_attrkey_item_destruct);
+                          ompi_attribute_keyval_construct,
+                          ompi_attribute_keyval_destruct);
 
 
 /* 
@@ -430,25 +431,43 @@ static void attribute_value_construct(attribute_value_t *item)
 
 
 /*
- * ompi_attrkey_item_t constructor / destructor
+ * ompi_attribute_keyval_t constructor / destructor
  */
 static void
-ompi_attrkey_item_construct(ompi_attrkey_item_t *item) 
+ompi_attribute_keyval_construct(ompi_attribute_keyval_t *keyval) 
 {
-    memset(&(item->attr_type), 0, 
-	   sizeof(ompi_attrkey_item_t) - sizeof(opal_object_t));
+    keyval->attr_type = UNUSED_ATTR;
+    keyval->attr_flag = 0;
+    keyval->copy_attr_fn.attr_communicator_copy_fn = NULL;
+    keyval->delete_attr_fn.attr_communicator_copy_fn = NULL;
+    keyval->extra_state = NULL;
+    keyval->extra_destructor = NULL;
+
+    /* Set the keyval->key value to an invalid value so that we can know
+       if it has been initialized with a proper value or not.
+       Specifically, the destructor may get invoked if we weren't able
+       to assign a key properly.  So we don't want to try to remove it
+       from the table if it wasn't there. */
+    keyval->key = -1;
 }
 
 
 static void 
-ompi_attrkey_item_destruct(ompi_attrkey_item_t *item) 
+ompi_attribute_keyval_destruct(ompi_attribute_keyval_t *keyval) 
 {
-    /* Remove the key entry from the hash and free the key */
+    /* THIS FUNCTION ASSUMES THAT THE CALLER ALREADY HAS OBTAINED THE
+       alock MUTEX!  Remove the keyval entry from the hash and free
+       the key. */
 
-    OPAL_THREAD_LOCK(&alock);
-    opal_hash_table_remove_value_uint32(keyval_hash, item->key);
-    FREE_KEY(item->key);
-    OPAL_THREAD_UNLOCK(&alock);
+    if (-1 != keyval->key) {
+        /* If the destructor function pointer is not NULL, call it */
+        if (NULL != keyval->extra_destructor) {
+            keyval->extra_destructor(keyval->key);
+        }
+
+        opal_hash_table_remove_value_uint32(keyval_hash, keyval->key);
+        FREE_KEY(keyval->key);
+    }
 }
 
 
@@ -465,11 +484,11 @@ int ompi_attr_init(void)
 
     keyval_hash = OBJ_NEW(opal_hash_table_t);
     if (NULL == keyval_hash) {
-	return MPI_ERR_SYSRESOURCE;
+        return MPI_ERR_SYSRESOURCE;
     }
     key_bitmap = OBJ_NEW(ompi_bitmap_t);
-    if (0 != ompi_bitmap_init(key_bitmap, 10)) {
-	return MPI_ERR_SYSRESOURCE;
+    if (0 != ompi_bitmap_init(key_bitmap, 32)) {
+        return MPI_ERR_SYSRESOURCE;
     }
 
     for (int_pos = 0; int_pos < (sizeof(void*) / sizeof(MPI_Fint)); 
@@ -485,7 +504,7 @@ int ompi_attr_init(void)
 
     if (OMPI_SUCCESS != (ret = opal_hash_table_init(keyval_hash,
                                                     ATTR_TABLE_SIZE))) {
-	return ret;
+        return ret;
     }
     if (OMPI_SUCCESS != (ret = ompi_attr_create_predefined())) {
         return ret;
@@ -514,47 +533,52 @@ int ompi_attr_finalize(void)
 int ompi_attr_create_keyval(ompi_attribute_type_t type,
                             ompi_attribute_fn_ptr_union_t copy_attr_fn,
                             ompi_attribute_fn_ptr_union_t delete_attr_fn,
-                            int *key, void *extra_state, int flags)
+                            int *key, void *extra_state, int flags,
+                            ompi_attribute_keyval_destructor_fn_t *destructor_fn)
 {
-    ompi_attrkey_item_t *attr;
+    ompi_attribute_keyval_t *keyval;
     int ret;
 
     /* Protect against the user calling ompi_attr_destroy and then
        calling any of the functions which use it  */
     if (NULL == keyval_hash) {
-	return MPI_ERR_INTERN;
+        return MPI_ERR_INTERN;
     }
 
     /* Allocate space for the list item */
 
-    attr = OBJ_NEW(ompi_attrkey_item_t);
-    if (NULL == attr) {
-	return MPI_ERR_SYSRESOURCE;
+    keyval = OBJ_NEW(ompi_attribute_keyval_t);
+    if (NULL == keyval) {
+        return MPI_ERR_SYSRESOURCE;
     }
 
+    /* Fill in the list item (must be done before we set the keyval
+       on the keyval_hash in case some other thread immediately reads
+       it from the keyval_hash) */
+  
+    keyval->copy_attr_fn = copy_attr_fn;
+    keyval->delete_attr_fn = delete_attr_fn;
+    keyval->extra_state = extra_state;
+    keyval->attr_type = type;
+    keyval->attr_flag = flags;
+    keyval->key = -1;
+    keyval->extra_destructor = destructor_fn;
+
     /* Create a new unique key and fill the hash */
   
     OPAL_THREAD_LOCK(&alock);
     ret = CREATE_KEY(key);
     if (OMPI_SUCCESS == ret) {
-        ret = opal_hash_table_set_value_uint32(keyval_hash, *key, attr);
+        keyval->key = *key;
+        ret = opal_hash_table_set_value_uint32(keyval_hash, *key, keyval);
     }
-    OPAL_THREAD_UNLOCK(&alock);
     if (OMPI_SUCCESS != ret) {
-	return ret;
+        OBJ_RELEASE(keyval);
+        OPAL_THREAD_UNLOCK(&alock);
+        return ret;
     }
 
-    /* Fill in the list item */
-  
-    attr->copy_attr_fn = copy_attr_fn;
-    attr->delete_attr_fn = delete_attr_fn;
-    attr->extra_state = extra_state;
-    attr->attr_type = type;
-    attr->key = *key;
-    attr->attr_flag = flags;
-
-    /* Other fields will be filled in by the create_attr function */
-  
+    OPAL_THREAD_UNLOCK(&alock);
     return MPI_SUCCESS;
 }
 
@@ -563,25 +587,25 @@ int ompi_attr_free_keyval(ompi_attribute_type_t type, int *key,
                           bool predefined)
 {
     int ret;
-    ompi_attrkey_item_t *key_item;
+    ompi_attribute_keyval_t *keyval;
 
     /* Protect against the user calling ompi_attr_destroy and then
        calling any of the functions which use it  */
     if (NULL == keyval_hash) {
-	return MPI_ERR_INTERN;
+        return MPI_ERR_INTERN;
     }
 
     /* Find the key-value pair */
 
     OPAL_THREAD_LOCK(&alock);
     ret = opal_hash_table_get_value_uint32(keyval_hash, *key, 
-					   (void **) &key_item);
-    OPAL_THREAD_UNLOCK(&alock);
+                                           (void **) &keyval);
   
-    if ((OMPI_SUCCESS != ret) || (NULL == key_item) || 
-        (key_item->attr_type != type) ||
-	((!predefined) && (key_item->attr_flag & OMPI_KEYVAL_PREDEFINED))) {
-	return OMPI_ERR_BAD_PARAM;
+    if ((OMPI_SUCCESS != ret) || (NULL == keyval) || 
+        (keyval->attr_type != type) ||
+        ((!predefined) && (keyval->attr_flag & OMPI_KEYVAL_PREDEFINED))) {
+        OPAL_THREAD_UNLOCK(&alock);
+        return OMPI_ERR_BAD_PARAM;
     }
 
     /* MPI says to set the returned value to MPI_KEYVAL_INVALID */
@@ -592,24 +616,25 @@ int ompi_attr_free_keyval(ompi_attribute_type_t type, int *key,
        with it, else it will just decrement the reference count, so that when
        the last attribute is deleted, this object gets deleted too */
 
-    OBJ_RELEASE(key_item);
+    OBJ_RELEASE(keyval);
+    OPAL_THREAD_UNLOCK(&alock);
 
     return MPI_SUCCESS;
 }
 
 
 int ompi_attr_delete(ompi_attribute_type_t type, void *object, 
-                     opal_hash_table_t *keyhash, int key,
+                     opal_hash_table_t *attr_hash, int key,
                      bool predefined, bool need_lock) 
 {
-    ompi_attrkey_item_t *key_item;
+    ompi_attribute_keyval_t *keyval;
     int ret = OMPI_SUCCESS, err;
     attribute_value_t *attr;
 
     /* Protect against the user calling ompi_attr_destroy and then
        calling any of the functions which use it  */
     if (NULL == keyval_hash) {
-	return MPI_ERR_INTERN;
+        return MPI_ERR_INTERN;
     }
 
     /* Note that this function can be invoked by
@@ -625,64 +650,69 @@ int ompi_attr_delete(ompi_attribute_type_t type, void *object,
     /* Check if the key is valid in the master keyval hash */
 
     ret = opal_hash_table_get_value_uint32(keyval_hash, key, 
-					   (void **) &key_item);
+                                           (void **) &keyval);
 
-    if ((OMPI_SUCCESS != ret) || (NULL == key_item) ||
-        (key_item->attr_type!= type) ||
-	((!predefined) && (key_item->attr_flag & OMPI_KEYVAL_PREDEFINED))) {
-	ret = OMPI_ERR_BAD_PARAM;
-	goto exit;
+    if ((OMPI_SUCCESS != ret) || (NULL == keyval) ||
+        (keyval->attr_type!= type) ||
+        ((!predefined) && (keyval->attr_flag & OMPI_KEYVAL_PREDEFINED))) {
+        ret = OMPI_ERR_BAD_PARAM;
+        goto exit;
     }
 
-    /* Ensure that we don't have an empty keyhash */
+    /* Ensure that we don't have an empty attr_hash */
 
-    if (NULL == keyhash) {
+    if (NULL == attr_hash) {
         ret = OMPI_ERR_BAD_PARAM;
-	goto exit;
+        goto exit;
     }
 
     /* Check if the key is valid for the communicator/window/dtype. If
-       yes, then delete the attribute and key entry from the object's key
+       yes, then delete the attribute and key entry from the object's
        hash */
 
-    ret = opal_hash_table_get_value_uint32(keyhash, key, (void**) &attr);
+    ret = opal_hash_table_get_value_uint32(attr_hash, key, (void**) &attr);
     if (OMPI_SUCCESS == ret) {
-	switch (type) {
+        switch (type) {
         case COMM_ATTR:
-            DELETE_ATTR_CALLBACKS(communicator, attr, key_item, object);
+            DELETE_ATTR_CALLBACKS(communicator, attr, keyval, object);
             break;
-		
+                
         case WIN_ATTR:
-            DELETE_ATTR_CALLBACKS(win, attr, key_item, object);
+            DELETE_ATTR_CALLBACKS(win, attr, keyval, object);
             break;
-		
+                
         case TYPE_ATTR:
-            DELETE_ATTR_CALLBACKS(datatype, attr, key_item, object);
+            DELETE_ATTR_CALLBACKS(datatype, attr, keyval, object);
             break;
-		
+                
         default:
             ret = MPI_ERR_INTERN;
             goto exit;
-	}
+        }
         OBJ_RELEASE(attr);
     
-	ret = opal_hash_table_remove_value_uint32(keyhash, key);
-	if (OMPI_SUCCESS != ret) {
-	    goto exit;
-	}
+        ret = opal_hash_table_remove_value_uint32(attr_hash, key);
+        if (OMPI_SUCCESS != ret) {
+            goto exit;
+        }
     }
 
 
  exit:
+    /* Decrement the ref count for the keyval.  If ref count goes to
+       0, destroy the keyval (the destructor deletes the key
+       implicitly for this object).  The ref count will only go to 0
+       here if MPI_*_FREE_KEYVAL was previously invoked and we just
+       freed the last attribute that was using the keyval. */
+
+    if (OMPI_SUCCESS == ret) {
+        OBJ_RELEASE(keyval);
+    }
+
     if (need_lock) {
-	OPAL_THREAD_UNLOCK(&alock);
-    }	
+        OPAL_THREAD_UNLOCK(&alock);
+    }        
 
-    /* Decrement the ref count for the key, and if ref count is 0,
-       remove the key (the destructor deletes the key implicitly for
-       this object */
-    
-    OBJ_RELEASE(key_item);
     return ret;
 }
 
@@ -692,7 +722,7 @@ int ompi_attr_delete(ompi_attribute_type_t type, void *object,
  * attribute.
  */
 int ompi_attr_set_c(ompi_attribute_type_t type, void *object, 
-                    opal_hash_table_t **keyhash,
+                    opal_hash_table_t **attr_hash,
                     int key, void *attribute, bool predefined, bool need_lock)
 {
     attribute_value_t *new_attr = OBJ_NEW(attribute_value_t);
@@ -702,7 +732,7 @@ int ompi_attr_set_c(ompi_attribute_type_t type, void *object,
 
     new_attr->av_value = attribute;
     new_attr->av_set_from = OMPI_ATTRIBUTE_C;
-    return set_value(type, object, keyhash, key, new_attr,
+    return set_value(type, object, attr_hash, key, new_attr,
                      predefined, need_lock);
 }
 
@@ -712,7 +742,7 @@ int ompi_attr_set_c(ompi_attribute_type_t type, void *object,
  * an attribute.
  */
 int ompi_attr_set_fortran_mpi1(ompi_attribute_type_t type, void *object, 
-                               opal_hash_table_t **keyhash,
+                               opal_hash_table_t **attr_hash,
                                int key, MPI_Fint attribute, 
                                bool predefined, bool need_lock)
 {
@@ -724,7 +754,7 @@ int ompi_attr_set_fortran_mpi1(ompi_attribute_type_t type, void *object,
     new_attr->av_value = (void *) 0;
     *new_attr->av_integer_pointer = attribute;
     new_attr->av_set_from = OMPI_ATTRIBUTE_FORTRAN_MPI1;
-    return set_value(type, object, keyhash, key, new_attr,
+    return set_value(type, object, attr_hash, key, new_attr,
                      predefined, need_lock);
 }
 
@@ -734,7 +764,7 @@ int ompi_attr_set_fortran_mpi1(ompi_attribute_type_t type, void *object,
  * an attribute.
  */
 int ompi_attr_set_fortran_mpi2(ompi_attribute_type_t type, void *object, 
-                               opal_hash_table_t **keyhash,
+                               opal_hash_table_t **attr_hash,
                                int key, MPI_Aint attribute, 
                                bool predefined, bool need_lock)
 {
@@ -745,7 +775,7 @@ int ompi_attr_set_fortran_mpi2(ompi_attribute_type_t type, void *object,
 
     new_attr->av_value = (void *) attribute;
     new_attr->av_set_from = OMPI_ATTRIBUTE_FORTRAN_MPI2;
-    return set_value(type, object, keyhash, key, new_attr,
+    return set_value(type, object, attr_hash, key, new_attr,
                      predefined, need_lock);
 }
 
@@ -754,13 +784,13 @@ int ompi_attr_set_fortran_mpi2(ompi_attribute_type_t type, void *object,
  * Front-end function called by the C MPI API functions to get
  * attributes.
  */
-int ompi_attr_get_c(opal_hash_table_t *keyhash, int key, 
+int ompi_attr_get_c(opal_hash_table_t *attr_hash, int key, 
                     void **attribute, int *flag)
 {
     attribute_value_t *val = NULL;
     int ret;
 
-    ret = get_value(keyhash, key, &val, flag);
+    ret = get_value(attr_hash, key, &val, flag);
     if (MPI_SUCCESS == ret && 1 == *flag) {
         *attribute = translate_to_c(val);
     }
@@ -773,13 +803,13 @@ int ompi_attr_get_c(opal_hash_table_t *keyhash, int key,
  * Front-end function called by the Fortran MPI-1 API functions to get
  * attributes.
  */
-int ompi_attr_get_fortran_mpi1(opal_hash_table_t *keyhash, int key, 
+int ompi_attr_get_fortran_mpi1(opal_hash_table_t *attr_hash, int key, 
                                MPI_Fint *attribute, int *flag)
 {
     attribute_value_t *val = NULL;
     int ret;
 
-    ret = get_value(keyhash, key, &val, flag);
+    ret = get_value(attr_hash, key, &val, flag);
     if (MPI_SUCCESS == ret && 1 == *flag) {
         *attribute = translate_to_fortran_mpi1(val);
     }
@@ -792,13 +822,13 @@ int ompi_attr_get_fortran_mpi1(opal_hash_table_t *keyhash, int key,
  * Front-end function called by the Fortran MPI-2 API functions to get
  * attributes.
  */
-int ompi_attr_get_fortran_mpi2(opal_hash_table_t *keyhash, int key, 
+int ompi_attr_get_fortran_mpi2(opal_hash_table_t *attr_hash, int key, 
                                MPI_Aint *attribute, int *flag)
 {
     attribute_value_t *val = NULL;
     int ret;
 
-    ret = get_value(keyhash, key, &val, flag);
+    ret = get_value(attr_hash, key, &val, flag);
     if (MPI_SUCCESS == ret && 1 == *flag) {
         *attribute = translate_to_fortran_mpi2(val);
     }
@@ -811,8 +841,8 @@ int ompi_attr_get_fortran_mpi2(opal_hash_table_t *keyhash, int key,
  * Copy all the attributes from one MPI object to another
  */
 int ompi_attr_copy_all(ompi_attribute_type_t type, void *old_object, 
-                       void *new_object, opal_hash_table_t *oldkeyhash,
-                       opal_hash_table_t *newkeyhash)
+                       void *new_object, opal_hash_table_t *oldattr_hash,
+                       opal_hash_table_t *newattr_hash)
 {
     int ret;
     int err;
@@ -820,7 +850,7 @@ int ompi_attr_copy_all(ompi_attribute_type_t type, void *old_object,
     int flag;
     void *node, *in_node;
     attribute_value_t *old_attr, *new_attr;
-    ompi_attrkey_item_t *hash_value;
+    ompi_attribute_keyval_t *hash_value;
 
     /* Protect against the user calling ompi_attr_destroy and then
        calling any of the functions which use it  */
@@ -830,7 +860,7 @@ int ompi_attr_copy_all(ompi_attribute_type_t type, void *old_object,
 
     /* If there's nothing to do, just return */
 
-    if (NULL == oldkeyhash) {
+    if (NULL == oldattr_hash) {
         return MPI_SUCCESS;
     }
 
@@ -840,17 +870,16 @@ int ompi_attr_copy_all(ompi_attribute_type_t type, void *old_object,
 
     OPAL_THREAD_LOCK(&alock);
 
-    /* Get the first key-attr in the object's key hash */
-    ret = opal_hash_table_get_first_key_uint32(oldkeyhash, &key, 
+    /* Get the first attribute in the object's hash */
+    ret = opal_hash_table_get_first_key_uint32(oldattr_hash, &key, 
                                                (void **) &old_attr,
                                                &node);
 
-    /* While we still have some key-attr pair in the object's key
-       hash */
+    /* While we still have some attribute in the object's key hash */
     while (OMPI_SUCCESS == ret) {
         in_node = node;
 
-        /* Get the attr_item in the main keyval hash - so that we know
+        /* Get the keyval in the main keyval hash - so that we know
            what the copy_attr_fn is */
 
         err = opal_hash_table_get_value_uint32(keyval_hash, key, 
@@ -858,12 +887,15 @@ int ompi_attr_copy_all(ompi_attribute_type_t type, void *old_object,
 
         new_attr = OBJ_NEW(attribute_value_t);
         switch (type) {
+        case UNUSED_ATTR:  /* keep the compiler happy */
+            assert(0);
+            break;
         case COMM_ATTR:
             /* Now call the copy_attr_fn */
             COPY_ATTR_CALLBACKS(communicator, old_object, hash_value, 
                                 old_attr, new_object, new_attr);
             break;
-	    
+            
         case TYPE_ATTR:
             /* Now call the copy_attr_fn */
             COPY_ATTR_CALLBACKS(datatype, old_object, hash_value, 
@@ -877,8 +909,8 @@ int ompi_attr_copy_all(ompi_attribute_type_t type, void *old_object,
             break;
         }
 
-        /* Hang this off the object's key hash */
-	    
+        /* Hang this off the object's hash */
+            
         /* The "predefined" parameter to ompi_attr_set() is set to 1,
            so that no comparison is done for prdefined at all and it
            just falls off the error checking loop in attr_set  */
@@ -893,13 +925,13 @@ int ompi_attr_copy_all(ompi_attribute_type_t type, void *old_object,
             } else {
                 new_attr->av_set_from = OMPI_ATTRIBUTE_C;
             }
-            set_value(type, new_object, &newkeyhash, key, 
+            set_value(type, new_object, &newattr_hash, key, 
                       new_attr, true, false);
         } else {
             OBJ_RELEASE(new_attr);
         }
 
-        ret = opal_hash_table_get_next_key_uint32(oldkeyhash, &key, 
+        ret = opal_hash_table_get_next_key_uint32(oldattr_hash, &key, 
                                                   (void **) &old_attr, 
                                                   in_node, &node);
     }
@@ -915,7 +947,7 @@ int ompi_attr_copy_all(ompi_attribute_type_t type, void *old_object,
  * Delete all the attributes on an MPI object
  */
 int ompi_attr_delete_all(ompi_attribute_type_t type, void *object, 
-                         opal_hash_table_t *keyhash)
+                         opal_hash_table_t *attr_hash)
 {
     int key_ret, del_ret;
     uint32_t key, oldkey;
@@ -929,18 +961,18 @@ int ompi_attr_delete_all(ompi_attribute_type_t type, void *object,
 
     /* Ensure that the table is not empty */
 
-    if (NULL == keyhash) {
+    if (NULL == attr_hash) {
         return MPI_SUCCESS;
     }
-	
+        
     /* Lock this whole sequence of events -- don't let any other
        thread modify the structure of the keyval hash or bitmap while
        we're traversing it */
 
     OPAL_THREAD_LOCK(&alock);
 
-    /* Get the first key in local object's key hash  */
-    key_ret = opal_hash_table_get_first_key_uint32(keyhash,
+    /* Get the first key in local object's hash  */
+    key_ret = opal_hash_table_get_first_key_uint32(attr_hash,
                                                &key, &old_attr,
                                                &node);
     del_ret = OMPI_SUCCESS;
@@ -951,15 +983,15 @@ int ompi_attr_delete_all(ompi_attribute_type_t type, void *object,
 
         in_node = node;
         oldkey = key;
-	
+        
         /* Move to the next node */
 
-        key_ret = opal_hash_table_get_next_key_uint32(keyhash,
+        key_ret = opal_hash_table_get_next_key_uint32(attr_hash,
                                                       &key, &old_attr, 
                                                       in_node, &node);
         /* Now delete this attribute */
 
-        del_ret = ompi_attr_delete(type, object, keyhash, oldkey, true, false);
+        del_ret = ompi_attr_delete(type, object, attr_hash, oldkey, true, false);
     }
 
     /* All done */
@@ -974,11 +1006,11 @@ int ompi_attr_delete_all(ompi_attribute_type_t type, void *object,
  * Back-end function to set an attribute on an MPI object
  */
 static int set_value(ompi_attribute_type_t type, void *object, 
-                     opal_hash_table_t **keyhash, int key, 
+                     opal_hash_table_t **attr_hash, int key, 
                      attribute_value_t *new_attr,
                      bool predefined, bool need_lock)
 {
-    ompi_attrkey_item_t *key_item;
+    ompi_attribute_keyval_t *keyval;
     int ret, err;
     attribute_value_t *old_attr;
     bool had_old = false;
@@ -986,9 +1018,9 @@ static int set_value(ompi_attribute_type_t type, void *object,
     /* Protect against the user calling ompi_attr_destroy and then
        calling any of the functions which use it  */
     if (NULL == keyval_hash) {
-	return MPI_ERR_INTERN;
+        return MPI_ERR_INTERN;
     }
-    if (NULL == keyhash) {
+    if (NULL == attr_hash) {
         return MPI_ERR_INTERN;
     }
 
@@ -1002,65 +1034,71 @@ static int set_value(ompi_attribute_type_t type, void *object,
         OPAL_THREAD_LOCK(&alock);
     }
     ret = opal_hash_table_get_value_uint32(keyval_hash, key, 
-					   (void **) &key_item);
+                                           (void **) &keyval);
 
     /* If key not found */
 
-    if ((OMPI_SUCCESS != ret ) || (NULL == key_item) || 
-        (key_item->attr_type != type) ||
-	((!predefined) && (key_item->attr_flag & OMPI_KEYVAL_PREDEFINED))) {
+    if ((OMPI_SUCCESS != ret ) || (NULL == keyval) || 
+        (keyval->attr_type != type) ||
+        ((!predefined) && (keyval->attr_flag & OMPI_KEYVAL_PREDEFINED))) {
         if (need_lock) {
             OPAL_THREAD_UNLOCK(&alock);
         }
-	return OMPI_ERR_BAD_PARAM;
+        return OMPI_ERR_BAD_PARAM;
     }
 
-    /* Do we need to make a new keyhash? */
+    /* Do we need to make a new attr_hash? */
 
-    if (NULL == *keyhash) {
-        ompi_attr_hash_init(keyhash);
+    if (NULL == *attr_hash) {
+        ompi_attr_hash_init(attr_hash);
     }
 
-    /* Now see if the key is present in the object's key hash. If so,
-       delete the old attribute value. */
+    /* Now see if an attribute is already present in the object's hash
+       on the old keyval. If so, delete the old attribute value. */
 
-    ret = opal_hash_table_get_value_uint32(*keyhash, key, (void**) &old_attr);
+    ret = opal_hash_table_get_value_uint32(*attr_hash, key, (void**) &old_attr);
     if (OMPI_SUCCESS == ret)  {
-	switch (type) {
-	case COMM_ATTR:
-	    DELETE_ATTR_CALLBACKS(communicator, old_attr, key_item, object);
-	    break;
+        switch (type) {
+        case COMM_ATTR:
+            DELETE_ATTR_CALLBACKS(communicator, old_attr, keyval, object);
+            break;
 
-	case WIN_ATTR:
-	    DELETE_ATTR_CALLBACKS(win, old_attr, key_item, object);
-	    break;
+        case WIN_ATTR:
+            DELETE_ATTR_CALLBACKS(win, old_attr, keyval, object);
+            break;
 
-	case TYPE_ATTR:
-	    DELETE_ATTR_CALLBACKS(datatype, old_attr, key_item, object);
-	    break;
+        case TYPE_ATTR:
+            DELETE_ATTR_CALLBACKS(datatype, old_attr, keyval, object);
+            break;
 
-	default:
+        default:
+            if (need_lock) {
+                OPAL_THREAD_UNLOCK(&alock);
+            }
             return MPI_ERR_INTERN;
-	}
-	had_old = true;
+        }
+        had_old = true;
         OBJ_RELEASE(old_attr);
     }
 
-    ret = opal_hash_table_set_value_uint32(*keyhash, key, new_attr);
+    ret = opal_hash_table_set_value_uint32(*attr_hash, key, new_attr);
+
+    /* Increase the reference count of the object, only if there was no
+       old atribute/no old entry in the object's key hash */
+
+    if (OMPI_SUCCESS == ret && !had_old) {
+        OBJ_RETAIN(keyval);
+    }
+
+    /* Release the lock if we grabbed it */
 
     if (need_lock) {
         OPAL_THREAD_UNLOCK(&alock);
     }
     if (OMPI_SUCCESS != ret) {
-	return ret;
+        return ret;
     }
 
-    /* Increase the reference count of the object, only if there was no
-       old atribute/no old entry in the object's key hash */
-
-    if (!had_old) {
-	OBJ_RETAIN(key_item);
-    }
     return MPI_SUCCESS;
 }
 
@@ -1071,41 +1109,42 @@ static int set_value(ompi_attribute_type_t type, void *object,
  * in small, standalone functions that are called from several
  * different places.
  */
-static int get_value(opal_hash_table_t *keyhash, int key, 
+static int get_value(opal_hash_table_t *attr_hash, int key, 
                      attribute_value_t **attribute, int *flag)
 {
     int ret;
     void *attr;
-    ompi_attrkey_item_t *key_item;
+    ompi_attribute_keyval_t *keyval;
 
-    /* According to MPI specs, the call is invalid if key is not
-       present in the main keyval hash at all.  If no attribute is
-       associated with the key, then the call is valid and returns
-       FALSE in the flag argument */
+    /* According to MPI specs, the call is invalid if the keyval does
+       not exist (i.e., the key is not present in the main keyval
+       hash).  If the keyval exists but no attribute is associated
+       with the key, then the call is valid and returns FALSE in the
+       flag argument */
 
     *flag = 0;
     OPAL_THREAD_LOCK(&alock);
     ret = opal_hash_table_get_value_uint32(keyval_hash, key, 
-					   (void**) &key_item);
+                                           (void**) &keyval);
 
     if (OMPI_ERR_NOT_FOUND == ret) {
         OPAL_THREAD_UNLOCK(&alock);
-	return MPI_KEYVAL_INVALID;
+        return MPI_KEYVAL_INVALID;
     }
 
-    /* If we have a null keyhash table, that means that nothing has
+    /* If we have a null attr_hash table, that means that nothing has
        been cached on this object yet.  So just return *flag = 0. */
 
-    if (NULL == keyhash) {
+    if (NULL == attr_hash) {
         OPAL_THREAD_UNLOCK(&alock);
         return OMPI_SUCCESS;
     }
 
-    ret = opal_hash_table_get_value_uint32(keyhash, key, &attr);
+    ret = opal_hash_table_get_value_uint32(attr_hash, key, &attr);
     OPAL_THREAD_UNLOCK(&alock);
     if (OMPI_SUCCESS == ret) {
-        *attribute = attr;
-	*flag = 1;
+        *attribute = (attribute_value_t*)attr;
+        *flag = 1;
     }
     return OMPI_SUCCESS;
 }
