@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -117,10 +117,6 @@ int ompi_comm_set ( ompi_communicator_t *newcomm,
     newcomm->error_handler = errh;
     OBJ_RETAIN ( newcomm->error_handler );
 
-    /* Set name for debugging purposes */
-    snprintf(newcomm->c_name, MPI_MAX_OBJECT_NAME, "MPI COMMUNICATOR %d", 
-             newcomm->c_contextid);
-
     /* Set Topology, if required */
     
     if ( NULL != topocomponent ) {
@@ -315,6 +311,10 @@ int ompi_comm_create ( ompi_communicator_t *comm, ompi_group_t *group,
         goto exit;
     }
 
+    /* Set name for debugging purposes */
+    snprintf(newcomp->c_name, MPI_MAX_OBJECT_NAME, "MPI COMMUNICATOR %d CREATE FROM %d", 
+             newcomp->c_contextid, comm->c_contextid );
+
     /* Activate the communicator and init coll-component */
     rc = ompi_comm_activate ( newcomp,  /* new communicator */ 
                               comm,     /* old comm */
@@ -536,6 +536,10 @@ int ompi_comm_split ( ompi_communicator_t* comm, int color, int key,
         goto exit;
     }
 
+    /* Set name for debugging purposes */
+    snprintf(newcomp->c_name, MPI_MAX_OBJECT_NAME, "MPI COMMUNICATOR %d SPLIT FROM %d", 
+             newcomp->c_contextid, comm->c_contextid );
+
     /* Activate the communicator and init coll-component */
     rc = ompi_comm_activate ( newcomp,  /* new communicator */ 
                              comm,     /* old comm */
@@ -635,6 +639,10 @@ int ompi_comm_dup ( ompi_communicator_t * comm, ompi_communicator_t **newcomm)
         return rc;
     }
 
+    /* Set name for debugging purposes */
+    snprintf(newcomp->c_name, MPI_MAX_OBJECT_NAME, "MPI COMMUNICATOR %d DUP FROM %d", 
+             newcomp->c_contextid, comm->c_contextid );
+
     /* activate communicator and init coll-module */
     rc = ompi_comm_activate (newcomp,  /* new communicator */ 
                              comp,     /* old comm */
@@ -830,8 +838,7 @@ ompi_proc_t **ompi_comm_get_rprocs ( ompi_communicator_t *local_comm,
     int rc;
     int local_rank, local_size;
     ompi_proc_t **rprocs=NULL;
-    char *rnamebuf=NULL;
-    size_t size_len;
+    orte_std_cntr_t size_len;
     int int_len, rlen;
     orte_buffer_t *sbuf=NULL, *rbuf=NULL;
     void *sendbuf;
@@ -847,8 +854,8 @@ ompi_proc_t **ompi_comm_get_rprocs ( ompi_communicator_t *local_comm,
             goto err_exit;
         }
 
-        rc = ompi_proc_get_namebuf (local_comm->c_local_group->grp_proc_pointers, 
-				    local_size, sbuf);
+        rc = ompi_proc_pack(local_comm->c_local_group->grp_proc_pointers, 
+                            local_size, sbuf);
         if ( OMPI_SUCCESS != rc ) {
             goto err_exit;
         }
@@ -862,9 +869,7 @@ ompi_proc_t **ompi_comm_get_rprocs ( ompi_communicator_t *local_comm,
         if ( OMPI_SUCCESS != rc ) {
             goto err_exit;
         }
-        if (OMPI_SUCCESS != (rc = opal_size2int(size_len, &int_len, true))) {
-            goto err_exit;
-        }
+        int_len = (int)size_len;
         
         rc = MCA_PML_CALL(send (&int_len, 1, MPI_INT, remote_leader, tag, 
                                 MCA_PML_BASE_SEND_STANDARD, bridge_comm ));
@@ -890,16 +895,6 @@ ompi_proc_t **ompi_comm_get_rprocs ( ompi_communicator_t *local_comm,
     if ( NULL == recvbuf ) {
 	goto err_exit;
     }
-    
-    rbuf = OBJ_NEW(orte_buffer_t);
-    if (NULL == rbuf) {
-        rc = ORTE_ERROR;
-        goto err_exit;
-    }
-    
-    if (ORTE_SUCCESS != (rc = orte_dss.load(rbuf, recvbuf, rlen))) {
-        goto err_exit;
-    }
 
     if ( local_rank == local_leader ) {
         /* local leader exchange name lists */
@@ -927,15 +922,22 @@ ompi_proc_t **ompi_comm_get_rprocs ( ompi_communicator_t *local_comm,
     if ( OMPI_SUCCESS != rc ) {
         goto err_exit;
     }
+
+    rbuf = OBJ_NEW(orte_buffer_t);
+    if (NULL == rbuf) {
+        rc = ORTE_ERROR;
+        goto err_exit;
+    }
+    
+    if (ORTE_SUCCESS != (rc = orte_dss.load(rbuf, recvbuf, rlen))) {
+        goto err_exit;
+    }
     
     /* decode the names into a proc-list */
-    rc = ompi_proc_get_proclist (rbuf, rsize, &rprocs );
+    rc = ompi_proc_unpack(rbuf, rsize, &rprocs );
     OBJ_RELEASE(rbuf);
     
  err_exit:
-    if ( NULL != rnamebuf) {
-        free ( rnamebuf );
-    }
     /* rprocs isn't freed unless we have an error, 
        since it is used in the communicator */
     if ( OMPI_SUCCESS !=rc ) {
@@ -1034,7 +1036,7 @@ int ompi_comm_determine_first ( ompi_communicator_t *intercomm, int high )
         theirproc = intercomm->c_remote_group->grp_proc_pointers[0];
 
         mask = ORTE_NS_CMP_CELLID | ORTE_NS_CMP_JOBID | ORTE_NS_CMP_VPID;
-        rc = orte_ns.compare (mask, &(ourproc->proc_name), &(theirproc->proc_name));
+        rc = orte_ns.compare_fields(mask, &(ourproc->proc_name), &(theirproc->proc_name));
         if ( 0 > rc ) {
             flag = true;
         }
@@ -1147,7 +1149,7 @@ int ompi_topo_create (ompi_communicator_t *old_comm,
 
     /* allocate the data for the common good */
 
-    new_comm->c_topo_comm = malloc(sizeof(mca_topo_base_comm_t));
+    new_comm->c_topo_comm = (mca_topo_base_comm_t*)malloc(sizeof(mca_topo_base_comm_t));
     if (NULL == new_comm->c_topo_comm) {
         OBJ_RELEASE(new_comm);
         return OMPI_ERR_OUT_OF_RESOURCE;
@@ -1344,10 +1346,19 @@ static int ompi_comm_fill_rest (ompi_communicator_t *comm,
                                 ompi_errhandler_t *errh )
 {
     int ret;
+    
+    /* properly decrement the ref counts on the groups.
+       We are doing this because this function is sort of a redo 
+       of what is done in comm.c.. No need to decrement the ref 
+       count on the proc pointers 
+       This is just a quick fix, and will be looking for a 
+       better solution */
+    OBJ_RELEASE ( comm->c_local_group );
+    OBJ_RELEASE ( comm->c_local_group );
 
     /* allocate a group structure for the new communicator */
     comm->c_local_group = ompi_group_allocate(num_procs);
-
+    
     /* free the malloced  proc pointers */
     free(comm->c_local_group->grp_proc_pointers);
     
@@ -1356,6 +1367,7 @@ static int ompi_comm_fill_rest (ompi_communicator_t *comm,
 
     /* set the remote group to be the same as local group */
     comm->c_remote_group = comm->c_local_group;
+    OBJ_RETAIN ( comm->c_remote_group );
 
     /* retain these proc pointers */
     ompi_group_increment_proc_count(comm->c_local_group);
