@@ -2,13 +2,17 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006      University of Houston. All rights reserved.
+ * Copyright (c) 2006-2007 Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
+ *
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -25,7 +29,11 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif  /* HAVE_SYS_TIME_H */
 
+#include "opal/util/opal_environ.h"
 #include "opal/util/printf.h"
 #include "opal/util/convert.h"
 #include "opal/threads/mutex.h"
@@ -45,28 +53,34 @@
 #include "orte/mca/ns/ns.h"
 #include "orte/mca/gpr/gpr.h"
 #include "orte/mca/errmgr/errmgr.h"
+#include "orte/mca/ras/ras_types.h"
+#include "orte/mca/rmaps/rmaps_types.h"
 #include "orte/mca/rmgr/rmgr.h"
-#include "orte/mca/soh/soh_types.h"
+#include "orte/mca/rmgr/base/base.h"
+#include "orte/mca/smr/smr_types.h"
 #include "orte/mca/rml/rml.h"
 
 #include "orte/runtime/runtime.h"
 
-extern char **environ;
+static int ompi_comm_get_rport (orte_process_name_t *port,
+                                int send_first, struct ompi_proc_t *proc,
+                                orte_rml_tag_t tag, orte_process_name_t *rport);
+
 
 int ompi_comm_connect_accept ( ompi_communicator_t *comm, int root,
                                orte_process_name_t *port, int send_first,
                                ompi_communicator_t **newcomm, orte_rml_tag_t tag )
 {
     int size, rsize, rank, rc;
-    size_t num_vals;
-    size_t rnamebuflen = 0;
+    orte_std_cntr_t num_vals;
+    orte_std_cntr_t rnamebuflen = 0;
     int rnamebuflen_int = 0;
     void *rnamebuf=NULL;
 
     ompi_communicator_t *newcomp=MPI_COMM_NULL;
     ompi_proc_t **rprocs=NULL;
     ompi_group_t *group=comm->c_local_group;
-    orte_process_name_t *rport=NULL;
+    orte_process_name_t *rport=NULL, tmp_port_name;
     orte_buffer_t *nbuf=NULL, *nrbuf=NULL;
 
     size = ompi_comm_size ( comm );
@@ -82,8 +96,11 @@ int ompi_comm_connect_accept ( ompi_communicator_t *comm, int root,
            exchange that.
         */
         if ( OMPI_COMM_JOIN_TAG != (int)tag ) {
-            rport = ompi_comm_get_rport( port,send_first,
-                                         group->grp_proc_pointers[rank], tag);
+            rc = ompi_comm_get_rport(port,send_first,
+                                     group->grp_proc_pointers[rank], tag,
+                                     &tmp_port_name);
+            if (OMPI_SUCCESS != rc) return rc;
+            rport = &tmp_port_name;
         } else {
             rport = port;
         }
@@ -96,9 +113,10 @@ int ompi_comm_connect_accept ( ompi_communicator_t *comm, int root,
         }
 
         if (ORTE_SUCCESS != (rc = orte_dss.pack(nbuf, &size, 1, ORTE_INT))) {
+            ORTE_ERROR_LOG(rc);
             goto exit;
         }
-        ompi_proc_get_namebuf (group->grp_proc_pointers, size, nbuf);
+        ompi_proc_pack(group->grp_proc_pointers, size, nbuf);
 
         nrbuf = OBJ_NEW(orte_buffer_t);
         if (NULL == nrbuf ) {
@@ -116,22 +134,24 @@ int ompi_comm_connect_accept ( ompi_communicator_t *comm, int root,
         }
 
         if (ORTE_SUCCESS != (rc = orte_dss.unload(nrbuf, &rnamebuf, &rnamebuflen))) {
+            ORTE_ERROR_LOG(rc);
             goto exit;
         }
     }
 
     /* First convert the size_t to an int so we can cast in the bcast to a void *
-     * if we don't then we will get badness when using big vs little endian */
-    if (OMPI_SUCCESS != (rc = opal_size2int(rnamebuflen, &rnamebuflen_int, true))) {
-        goto exit;
-    }
+     * if we don't then we will get badness when using big vs little endian
+     * THIS IS NO LONGER REQUIRED AS THE LENGTH IS NOW A STD_CNTR_T, WHICH
+     * CORRELATES TO AN INT32
+     */
+    rnamebuflen_int = (int)rnamebuflen;
 
     /* bcast the buffer-length to all processes in the local comm */
     rc = comm->c_coll.coll_bcast (&rnamebuflen_int, 1, MPI_INT, root, comm );
     if ( OMPI_SUCCESS != rc ) {
         goto exit;
     }
-    rnamebuflen = (size_t)rnamebuflen_int;
+    rnamebuflen = rnamebuflen_int;
 
     if ( rank != root ) {
         /* non root processes need to allocate the buffer manually */
@@ -157,15 +177,17 @@ int ompi_comm_connect_accept ( ompi_communicator_t *comm, int root,
         goto exit;
     }
     if ( ORTE_SUCCESS != ( rc = orte_dss.load(nrbuf, rnamebuf, rnamebuflen))) {
+        ORTE_ERROR_LOG(rc);
         goto exit;
     }
 
     num_vals = 1;
     if (ORTE_SUCCESS != (rc = orte_dss.unpack(nrbuf, &rsize, &num_vals, ORTE_INT))) {
+        ORTE_ERROR_LOG(rc);
         goto exit;
     }
 
-    rc = ompi_proc_get_proclist (nrbuf, rsize, &rprocs);
+    rc = ompi_proc_unpack(nrbuf, rsize, &rprocs);
     if ( OMPI_SUCCESS != rc ) {
         goto exit;
     }
@@ -256,56 +278,62 @@ int ompi_comm_connect_accept ( ompi_communicator_t *comm, int root,
  * accept get their own port_name = OOB contact information passed in as
  * an argument. This is however useless.
  *
- * Therefore, the two root processes exchange this information at this point.
+ * Therefore, the two root processes exchange this information at this
+ * point.
  *
  */
-orte_process_name_t *ompi_comm_get_rport (orte_process_name_t *port, int send_first,
-                                          ompi_proc_t *proc, orte_rml_tag_t tag)
+int ompi_comm_get_rport(orte_process_name_t *port, int send_first,
+                        ompi_proc_t *proc, orte_rml_tag_t tag, 
+                        orte_process_name_t *rport_name)
 {
     int rc;
-    size_t num_vals;
-    orte_process_name_t *rport, tbuf;
-    ompi_proc_t *rproc=NULL;
-    bool isnew = false;
+    orte_std_cntr_t num_vals;
 
     if ( send_first ) {
         orte_buffer_t *sbuf;
 
-        rproc = ompi_proc_find_and_add(port, &isnew);
         sbuf = OBJ_NEW(orte_buffer_t);
         if (NULL == sbuf) {
-            return NULL;
+            return OMPI_ERROR;
         }
-        if (ORTE_SUCCESS != orte_dss.pack(sbuf, &(proc->proc_name), 1, ORTE_NAME)) {
-            return NULL;
+        if (ORTE_SUCCESS != (rc = orte_dss.pack(sbuf, &(proc->proc_name), 1, ORTE_NAME))) {
+            ORTE_ERROR_LOG(rc);
+            OBJ_RELEASE(sbuf);
+            return rc;
         }
+
         rc = orte_rml.send_buffer(port, sbuf, tag, 0);
         OBJ_RELEASE(sbuf);
-
-        rport = port;
+	if ( 0 > rc ) {
+	    ORTE_ERROR_LOG(rc);
+	    return rc;
+	}
+	    
+        *rport_name = *port;
     }
     else {
         orte_buffer_t *rbuf;
 
         rbuf = OBJ_NEW(orte_buffer_t);
         if (NULL == rbuf) {
-            return NULL;
+            return ORTE_ERROR;
         }
-        rc = orte_rml.recv_buffer(ORTE_RML_NAME_ANY, rbuf, tag);
+        if (ORTE_SUCCESS != (rc = orte_rml.recv_buffer(ORTE_NAME_WILDCARD, rbuf, tag))) {
+            ORTE_ERROR_LOG(rc);
+            OBJ_RELEASE(rbuf);
+            return rc;
+        }
+
         num_vals = 1;
-        if (ORTE_SUCCESS != orte_dss.unpack(rbuf, &tbuf, &num_vals, ORTE_NAME)) {
-            return NULL;
+        if (ORTE_SUCCESS != (rc = orte_dss.unpack(rbuf, rport_name, &num_vals, ORTE_NAME))) {
+            ORTE_ERROR_LOG(rc);
+            OBJ_RELEASE(rbuf);
+            return rc;
         }
         OBJ_RELEASE(rbuf);
-        rproc = ompi_proc_find_and_add(&tbuf, &isnew);
-        rport = &(rproc->proc_name);
-
-    }
-    if (isnew) {
-        MCA_PML_CALL(add_procs(&rproc, 1));
     }
 
-    return rport;
+    return OMPI_SUCCESS;
 }
 
 
@@ -321,16 +349,29 @@ ompi_comm_start_processes(int count, char **array_of_commands,
 {
     int rc, i, j, counter;
     int have_wdir=0;
+    bool have_prefix;
     int valuelen=OMPI_PATH_MAX, flag=0;
     char cwd[OMPI_PATH_MAX];
+    char host[OMPI_PATH_MAX];  /*** should define OMPI_HOST_MAX ***/
+    char prefix[OMPI_PATH_MAX];
+    char *base_prefix;
 
-    orte_jobid_t new_jobid;
+    orte_std_cntr_t num_apps, ai;
+    orte_jobid_t new_jobid=ORTE_JOBID_INVALID;
     orte_app_context_t **apps=NULL;
+    
+    opal_list_t attributes;
+    opal_list_item_t *item;
 
-
+    bool timing = false;
+    struct timeval ompistart, ompistop;
+    int param, value;
+    
     /* parse the info object */
     /* check potentially for:
        - "host": desired host where to spawn the processes
+       - "prefix": the path to the root of the directory tree where ompi
+                   executables and libraries can be found
        - "arch": desired architecture
        - "wdir": directory, where executable can be found
        - "path": list of directories where to look for the executable
@@ -340,6 +381,49 @@ ompi_comm_start_processes(int count, char **array_of_commands,
 
     /* make sure the progress engine properly trips the event library */
     opal_progress_event_increment();
+    
+    /* check to see if we want timing information */
+    param = mca_base_param_reg_int_name("ompi", "timing",
+                                        "Request that critical timing loops be measured",
+                                        false, false, 0, &value);
+    if (value != 0) {
+        timing = true;
+        if (0 != gettimeofday(&ompistart, NULL)) {
+            opal_output(0, "ompi_comm_start_procs: could not obtain start time");
+            ompistart.tv_sec = 0;
+            ompistart.tv_usec = 0;
+        }
+    }
+    
+    /* setup to record the attributes */
+    OBJ_CONSTRUCT(&attributes, opal_list_t);
+    
+    /* we want to be able to default the prefix to the one used for this job
+     * so that the ompi executables and libraries can be found. the user can
+     * later override this value by providing an MPI_Info value. for now, though,
+     * let's get the default value off the registry
+     */
+    if (ORTE_SUCCESS != (rc = orte_rmgr.get_app_context(orte_process_info.my_name->jobid, &apps, &num_apps))) {
+        ORTE_ERROR_LOG(rc);
+        return rc;
+    }
+    /* we'll just use the prefix from the first member of the app_context array.
+     * this shouldn't matter as they all should be the same. it could be NULL, of
+     * course (user might not have specified it), so we need to protect against that.
+     *
+     * It's possible that no app_contexts are returned (e.g., during a comm_spawn
+     * from a singleton), so check first
+     */
+    if (NULL != apps && NULL != apps[0]->prefix_dir) {
+        base_prefix = strdup(apps[0]->prefix_dir);
+    } else {
+        base_prefix = NULL;
+    }
+    /* cleanup the memory we used */
+    for (ai = 0; ai < num_apps; ai++) {
+        OBJ_RELEASE(apps[ai]);
+    }
+    if (NULL != apps) free(apps);
 
     /* Convert the list of commands to an array of orte_app_context_t
        pointers */
@@ -370,7 +454,7 @@ ompi_comm_start_processes(int count, char **array_of_commands,
         apps[i]->num_procs = array_of_maxprocs[i];
 
         /* copy over the argv array */
-    counter = 1;
+        counter = 1;
 
         if (MPI_ARGVS_NULL != array_of_argv &&
             MPI_ARGV_NULL != array_of_argv[i]) {
@@ -380,24 +464,24 @@ ompi_comm_start_processes(int count, char **array_of_commands,
                 j++;
             }
             counter += j;
-    }
-
-    /* now copy them over, ensuring to NULL terminate the array */
-    apps[i]->argv = (char**)malloc((1 + counter) * sizeof(char*));
-    if (NULL == apps[i]->argv) {
-        ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
-        /* rollback what was already done */
-        for (j=0; j < i; j++) {
-        OBJ_RELEASE(apps[j]);
         }
+
+        /* now copy them over, ensuring to NULL terminate the array */
+        apps[i]->argv = (char**)malloc((1 + counter) * sizeof(char*));
+        if (NULL == apps[i]->argv) {
+            ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
+            /* rollback what was already done */
+            for (j=0; j < i; j++) {
+                OBJ_RELEASE(apps[j]);
+            }
             opal_progress_event_decrement();
-        return ORTE_ERR_OUT_OF_RESOURCE;
-    }
-    apps[i]->argv[0] = strdup(array_of_commands[i]);
-    for (j=1; j < counter; j++) {
-        apps[i]->argv[j] = strdup(array_of_argv[i][j-1]);
-    }
-    apps[i]->argv[counter] = NULL;
+            return ORTE_ERR_OUT_OF_RESOURCE;
+        }
+        apps[i]->argv[0] = strdup(array_of_commands[i]);
+        for (j=1; j < counter; j++) {
+            apps[i]->argv[j] = strdup(array_of_argv[i][j-1]);
+        }
+        apps[i]->argv[counter] = NULL;
 
 
         /* the environment gets set by the launcher
@@ -406,7 +490,7 @@ ompi_comm_start_processes(int count, char **array_of_commands,
          */
         /* Add environment variable with the contact information for the
            child processes.
-    */
+        */
         counter = 1;
         apps[i]->env = (char**)malloc((1+counter) * sizeof(char*));
         if (NULL == apps[i]->env) {
@@ -424,15 +508,38 @@ ompi_comm_start_processes(int count, char **array_of_commands,
             }
         }
 
-        /* Check for the 'wdir' and later potentially for the
-           'path' Info object */
+        /* Check for well-known info keys */
         have_wdir = 0;
+        have_prefix = false;
         if ( array_of_info != NULL && array_of_info[i] != MPI_INFO_NULL ) {
+
+            /* check for 'wdir' */ 
             ompi_info_get (array_of_info[i], "wdir", valuelen, cwd, &flag);
             if ( flag ) {
-                apps[i]->cwd = cwd;
+                apps[i]->cwd = strdup(cwd);
                 have_wdir = 1;
             }
+
+            /* check for 'host' */
+            ompi_info_get (array_of_info[i], "host", sizeof(host), host, &flag);
+            if ( flag ) {
+                apps[i]->num_map = 1;
+                apps[i]->map_data = (orte_app_context_map_t **) malloc(sizeof(orte_app_context_map_t *));
+                apps[i]->map_data[0] = OBJ_NEW(orte_app_context_map_t);
+                apps[i]->map_data[0]->map_type = ORTE_APP_CONTEXT_MAP_HOSTNAME;
+                apps[i]->map_data[0]->map_data = strdup(host);
+            }
+ 
+            /* 'path', 'arch', 'file', 'soft' -- to be implemented */ 
+            
+            /* check for 'ompi_prefix' (OMPI-specific -- to effect the same
+             * behavior as --prefix option to orterun)
+             */
+            ompi_info_get (array_of_info[i], "ompi_prefix", sizeof(prefix), prefix, &flag);
+            if ( flag ) {
+                apps[i]->prefix_dir = strdup(prefix);
+                have_prefix = true;
+            }
         }
 
         /* default value: If the user did not tell us where to look for the
@@ -441,21 +548,84 @@ ompi_comm_start_processes(int count, char **array_of_commands,
             getcwd(cwd, OMPI_PATH_MAX);
             apps[i]->cwd = strdup(cwd);
         }
+        
+        /* if the user told us a new prefix, then we leave it alone. otherwise, if
+         * a prefix had been provided before, copy that one into the new app_context
+         * for use by the spawned children
+         */
+        if ( !have_prefix && NULL != base_prefix) {
+            apps[i]->prefix_dir = strdup(base_prefix);
+        }
+        
         /* leave the map info alone - the launcher will
          * decide where to put things
          */
     } /* for (i = 0 ; i < count ; ++i) */
 
+    /* cleanup */
+    if (NULL != base_prefix) free(base_prefix);
 
+    /* tell the RTE that we want to the children to run inside of our allocation -
+     * don't go get one just for them
+     */
+    if (ORTE_SUCCESS != (rc = orte_rmgr.add_attribute(&attributes, ORTE_RAS_USE_PARENT_ALLOCATION,
+                                                      ORTE_JOBID, &(orte_process_info.my_name->jobid),
+                                                      ORTE_RMGR_ATTR_OVERRIDE))) {
+        ORTE_ERROR_LOG(rc);
+        OBJ_DESTRUCT(&attributes);
+        opal_progress_event_decrement();
+        return MPI_ERR_SPAWN;
+    }
+
+    /* tell the RTE that we want the children mapped the same way as their parent */
+    if (ORTE_SUCCESS != (rc = orte_rmgr.add_attribute(&attributes, ORTE_RMAPS_USE_PARENT_PLAN,
+                                                      ORTE_JOBID, &(orte_process_info.my_name->jobid),
+                                                      ORTE_RMGR_ATTR_OVERRIDE))) {
+        ORTE_ERROR_LOG(rc);
+        OBJ_DESTRUCT(&attributes);
+        opal_progress_event_decrement();
+        return MPI_ERR_SPAWN;
+    }
+
+    /* check for timing request - get stop time and report elapsed time if so */
+    if (timing) {
+        if (0 != gettimeofday(&ompistop, NULL)) {
+            opal_output(0, "ompi_comm_start_procs: could not obtain stop time");
+        } else {
+            opal_output(0, "ompi_comm_start_procs: time from start to prepare to spawn %ld usec",
+                        (long int)((ompistop.tv_sec - ompistart.tv_sec)*1000000 +
+                                   (ompistop.tv_usec - ompistart.tv_usec)));
+            if (0 != gettimeofday(&ompistart, NULL)) {
+                opal_output(0, "ompi_comm_start_procs: could not obtain new start time");
+                ompistart.tv_sec = ompistop.tv_sec;
+                ompistart.tv_usec = ompistop.tv_usec;
+            }
+        }
+    }
+    
     /* spawn procs */
-    if (ORTE_SUCCESS != (rc = orte_rmgr.spawn(apps, count, &new_jobid, NULL, ORTE_PROC_STATE_NONE))) {
+    if (ORTE_SUCCESS != (rc = orte_rmgr.spawn_job(apps, count, &new_jobid, 0, NULL, NULL, ORTE_PROC_STATE_NONE, &attributes))) {
         ORTE_ERROR_LOG(rc);
         opal_progress_event_decrement();
         return MPI_ERR_SPAWN;
     }
 
+    /* check for timing request - get stop time and report elapsed time if so */
+    if (timing) {
+        if (0 != gettimeofday(&ompistop, NULL)) {
+            opal_output(0, "ompi_comm_start_procs: could not obtain stop time");
+        } else {
+            opal_output(0, "ompi_comm_start_procs: time to spawn %ld usec",
+                        (long int)((ompistop.tv_sec - ompistart.tv_sec)*1000000 +
+                                   (ompistop.tv_usec - ompistart.tv_usec)));
+         }
+    }
+    
     /* clean up */
     opal_progress_event_decrement();
+    while (NULL != (item = opal_list_remove_first(&attributes))) OBJ_RELEASE(item);
+    OBJ_DESTRUCT(&attributes);
+
     for ( i=0; i<count; i++) {
     OBJ_RELEASE(apps[i]);
     }
@@ -491,8 +661,9 @@ int ompi_comm_dyn_init (void)
        its pieces, which are : port_name and tag */
     oob_port = ompi_parse_port (port_name, &tag);
     if (ORTE_SUCCESS != (rc = orte_ns.convert_string_to_process_name(&port_proc_name, oob_port))) {
-          return rc;
-        }
+        ORTE_ERROR_LOG(rc);
+        return rc;
+    }
 
     rc = ompi_comm_connect_accept (MPI_COMM_WORLD, root, port_proc_name,
                   send_first, &newcomm, tag );
@@ -704,38 +875,34 @@ void ompi_comm_mark_dyncomm (ompi_communicator_t *comm)
        of different jobids.  */
     grp = comm->c_local_group;
     for (i=0; i< size; i++) {
-    if (ORTE_SUCCESS != orte_ns.get_jobid(&thisjobid, &(grp->grp_proc_pointers[i]->proc_name))) {
-        return;
-    }
-    found = 0;
-    for ( j=0; j<numjobids; j++) {
-        if ( thisjobid == jobids[j]) {
-        found = 1;
-        break;
+        thisjobid = grp->grp_proc_pointers[i]->proc_name.jobid;
+        found = 0;
+        for ( j=0; j<numjobids; j++) {
+            if ( thisjobid == jobids[j]) {
+                found = 1;
+                break;
+            }
+        }
+        if (!found ) {
+            jobids[numjobids++] = thisjobid;
         }
-    }
-    if (!found ) {
-        jobids[numjobids++] = thisjobid;
-    }
     }
 
     /* if inter-comm, loop over all processes in remote_group
        and count number of different jobids */
     grp = comm->c_remote_group;
     for (i=0; i< rsize; i++) {
-    if (ORTE_SUCCESS != orte_ns.get_jobid(&thisjobid, &(grp->grp_proc_pointers[i]->proc_name))) {
-        return;
-    }
-    found = 0;
-    for ( j=0; j<numjobids; j++) {
-        if ( thisjobid == jobids[j]) {
-        found = 1;
-        break;
+        thisjobid = grp->grp_proc_pointers[i]->proc_name.jobid;
+        found = 0;
+        for ( j=0; j<numjobids; j++) {
+            if ( thisjobid == jobids[j]) {
+                found = 1;
+                break;
+            }
+        }
+        if (!found ) {
+            jobids[numjobids++] = thisjobid;
         }
-    }
-    if (!found ) {
-        jobids[numjobids++] = thisjobid;
-    }
     }
 
     /* if number of joibds larger than one, set the disconnect flag*/
