@@ -53,16 +53,14 @@ int32_t
 ompi_pack_homogeneous_contig_function( ompi_convertor_t* pConv,
                                        struct iovec* iov,
                                        uint32_t* out_size,
-                                       size_t* max_data,
-                                       int* freeAfter )
+                                       size_t* max_data )
 {
     dt_stack_t* pStack = pConv->pStack;
     char *source_base = NULL;
-    size_t length = pConv->local_size - pConv->bConverted;
-    uint32_t iov_count, initial_amount = pConv->bConverted;
-    long initial_displ = pConv->use_desc->desc[pConv->use_desc->used].end_loop.first_elem_disp;
+    uint32_t iov_count;
+    size_t length = pConv->local_size - pConv->bConverted, initial_amount = pConv->bConverted;
+    ptrdiff_t initial_displ = pConv->use_desc->desc[pConv->use_desc->used].end_loop.first_elem_disp;
 
-    *freeAfter = 0;
     source_base = (pConv->pBaseBuf + initial_displ + pStack[0].disp + pStack[1].disp);
 
     /* There are some optimizations that can be done if the upper level
@@ -73,7 +71,7 @@ ompi_pack_homogeneous_contig_function( ompi_convertor_t* pConv,
         if( (size_t)iov[iov_count].iov_len > length )
             iov[iov_count].iov_len = length;
         if( iov[iov_count].iov_base == NULL ) {
-            iov[iov_count].iov_base = source_base;
+            iov[iov_count].iov_base = (IOVBASE_TYPE *) source_base;
             COMPUTE_CSUM( iov[iov_count].iov_base, iov[iov_count].iov_len, pConv );
         } else {
             /* contiguous data just memcpy the smallest data in the user buffer */
@@ -101,28 +99,26 @@ int32_t
 ompi_pack_homogeneous_contig_with_gaps_function( ompi_convertor_t* pConv,
                                                  struct iovec* iov,
                                                  uint32_t* out_size,
-                                                 size_t* max_data,
-                                                 int* freeAfter )
+                                                 size_t* max_data )
 {
     const ompi_datatype_t* pData = pConv->pDesc;
     dt_stack_t* pStack = pConv->pStack;
     char *user_memory, *packed_buffer;
-    long extent;
-    uint32_t max_allowed, i, index;
-    uint32_t iov_count, total_bytes_converted = 0;
-    long initial_displ = pConv->use_desc->desc[pConv->use_desc->used].end_loop.first_elem_disp;
+    uint32_t i, index, iov_count;
+    size_t max_allowed, total_bytes_converted = 0;
+    ptrdiff_t extent;
+    ptrdiff_t initial_displ = pConv->use_desc->desc[pConv->use_desc->used].end_loop.first_elem_disp;
 
     extent = pData->ub - pData->lb;
-    assert( (pData->flags & DT_FLAG_CONTIGUOUS) && ((long)pData->size != extent) );
+    assert( (pData->flags & DT_FLAG_CONTIGUOUS) && ((ptrdiff_t)pData->size != extent) );
 
     /* Limit the amount of packed data to the data left over on this convertor */
     max_allowed = pConv->local_size - pConv->bConverted;
     if( max_allowed > (*max_data) )
         max_allowed = (*max_data);
 
-    i = pConv->bConverted / pData->size;  /* how many we already pack */
+    i = (uint32_t)(pConv->bConverted / pData->size);  /* how many we already pack */
 
-    *freeAfter = 0;
     /* There are some optimizations that can be done if the upper level
      * does not provide a buffer.
      */
@@ -137,7 +133,7 @@ ompi_pack_homogeneous_contig_with_gaps_function( ompi_convertor_t* pConv,
             if( (uint32_t)pStack->count < ((*out_size) - iov_count) ) {
                 pStack[1].count = pData->size - (pConv->bConverted % pData->size);
                 for( index = iov_count; i < pConv->count; i++, index++ ) {
-                    iov[index].iov_base = user_memory;
+                    iov[index].iov_base = (IOVBASE_TYPE *) user_memory;
                     iov[index].iov_len = pStack[1].count;
                     pStack[0].disp += extent;
                     total_bytes_converted += pStack[1].count;
@@ -160,16 +156,16 @@ ompi_pack_homogeneous_contig_with_gaps_function( ompi_convertor_t* pConv,
                 for( index = iov_count; (i < pConv->count) && (index < (*out_size));
                      i++, index++ ) {
                     if( max_allowed < pData->size ) {
-                        iov[index].iov_base = user_memory;
+                        iov[index].iov_base = (IOVBASE_TYPE *) user_memory;
                         iov[index].iov_len = max_allowed;
                         max_allowed = 0;
                         COMPUTE_CSUM( iov[index].iov_base, iov[index].iov_len, pConv );
                         break;
                     } else {
-                        iov[index].iov_base = user_memory;
+                        iov[index].iov_base = (IOVBASE_TYPE *) user_memory;
                         iov[index].iov_len = pData->size;
                         user_memory += extent;
-                        COMPUTE_CSUM( iov[index].iov_base, iov[index].iov_len, pConv );
+                        COMPUTE_CSUM( iov[index].iov_base, (size_t)iov[index].iov_len, pConv );
                     }
                     max_allowed -= iov[index].iov_len;
                     total_bytes_converted += iov[index].iov_len;
@@ -186,33 +182,21 @@ ompi_pack_homogeneous_contig_with_gaps_function( ompi_convertor_t* pConv,
         }
 
         {
-            uint32_t done, counter;
+            uint32_t counter;
+            size_t done;
 
-            if( iov[iov_count].iov_base == NULL ) {
-                size_t length = iov[iov_count].iov_len;
-                if( 0 == length ) length = max_allowed;
-                iov[iov_count].iov_base = pConv->memAlloc_fn( &length, pConv->memAlloc_userdata );
-                iov[iov_count].iov_len = length;
-                (*freeAfter) |= (1 << 0);
-                if( max_allowed < (size_t)iov[iov_count].iov_len )
-                    iov[iov_count].iov_len = max_allowed;
-                else
-                    max_allowed = iov[iov_count].iov_len;
-            }
             packed_buffer = iov[iov_count].iov_base;
-            done = pConv->bConverted - i * pData->size;  /* how much data left last time */
-            user_memory += done;
+            done = pConv->bConverted - i * pData->size;  /* partial data from last pack */
             if( done != 0 ) {  /* still some data to copy from the last time */
                 done = pData->size - done;
                 OMPI_DDT_SAFEGUARD_POINTER( user_memory, done, pConv->pBaseBuf, pData, pConv->count );
                 MEMCPY_CSUM( packed_buffer, user_memory, done, pConv );
                 packed_buffer += done;
                 max_allowed -= done;
-                i++;  /* just to compute the correct source pointer */
                 total_bytes_converted += done;
+                user_memory += (extent - pData->size + done);
             }
-            user_memory = pConv->pBaseBuf + initial_displ + i * extent;
-            counter = max_allowed / pData->size;
+            counter = (uint32_t)(max_allowed / pData->size);
             if( counter > pConv->count ) counter = pConv->count;
             for( i = 0; i < counter; i++ ) {
                 OMPI_DDT_SAFEGUARD_POINTER( user_memory, pData->size, pConv->pBaseBuf, pData, pConv->count );
@@ -220,16 +204,23 @@ ompi_pack_homogeneous_contig_with_gaps_function( ompi_convertor_t* pConv,
                 packed_buffer+= pData->size;
                 user_memory += extent;
             }
-            max_allowed -= (counter * pData->size);
-            iov[iov_count].iov_len -= max_allowed;
-            total_bytes_converted += iov[iov_count].iov_len;
+            done = (counter * pData->size);
+            max_allowed -= done;
+            total_bytes_converted += done;
+            /* If there is anything pending ... */
+            if( 0 != max_allowed ) {
+                done = max_allowed;
+                OMPI_DDT_SAFEGUARD_POINTER( user_memory, done, pConv->pBaseBuf, pData, pConv->count );
+                MEMCPY_CSUM( packed_buffer, user_memory, done, pConv );
+                packed_buffer += done;
+                max_allowed = 0;
+                total_bytes_converted += done;
+                user_memory += done;
+            }
         }
-        /* Now update the user_memory pointer. At the end of each parth we have to update
-         * the pStack[0].disp field. BEWARE here we remove the pStack[1].disp as
-         * it's supposed to be useless from now.
-         */
-        user_memory = pConv->pBaseBuf + initial_displ + pStack[0].disp;
     }
+    pStack[0].disp = (intptr_t)user_memory - (intptr_t)pConv->pBaseBuf - initial_displ;
+    pStack[1].disp = max_allowed;
     *max_data = total_bytes_converted;
     pConv->bConverted += total_bytes_converted;
     *out_size = iov_count;
@@ -253,22 +244,21 @@ ompi_pack_homogeneous_contig_with_gaps_function( ompi_convertor_t* pConv,
 int32_t
 ompi_generic_simple_pack_function( ompi_convertor_t* pConvertor,
                                    struct iovec* iov, uint32_t* out_size,
-                                   size_t* max_data,
-                                   int32_t* freeAfter )
+                                   size_t* max_data )
 {
     dt_stack_t* pStack;       /* pointer to the position on the stack */
     uint32_t pos_desc;        /* actual position in the description of the derived datatype */
     uint32_t count_desc;      /* the number of items already done in the actual pos_desc */
-    uint16_t type;            /* type at current position */
     size_t total_packed = 0;  /* total amount packed this time */
     dt_elem_desc_t* description;
     dt_elem_desc_t* pElem;
     const ompi_datatype_t *pData = pConvertor->pDesc;
     char *source_base, *destination;
-    uint32_t iov_len_local, iov_count, required_space = 0;
+    size_t iov_len_local;
+    uint32_t iov_count;
 
-    DO_DEBUG( opal_output( 0, "ompi_convertor_generic_simple_pack( %p, {%p, %d}, %d )\n", (void*)pConvertor,
-                           iov[0].iov_base, iov[0].iov_len, *out_size ); );
+    DO_DEBUG( opal_output( 0, "ompi_convertor_generic_simple_pack( %p, {%p, %lu}, %d )\n", (void*)pConvertor,
+                           iov[0].iov_base, (unsigned long)iov[0].iov_len, *out_size ); );
 
     description = pConvertor->use_desc->desc;
 
@@ -279,7 +269,7 @@ ompi_generic_simple_pack_function( ompi_convertor_t* pConvertor,
     pStack = pConvertor->pStack + pConvertor->stack_pos;
     pos_desc     = pStack->index;
     source_base  = pConvertor->pBaseBuf + pStack->disp;
-    count_desc   = pStack->count;
+    count_desc   = (uint32_t)pStack->count;
     pStack--;
     pConvertor->stack_pos--;
     pElem = &(description[pos_desc]);
@@ -287,26 +277,10 @@ ompi_generic_simple_pack_function( ompi_convertor_t* pConvertor,
 
     DO_DEBUG( opal_output( 0, "unpack start pos_desc %d count_desc %d disp %ld\n"
                            "stack_pos %d pos_desc %d count_desc %d disp %ld\n",
-                           pos_desc, count_desc, source_base - pConvertor->pBaseBuf,
-                           pConvertor->stack_pos, pStack->index, pStack->count, pStack->disp ); );
+                           pos_desc, count_desc, (long)(source_base - pConvertor->pBaseBuf),
+                           pConvertor->stack_pos, pStack->index, (int)pStack->count, (long)pStack->disp ); );
 
     for( iov_count = 0; iov_count < (*out_size); iov_count++ ) {
-        if( required_space > ((*max_data) - total_packed) )
-            break;  /* do not pack over the boundaries even if there are more iovecs */
-        if( iov[iov_count].iov_base == NULL ) {
-            /*
-             *  ALLOCATE SOME MEMORY ...
-             */
-            size_t length = iov[iov_count].iov_len;
-            if( length <= 0 )
-                length = pConvertor->local_size - pConvertor->bConverted;
-            if( ((*max_data) - total_packed) < length )
-                length = (*max_data) - total_packed;
-            assert( 0 < length );
-            iov[iov_count].iov_base = pConvertor->memAlloc_fn( &length, pConvertor->memAlloc_userdata );
-            iov[iov_count].iov_len = length;
-            *freeAfter = (*freeAfter) | (1 << iov_count);
-        }
         destination = iov[iov_count].iov_base;
         iov_len_local = iov[iov_count].iov_len;
         while( 1 ) {
@@ -320,19 +294,17 @@ ompi_generic_simple_pack_function( ompi_convertor_t* pConvertor,
                     UPDATE_INTERNAL_COUNTERS( description, pos_desc, pElem, count_desc );
                     continue;
                 }
-                type = pElem->elem.common.type;
-                required_space = ompi_ddt_basicDatatypes[type]->size;
                 goto complete_loop;
             }
             if( DT_END_LOOP == pElem->elem.common.type ) { /* end of the current loop */
-                DO_DEBUG( opal_output( 0, "pack end_loop count %d stack_pos %d pos_desc %d disp %ld space %d\n",
-                                       pStack->count, pConvertor->stack_pos, pos_desc, pStack->disp, iov_len_local ); );
+                DO_DEBUG( opal_output( 0, "pack end_loop count %d stack_pos %d pos_desc %d disp %ld space %lu\n",
+                                       (int)pStack->count, pConvertor->stack_pos, pos_desc, (long)pStack->disp, (unsigned long)iov_len_local ); );
                 if( --(pStack->count) == 0 ) { /* end of loop */
                     if( pConvertor->stack_pos == 0 ) {
                         /* we lie about the size of the next element in order to
                          * make sure we exit the main loop.
                          */
-                        required_space = 0xffffffff;
+                        *out_size = iov_count;
                         goto complete_loop;  /* completed */
                     }
                     pConvertor->stack_pos--;
@@ -349,11 +321,11 @@ ompi_generic_simple_pack_function( ompi_convertor_t* pConvertor,
                 }
                 source_base = pConvertor->pBaseBuf + pStack->disp;
                 UPDATE_INTERNAL_COUNTERS( description, pos_desc, pElem, count_desc );
-                DO_DEBUG( opal_output( 0, "pack new_loop count %d stack_pos %d pos_desc %d disp %ld space %d\n",
-                                       pStack->count, pConvertor->stack_pos, pos_desc, pStack->disp, iov_len_local ); );
+                DO_DEBUG( opal_output( 0, "pack new_loop count %d stack_pos %d pos_desc %d disp %ld space %lu\n",
+                                       (int)pStack->count, pConvertor->stack_pos, pos_desc, (long)pStack->disp, (unsigned long)iov_len_local ); );
             }
             if( DT_LOOP == pElem->elem.common.type ) {
-                long local_disp = (long)source_base;
+                ptrdiff_t local_disp = (ptrdiff_t)source_base;
                 if( pElem->loop.common.flags & DT_FLAG_CONTIGUOUS ) {
                     PACK_CONTIGUOUS_LOOP( pConvertor, pElem, count_desc,
                                           source_base, destination, iov_len_local );
@@ -363,9 +335,9 @@ ompi_generic_simple_pack_function( ompi_convertor_t* pConvertor,
                     }
                     /* Save the stack with the correct last_count value. */
                 }
-                local_disp = (long)source_base - local_disp;
+                local_disp = (ptrdiff_t)source_base - local_disp;
                 PUSH_STACK( pStack, pConvertor->stack_pos, pos_desc, DT_LOOP, count_desc,
-                            pStack->disp + local_disp, pos_desc + pElem->elem.disp + 1);
+                            pStack->disp + local_disp);
                 pos_desc++;
             update_loop_description:  /* update the current state */
                 source_base = pConvertor->pBaseBuf + pStack->disp;
@@ -382,31 +354,14 @@ ompi_generic_simple_pack_function( ompi_convertor_t* pConvertor,
     *max_data = total_packed;
     *out_size = iov_count;
 
-#if 0
-    if( pConvertor->flags & CONVERTOR_WITH_CHECKSUM ) {
-        uint32_t ui1 = 0, ui2 = 0, csum = OPAL_CSUM_ZERO;
-        /**
-         * Check the checksum correctness.
-         */
-        for( iov_count = 0; iov_count < (*out_size); iov_count++ ) {
-            csum += opal_uicsum_partial( iov[iov_count].iov_base, iov[iov_count].iov_len,
-                                         &ui1, &ui2 );
-        }
-        if( csum != pConvertor->checksum ) {
-            opal_output( 0, "error in the pack function the checksum does not match\n"
-                         "(%d != %d)\n", csum, pConvertor->checksum );
-        }
-    }
-#endif
-
     if( pConvertor->bConverted == pConvertor->local_size ) {
         pConvertor->flags |= CONVERTOR_COMPLETED;
         return 1;
     }
     /* I complete an element, next step I should go to the next one */
     PUSH_STACK( pStack, pConvertor->stack_pos, pos_desc, DT_BYTE, count_desc,
-                source_base - pStack->disp - pConvertor->pBaseBuf, pos_desc );
+                source_base - pStack->disp - pConvertor->pBaseBuf );
     DO_DEBUG( opal_output( 0, "pack save stack stack_pos %d pos_desc %d count_desc %d disp %ld\n",
-                           pConvertor->stack_pos, pStack->index, pStack->count, pStack->disp ); );
+                           pConvertor->stack_pos, pStack->index, (int)pStack->count, (long)pStack->disp ); );
     return 0;
 }
