@@ -58,95 +58,93 @@ int32_t
 ompi_unpack_general_function( ompi_convertor_t* pConvertor,
                               struct iovec* iov,
                               uint32_t* out_size,
-                              size_t* max_data,
-                              int32_t* freeAfter )
+                              size_t* max_data )
 {
-    dt_stack_t* pStack;    /* pointer to the position on the stack */
-    uint32_t pos_desc;     /* actual position in the description of the derived datatype */
-    int count_desc;        /* the number of items already done in the actual pos_desc */
-    int type = DT_CHAR;    /* type at current position */
-    uint32_t advance;      /* number of bytes that we should advance the buffer */
-    long disp_desc = 0;    /* compute displacement for truncated data */
-    int bConverted = 0;    /* number of bytes converted this time */
+    dt_stack_t* pStack;      /* pointer to the position on the stack */
+    uint32_t pos_desc;       /* actual position in the description of the derived datatype */
+    int32_t count_desc;      /* the number of items already done in the actual pos_desc */
+    int type = DT_CHAR;      /* type at current position */
+    ptrdiff_t advance;       /* number of bytes that we should advance the buffer */
+    ptrdiff_t disp_desc = 0; /* compute displacement for truncated data */
+    size_t bConverted = 0;   /* number of bytes converted this time */
     const ompi_convertor_master_t* master = pConvertor->master;
-    dt_elem_desc_t* pElems;
-    long extent = pConvertor->pDesc->ub - pConvertor->pDesc->lb;
-    int oCount = extent * pConvertor->count;
+    dt_elem_desc_t* description;
+    ptrdiff_t extent = pConvertor->pDesc->ub - pConvertor->pDesc->lb;
+    size_t oCount = extent * pConvertor->count;
+    size_t iCount, total_bytes_converted = 0;
     char* pInput;
-    int iCount, rc;
-    uint32_t iov_count, total_bytes_converted = 0;
+    int32_t rc;
+    uint32_t iov_count;
 
     /* For the general case always use the user data description */
-    pElems = pConvertor->pDesc->desc.desc;
+    description = pConvertor->use_desc->desc;
 
     pStack = pConvertor->pStack + pConvertor->stack_pos;
     pos_desc   = pStack->index;
-    count_desc = pStack->count;
+    count_desc = (int32_t)pStack->count;
     disp_desc  = pStack->disp;
     pStack--;
     pConvertor->stack_pos--;
 
-    DDT_DUMP_STACK( pConvertor->pStack, pConvertor->stack_pos, pElems, "starting" );
-    DUMP( "remember position on stack %d last_elem at %d\n", pConvertor->stack_pos, pos_desc );
-    DUMP( "top stack info {index = %d, count = %d}\n", pStack->index, pStack->count );
+    DDT_DUMP_STACK( pConvertor->pStack, pConvertor->stack_pos, description, "starting" );
 
     for( iov_count = 0; iov_count < (*out_size); iov_count++ ) {
         bConverted = 0;
         pInput = iov[iov_count].iov_base;
         iCount = iov[iov_count].iov_len;
         while( 1 ) {
-            if( DT_END_LOOP == pElems[pos_desc].elem.common.type ) { /* end of the current loop */
+            if( DT_END_LOOP == description[pos_desc].elem.common.type ) { /* end of the current loop */
                 if( --(pStack->count) == 0 ) { /* end of loop */
                     if( pConvertor->stack_pos == 0 ) {
                         goto save_and_return;  /* completed */
                     }
                     pConvertor->stack_pos--;
                     pStack--;
-                }
-
-                if( pStack->index == -1 ) {
-                    pStack->disp += extent;
+                    pos_desc++;
                 } else {
-                    assert( DT_LOOP == pElems[pStack->index].elem.common.type );
-                    pStack->disp += pElems[pStack->index].loop.extent;
+                    pos_desc = pStack->index + 1;
+                    if( pStack->index == -1 ) {
+                        pStack->disp += extent;
+                    } else {
+                        assert( DT_LOOP == description[pStack->index].elem.common.type );
+                        pStack->disp += description[pStack->index].loop.extent;
+                    }
                 }
-                pos_desc = pStack->index + 1;
-                count_desc = pElems[pos_desc].elem.count;
-                disp_desc = pElems[pos_desc].elem.disp;
+                count_desc = description[pos_desc].elem.count;
+                disp_desc = description[pos_desc].elem.disp;
             }
-            if( DT_LOOP == pElems[pos_desc].elem.common.type ) {
+            if( DT_LOOP == description[pos_desc].elem.common.type ) {
                 do {
                     PUSH_STACK( pStack, pConvertor->stack_pos,
-                                pos_desc, DT_LOOP, pElems[pos_desc].loop.loops,
-                                pStack->disp, pos_desc + pElems[pos_desc].loop.items + 1 );
+                                pos_desc, DT_LOOP, description[pos_desc].loop.loops, pStack->disp );
                     pos_desc++;
-                } while( DT_LOOP == pElems[pos_desc].loop.common.type ); /* let's start another loop */
-                DDT_DUMP_STACK( pConvertor->pStack, pConvertor->stack_pos, pElems, "advance loops" );
+                } while( DT_LOOP == description[pos_desc].loop.common.type ); /* let's start another loop */
+                DDT_DUMP_STACK( pConvertor->pStack, pConvertor->stack_pos, description, "advance loops" );
                 /* update the current state */
-                count_desc = pElems[pos_desc].elem.count;
-                disp_desc = pElems[pos_desc].elem.disp;
+                count_desc = description[pos_desc].elem.count;
+                disp_desc = description[pos_desc].elem.disp;
             }
-            while( pElems[pos_desc].elem.common.flags & DT_FLAG_DATA ) {
+            while( description[pos_desc].elem.common.flags & DT_FLAG_DATA ) {
                 /* now here we have a basic datatype */
-                type = pElems[pos_desc].elem.common.type;
+                type = description[pos_desc].elem.common.type;
                 rc = master->pFunctions[type]( pConvertor, count_desc,
                                                pInput, iCount, ompi_ddt_basicDatatypes[type]->size,
                                                pConvertor->pBaseBuf + pStack->disp + disp_desc,
-                                               oCount, pElems[pos_desc].elem.extent, &advance );
+                                               oCount, description[pos_desc].elem.extent, &advance );
                 iCount -= advance;      /* decrease the available space in the buffer */
                 pInput += advance;      /* increase the pointer to the buffer */
                 bConverted += advance;
                 if( rc != count_desc ) {
                     /* not all data has been converted. Keep the state */
                     count_desc -= rc;
-                    disp_desc += rc * pElems[pos_desc].elem.extent;
+                    disp_desc += rc * description[pos_desc].elem.extent;
                     if( iCount != 0 )
-                        printf( "unpack there is still room in the input buffer %d bytes\n", iCount );
+                        printf( "unpack there is still room in the input buffer %ld bytes\n", (long)iCount );
                     goto save_and_return;
                 }
                 pos_desc++;  /* advance to the next data */
-                count_desc = pElems[pos_desc].elem.count;
-                disp_desc = pElems[pos_desc].elem.disp;
+                count_desc = description[pos_desc].elem.count;
+                disp_desc = description[pos_desc].elem.disp;
                 if( iCount == 0 )
                     goto save_and_return;  /* break if there is no more data in the buffer */
             }
@@ -167,7 +165,7 @@ ompi_unpack_general_function( ompi_convertor_t* pConvertor,
 
     /* I complete an element, next step I should go to the next one */
     PUSH_STACK( pStack, pConvertor->stack_pos, pos_desc, type,
-                count_desc, disp_desc, pos_desc );
+                count_desc, disp_desc );
 
     return 0;
 }
@@ -185,17 +183,18 @@ int32_t
 ompi_unpack_homogeneous_contig_function( ompi_convertor_t* pConv,
                                          struct iovec* iov,
                                          uint32_t* out_size,
-                                         size_t* max_data,
-                                         int32_t* freeAfter )
+                                         size_t* max_data )
 {
     const ompi_datatype_t *pData = pConv->pDesc;
     char *user_memory, *packed_buffer;
-    uint32_t iov_count, initial_bytes_converted = pConv->bConverted;
-    long extent = pData->ub - pData->lb;
-    uint32_t bConverted, length, remaining, i;
-    dt_stack_t* stack = &(pConv->pStack[1]);
-    long initial_displ = pConv->use_desc->desc[pConv->use_desc->used].end_loop.first_elem_disp;
+    uint32_t iov_count, i;
+    size_t bConverted, remaining, length, initial_bytes_converted = pConv->bConverted;
+    dt_stack_t* stack = pConv->pStack;
+    ptrdiff_t extent = pData->ub - pData->lb;
+    ptrdiff_t initial_displ = pConv->use_desc->desc[pConv->use_desc->used].end_loop.first_elem_disp;
 
+    DO_DEBUG( opal_output( 0, "unpack_homogeneous_contig( pBaseBuf %p, iov_count %d )\n",
+                           pConv->pBaseBuf, *out_size ); );
     for( iov_count = 0; iov_count < (*out_size); iov_count++ ) {
         packed_buffer = (char*)iov[iov_count].iov_base;
         remaining = pConv->local_size - pConv->bConverted;
@@ -204,54 +203,61 @@ ompi_unpack_homogeneous_contig_function( ompi_convertor_t* pConv,
         bConverted = remaining; /* how much will get unpacked this time */
         user_memory = pConv->pBaseBuf + initial_displ;
 
-        /*opal_output( 0, "unpack_homogeneous_contig( user_memory %p, packed_buffer %p length %d\n",
-          user_memory, packed_buffer, remaining );*/
-
-        if( (long)pData->size == extent ) {
+        if( (ptrdiff_t)pData->size == extent ) {
             user_memory += pConv->bConverted;
+            DO_DEBUG( opal_output( 0, "unpack_homogeneous_contig( user_memory %p, packed_buffer %p length %lu\n",
+                                   user_memory, packed_buffer, (unsigned long)remaining ); );
 
             /* contiguous data or basic datatype with count */
             OMPI_DDT_SAFEGUARD_POINTER( user_memory, remaining,
                                         pConv->pBaseBuf, pData, pConv->count );
-            /*opal_output( 0, "1. unpack contig dest %p src %p length %d\n",
-              user_memory, packed_buffer, remaining );*/
+            DO_DEBUG( opal_output( 0, "1. unpack contig dest %p src %p length %lu\n",
+                                   user_memory, packed_buffer, (unsigned long)remaining ); );
             MEMCPY_CSUM( user_memory, packed_buffer, remaining, pConv );
         } else {
-            user_memory += stack->disp;
+            user_memory += stack[0].disp + stack[1].disp;
+
+            DO_DEBUG( opal_output( 0, "unpack_homogeneous_contig( user_memory %p, packed_buffer %p length %lu\n",
+                                   user_memory, packed_buffer, (unsigned long)remaining ); );
 
             length = pConv->bConverted / pData->size;  /* already done */
-            length = pConv->bConverted - length * pData->size;  /* still left on the last element */
+            length = pConv->bConverted - length * pData->size;  /* how much of the last data we convert */
+
             /* complete the last copy */
             if( length != 0 ) {
-                OMPI_DDT_SAFEGUARD_POINTER( user_memory, length, pConv->pBaseBuf,
-                                            pData, pConv->count );
-                /*opal_output( 0, "1. unpack dest %p src %p length %d\n",
-                  user_memory, packed_buffer, length );*/
-                MEMCPY_CSUM( user_memory, packed_buffer, length, pConv );
-                packed_buffer += length;
-                user_memory   += (extent - (pData->size - length));
-                remaining     -= length;
+                length = pData->size - length;
+                if( length <= remaining ) {
+                    OMPI_DDT_SAFEGUARD_POINTER( user_memory, length, pConv->pBaseBuf,
+                                                pData, pConv->count );
+                    DO_DEBUG( opal_output( 0, "2. unpack dest %p src %p length %lu\n",
+                                           user_memory, packed_buffer, (unsigned long)length ); );
+                    MEMCPY_CSUM( user_memory, packed_buffer, length, pConv );
+                    packed_buffer += length;
+                    user_memory   += (extent - (pData->size - length));
+                    remaining     -= length;
+                }
             }
             for( i = 0; pData->size <= remaining; i++ ) {
                 OMPI_DDT_SAFEGUARD_POINTER( user_memory, pData->size, pConv->pBaseBuf,
                                             pData, pConv->count );
-                /*opal_output( 0, "2. unpack dest %p src %p length %d\n",
-                  user_memory, packed_buffer, pData->size );*/
+                DO_DEBUG( opal_output( 0, "3. unpack dest %p src %p length %lu\n",
+                                       user_memory, packed_buffer, (unsigned long)pData->size ); );
                 MEMCPY_CSUM( user_memory, packed_buffer, pData->size, pConv );
                 packed_buffer += pData->size;
                 user_memory   += extent;
                 remaining     -= pData->size;
             }
+            stack[0].disp = (intptr_t)user_memory - (intptr_t)pConv->pBaseBuf - initial_displ;
+            stack[1].disp = remaining;
             /* copy the last bits */
             if( remaining != 0 ) {
                 OMPI_DDT_SAFEGUARD_POINTER( user_memory, remaining, pConv->pBaseBuf,
                                             pData, pConv->count );
-                /*opal_output( 0, "3. unpack dest %p src %p length %d\n",
-                  user_memory, packed_buffer, remaining );*/
+                DO_DEBUG( opal_output( 0, "4. unpack dest %p src %p length %lu\n",
+                                       user_memory, packed_buffer, (unsigned long)remaining ); );
                 MEMCPY_CSUM( user_memory, packed_buffer, remaining, pConv );
                 user_memory += remaining;
             }
-            stack->disp = user_memory - pConv->pBaseBuf;  /* save the position */
         }
         pConv->bConverted += bConverted;
     }
@@ -276,26 +282,27 @@ ompi_unpack_homogeneous_contig_function( ompi_convertor_t* pConv,
  * change the content of the data (as in all conversions that require changing the size
  * of the exponent or mantissa).
  */
-static uint32_t
+static inline uint32_t
 ompi_unpack_partial_datatype( ompi_convertor_t* pConvertor, dt_elem_desc_t* pElem,
                               char* partial_data,
-                              uint32_t start_position, uint32_t end_position,
+                              ptrdiff_t start_position, ptrdiff_t end_position,
                               char** user_buffer )
 {
     char unused_byte = 0x7F, saved_data[16];
     char temporary[16], *temporary_buffer = temporary;
     char* real_data = *user_buffer + pElem->elem.disp;
-    uint32_t i, count_desc = 1;
-    uint32_t data_length = ompi_ddt_basicDatatypes[pElem->elem.common.type]->size;
+    uint32_t i, length, count_desc = 1;
+    size_t data_length = ompi_ddt_basicDatatypes[pElem->elem.common.type]->size;
 
-    DO_DEBUG( opal_output( 0, "unpack partial data start %d end %d data_length %d user %p\n"
-                           "\tbConverted %d total_length %d count %d\n",
-                           start_position, end_position, data_length, *user_buffer,
-                           pConvertor->bConverted,pConvertor->local_size, pConvertor->count ); );
+    DO_DEBUG( opal_output( 0, "unpack partial data start %lu end %lu data_length %lu user %p\n"
+                           "\tbConverted %lu total_length %lu count %d\n",
+                           (unsigned long)start_position, (unsigned long)end_position, (unsigned long)data_length, *user_buffer,
+                           (unsigned long)pConvertor->bConverted, (unsigned long)pConvertor->local_size, pConvertor->count ); );
 
     /* Find a byte that is not used in the partial buffer */
  find_unused_byte:
-    for( i = 0; i < (end_position - start_position); i++ ) {
+    length = (uint32_t)(end_position - start_position);
+    for( i = 0; i < length; i++ ) {
         if( unused_byte == partial_data[i] ) {
             unused_byte--;
             goto find_unused_byte;
@@ -339,8 +346,7 @@ ompi_unpack_partial_datatype( ompi_convertor_t* pConvertor, dt_elem_desc_t* pEle
 int32_t
 ompi_generic_simple_unpack_function( ompi_convertor_t* pConvertor,
                                      struct iovec* iov, uint32_t* out_size,
-                                     size_t* max_data,
-                                     int32_t* freeAfter )
+                                     size_t* max_data )
 {
     dt_stack_t* pStack;                /* pointer to the position on the stack */
     uint32_t pos_desc;                 /* actual position in the description of the derived datatype */
@@ -351,10 +357,11 @@ ompi_generic_simple_unpack_function( ompi_convertor_t* pConvertor,
     dt_elem_desc_t* pElem;
     const ompi_datatype_t *pData = pConvertor->pDesc;
     char *user_memory_base, *packed_buffer;
-    uint32_t iov_len_local, iov_count;
+    size_t iov_len_local;
+    uint32_t iov_count;
 
     DO_DEBUG( opal_output( 0, "ompi_convertor_generic_simple_unpack( %p, {%p, %lu}, %u )\n",
-                           (void*)pConvertor, iov[0].iov_base, (size_t)iov[0].iov_len, *out_size ); );
+                           (void*)pConvertor, iov[0].iov_base, (unsigned long)iov[0].iov_len, *out_size ); );
 
     description = pConvertor->use_desc->desc;
 
@@ -365,7 +372,7 @@ ompi_generic_simple_unpack_function( ompi_convertor_t* pConvertor,
     pStack = pConvertor->pStack + pConvertor->stack_pos;
     pos_desc          = pStack->index;
     user_memory_base  = pConvertor->pBaseBuf + pStack->disp;
-    count_desc        = pStack->count;
+    count_desc        = (uint32_t)pStack->count;
     pStack--;
     pConvertor->stack_pos--;
     pElem = &(description[pos_desc]); 
@@ -373,16 +380,16 @@ ompi_generic_simple_unpack_function( ompi_convertor_t* pConvertor,
 
     DO_DEBUG( opal_output( 0, "unpack start pos_desc %d count_desc %d disp %ld\n"
                            "stack_pos %d pos_desc %d count_desc %d disp %ld\n",
-                           pos_desc, count_desc, user_memory_base - pConvertor->pBaseBuf,
-                           pConvertor->stack_pos, pStack->index, pStack->count, pStack->disp ); );
+                           pos_desc, count_desc, (long)(user_memory_base - pConvertor->pBaseBuf),
+                           pConvertor->stack_pos, pStack->index, (int)pStack->count, (long)(pStack->disp) ); );
 
     for( iov_count = 0; iov_count < (*out_size); iov_count++ ) {
 
         packed_buffer = iov[iov_count].iov_base;
         iov_len_local = iov[iov_count].iov_len;
         if( 0 != pConvertor->partial_length ) {
-            uint32_t element_length = ompi_ddt_basicDatatypes[pElem->elem.common.type]->size;
-            uint32_t missing_length = element_length - pConvertor->partial_length;
+            size_t element_length = ompi_ddt_basicDatatypes[pElem->elem.common.type]->size;
+            size_t missing_length = element_length - pConvertor->partial_length;
 
             assert( pElem->elem.common.flags & DT_FLAG_DATA );
             COMPUTE_CSUM( packed_buffer, missing_length, pConvertor );
@@ -395,8 +402,6 @@ ompi_generic_simple_unpack_function( ompi_convertor_t* pConvertor,
                 user_memory_base = pConvertor->pBaseBuf + pStack->disp;
                 pos_desc++;  /* advance to the next data */
                 UPDATE_INTERNAL_COUNTERS( description, pos_desc, pElem, count_desc );
-            } else {
-                user_memory_base += pElem->elem.extent;
             }
             packed_buffer += missing_length;
             iov_len_local -= missing_length;
@@ -422,21 +427,20 @@ ompi_generic_simple_unpack_function( ompi_convertor_t* pConvertor,
                      */
                     assert( iov_len_local < ompi_ddt_basicDatatypes[type]->size );
                     COMPUTE_CSUM( packed_buffer, iov_len_local, pConvertor );
-                    /*opal_output( 0, "Saving %d bytes for the next call at address %p\n",
-                      iov_len_local, user_memory_base );*/
 
                     ompi_unpack_partial_datatype( pConvertor, pElem,
                                                   packed_buffer, 0, iov_len_local,
                                                   &temp );
 
-                    pConvertor->partial_length = iov_len_local;
+                    pConvertor->partial_length = (uint32_t)iov_len_local;
                     iov_len_local = 0;
                 }
                 goto complete_loop;
             }
             if( DT_END_LOOP == pElem->elem.common.type ) { /* end of the current loop */
-                DO_DEBUG( opal_output( 0, "unpack end_loop count %d stack_pos %d pos_desc %d disp %ld space %d\n",
-                                       pStack->count, pConvertor->stack_pos, pos_desc, pStack->disp, iov_len_local ); );
+                DO_DEBUG( opal_output( 0, "unpack end_loop count %d stack_pos %d pos_desc %d disp %ld space %lu\n",
+                                       (int)pStack->count, pConvertor->stack_pos, pos_desc,
+                                       (long)pStack->disp, (unsigned long)iov_len_local ); );
                 if( --(pStack->count) == 0 ) { /* end of loop */
                     if( pConvertor->stack_pos == 0 ) {
                         /* Force the conversion to stop by lowering the number of iovecs. */
@@ -457,11 +461,12 @@ ompi_generic_simple_unpack_function( ompi_convertor_t* pConvertor,
                 }
                 user_memory_base = pConvertor->pBaseBuf + pStack->disp;
                 UPDATE_INTERNAL_COUNTERS( description, pos_desc, pElem, count_desc );
-                DO_DEBUG( opal_output( 0, "unpack new_loop count %d stack_pos %d pos_desc %d disp %ld space %d\n",
-                                       pStack->count, pConvertor->stack_pos, pos_desc, pStack->disp, iov_len_local ); );
+                DO_DEBUG( opal_output( 0, "unpack new_loop count %d stack_pos %d pos_desc %d disp %ld space %lu\n",
+                                       (int)pStack->count, pConvertor->stack_pos, pos_desc,
+                                       (long)pStack->disp, (unsigned long)iov_len_local ); );
             }
             if( DT_LOOP == pElem->elem.common.type ) {
-                long local_disp = (long)user_memory_base;
+                ptrdiff_t local_disp = (ptrdiff_t)user_memory_base;
                 if( pElem->loop.common.flags & DT_FLAG_CONTIGUOUS ) {
                     UNPACK_CONTIGUOUS_LOOP( pConvertor, pElem, count_desc, 
                                             packed_buffer, user_memory_base, iov_len_local );
@@ -471,9 +476,9 @@ ompi_generic_simple_unpack_function( ompi_convertor_t* pConvertor,
                     }
                     /* Save the stack with the correct last_count value. */
                 }
-                local_disp = (long)user_memory_base - local_disp;
+                local_disp = (ptrdiff_t)user_memory_base - local_disp;
                 PUSH_STACK( pStack, pConvertor->stack_pos, pos_desc, DT_LOOP, count_desc,
-                            pStack->disp + local_disp, pos_desc + pElem->elem.disp + 1);
+                            pStack->disp + local_disp);
                 pos_desc++;
             update_loop_description:  /* update the current state */
                 user_memory_base = pConvertor->pBaseBuf + pStack->disp;
@@ -495,8 +500,8 @@ ompi_generic_simple_unpack_function( ompi_convertor_t* pConvertor,
     }
     /* I complete an element, next step I should go to the next one */
     PUSH_STACK( pStack, pConvertor->stack_pos, pos_desc, DT_BYTE, count_desc,
-                user_memory_base - pStack->disp - pConvertor->pBaseBuf, pos_desc );
+                user_memory_base - pStack->disp - pConvertor->pBaseBuf );
     DO_DEBUG( opal_output( 0, "unpack save stack stack_pos %d pos_desc %d count_desc %d disp %ld\n",
-                           pConvertor->stack_pos, pStack->index, pStack->count, pStack->disp ); );
+                           pConvertor->stack_pos, pStack->index, (int)pStack->count, (long)pStack->disp ); );
     return 0;
 }
