@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -107,7 +107,7 @@ OMPI_DECLSPEC extern ompi_file_t  ompi_mpi_file_null;
 /**
  * Fortran to C conversion table
  */
-OMPI_DECLSPEC extern ompi_pointer_array_t ompi_file_f_to_c_table;
+extern ompi_pointer_array_t ompi_file_f_to_c_table;
 
 
 
