@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -21,6 +21,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include "opal/util/show_help.h"
+#include "orte/mca/ns/ns.h"
 #include "ompi/class/ompi_bitmap.h"
 #include "ompi/mca/bml/bml.h"
 #include "ompi/mca/btl/btl.h"
@@ -45,7 +47,9 @@ mca_bml_r2_module_t mca_bml_r2 = {
         mca_bml_r2_del_procs,
         mca_bml_r2_add_btl,
         mca_bml_r2_del_btl,
+        mca_bml_r2_del_proc_btl,
         mca_bml_r2_register, 
+        mca_bml_r2_register_error,
         mca_bml_r2_finalize, 
         mca_bml_r2_progress
     }
@@ -96,7 +100,7 @@ int mca_bml_r2_progress( void ) {
     int count = 0;
 
     /*
-     * Progress each of the PTL modules
+     * Progress each of the BTL modules
      */
     for(i=0; i<mca_bml_r2.num_btl_progress; i++) {
         int rc = mca_bml_r2.btl_progress[i]();
@@ -105,7 +109,6 @@ int mca_bml_r2_progress( void ) {
         }
     }
     return count;
-    
 }
 
 
@@ -173,6 +176,8 @@ int mca_bml_r2_add_procs(
     struct mca_btl_base_endpoint_t ** btl_endpoints = NULL;  
     struct ompi_proc_t** new_procs = NULL; 
     size_t n_new_procs = 0;
+    int ret = OMPI_SUCCESS;
+    struct ompi_proc_t *unreach_proc = NULL;
 
     if(0 == nprocs) {
         return OMPI_SUCCESS;
@@ -194,9 +199,9 @@ int mca_bml_r2_add_procs(
         proc = procs[p_index]; 
         OBJ_RETAIN(proc); 
         
-        if(NULL !=  proc->proc_pml) { 
+        if(NULL !=  proc->proc_bml) { 
             bml_endpoints[p_index] = 
-                (mca_bml_base_endpoint_t*) proc->proc_pml; 
+                (mca_bml_base_endpoint_t*) proc->proc_bml; 
         } else { 
             new_procs[n_new_procs++] = proc; 
         }
@@ -238,7 +243,7 @@ int mca_bml_r2_add_procs(
         for(p=0; p<n_new_procs; p++) {
             if(ompi_bitmap_is_set_bit(reachable, p)) {
                 ompi_proc_t *proc = new_procs[p]; 
-                mca_bml_base_endpoint_t * bml_endpoint = (mca_bml_base_endpoint_t*) proc->proc_pml; 
+                mca_bml_base_endpoint_t * bml_endpoint = (mca_bml_base_endpoint_t*) proc->proc_bml; 
                 mca_bml_base_btl_t* bml_btl; 
                 size_t size;
                 
@@ -248,8 +253,7 @@ int mca_bml_r2_add_procs(
                     
                     
                     /* allocate bml specific proc data */
-                    bml_endpoint = (mca_bml_base_endpoint_t*) 
-                        opal_obj_new(mca_bml_r2.endpoint_class);
+                    bml_endpoint = OBJ_NEW(mca_bml_base_endpoint_t);
                     if (NULL == bml_endpoint) {
                         opal_output(0, "mca_bml_r2_add_procs: unable to allocate resources");
                         free(btl_endpoints);
@@ -261,13 +265,14 @@ int mca_bml_r2_add_procs(
                     mca_bml_base_btl_array_reserve(&bml_endpoint->btl_send,  mca_bml_r2.num_btl_modules);
                     mca_bml_base_btl_array_reserve(&bml_endpoint->btl_rdma,  mca_bml_r2.num_btl_modules);
                     bml_endpoint->btl_max_send_size = -1;
-                    bml_endpoint->btl_rdma_size = -1;
-                    bml_endpoint->super.proc_ompi = proc;
-                    proc->proc_pml = (struct mca_pml_proc_t*) bml_endpoint; 
-                    
+                    bml_endpoint->btl_proc = proc;
+                    proc->proc_bml = bml_endpoint; 
+                 
+                    bml_endpoint->btl_flags_and = 0;
+                    bml_endpoint->btl_flags_or = 0;
                 }
 
-                bml_endpoints[p] =(mca_bml_base_endpoint_t*)  proc->proc_pml; 
+                bml_endpoints[p] =(mca_bml_base_endpoint_t*)  proc->proc_bml; 
                 
                 
                 /* dont allow an additional BTL with a lower exclusivity ranking */
@@ -279,12 +284,10 @@ int mca_bml_r2_add_procs(
                         if(btl_endpoints[p] != NULL) {
                             btl->btl_del_procs(btl, 1, &proc, &btl_endpoints[p]);
                         }
+                        btl_inuse--;
                         continue;
                     }
                 }
-                
-                
-                
 
                 /* cache the endpoint on the proc */
                 bml_btl = mca_bml_base_btl_array_insert(&bml_endpoint->btl_send);
@@ -302,10 +305,35 @@ int mca_bml_r2_add_procs(
                 bml_btl->btl_prepare_src = btl->btl_prepare_src;
                 bml_btl->btl_prepare_dst = btl->btl_prepare_dst;
                 bml_btl->btl_send = btl->btl_send;
-                bml_btl->btl_put = btl->btl_put;
-                bml_btl->btl_get = btl->btl_get;
                 bml_btl->btl_flags = btl->btl_flags; 
+                bml_btl->btl_put = btl->btl_put;
+                if( (bml_btl->btl_flags & MCA_BTL_FLAGS_PUT) && (NULL == bml_btl->btl_put) ) {
+                    opal_output(0, "mca_bml_r2_add_procs: The PUT flag is specified for"
+                                " the %s BTL without any PUT function attached. Disard the flag !",
+                                bml_btl->btl->btl_component->btl_version.mca_component_name);
+                    bml_btl->btl_flags ^= MCA_BTL_FLAGS_PUT;
+                }
+                bml_btl->btl_get = btl->btl_get;
+                if( (bml_btl->btl_flags & MCA_BTL_FLAGS_GET) && (NULL == bml_btl->btl_get) ) {
+                    opal_output(0, "mca_bml_r2_add_procs: The GET flag is specified for"
+                                " the %s BTL without any GET function attached. Disard the flag !",
+                                bml_btl->btl->btl_component->btl_version.mca_component_name);
+                    bml_btl->btl_flags ^= MCA_BTL_FLAGS_GET;
+                }
                 bml_btl->btl_mpool = btl->btl_mpool;
+                if( (bml_btl->btl_flags & (MCA_BTL_FLAGS_PUT | MCA_BTL_FLAGS_GET | MCA_BTL_FLAGS_SEND)) == 0 ) {
+                    /**
+                     * If no protocol specified, we have 2 choices: we ignore the BTL
+                     * as we don't know which protocl to use, or we suppose that all
+                     * BTLs support the send protocol. 
+                     */
+                    bml_btl->btl_flags |= MCA_BTL_FLAGS_SEND;
+                }
+                /**
+                 * calculate the bitwise OR and AND of the btl flags 
+                 */
+                bml_endpoint->btl_flags_or |= bml_btl->btl_flags;
+                bml_endpoint->btl_flags_and &= bml_btl->btl_flags;
             }
         }
         if(btl_inuse > 0 && NULL != btl->btl_component->btl_progress) {
@@ -329,9 +357,9 @@ int mca_bml_r2_add_procs(
     /* iterate back through procs and compute metrics for registered r2s */
     for(p=0; p<n_new_procs; p++) {
         ompi_proc_t *proc = new_procs[p];
-        mca_bml_base_endpoint_t* bml_endpoint = (mca_bml_base_endpoint_t*) proc->proc_pml;
+        mca_bml_base_endpoint_t* bml_endpoint = (mca_bml_base_endpoint_t*) proc->proc_bml;
         double total_bandwidth = 0;
-        uint32_t latency = 0;
+        uint32_t latency = 0xffffffff;
         size_t n_index;
         size_t n_size;
 
@@ -343,23 +371,29 @@ int mca_bml_r2_add_procs(
         /* (1) determine the total bandwidth available across all btls
          *     note that we need to do this here, as we may already have btls configured
          * (2) determine the highest priority ranking for latency
+         * (3) compute the maximum amount of bytes that can be send without any
+         *     weighting. Once the left over is smaller than this number we will
+         *     start using the weight to compute the correct amount.
          */
         n_size = mca_bml_base_btl_array_get_size(&bml_endpoint->btl_send); 
+        bml_endpoint->bml_max_send_length = 0;
+        bml_endpoint->bml_max_rdma_length = 0;
+        bml_endpoint->btl_rdma_index = 0;
         for(n_index = 0; n_index < n_size; n_index++) {
             mca_bml_base_btl_t* bml_btl = 
                 mca_bml_base_btl_array_get_index(&bml_endpoint->btl_send, n_index);
             mca_btl_base_module_t* btl = bml_btl->btl;
-            total_bandwidth += bml_btl->btl->btl_bandwidth; 
-            if(btl->btl_latency > latency) {
+            total_bandwidth += bml_btl->btl->btl_bandwidth;
+            if(btl->btl_latency < latency) {
                 latency = btl->btl_latency;
             }
+            bml_endpoint->bml_max_send_length += bml_btl->btl->btl_bandwidth;
         }
-
+        
         /* (1) set the weight of each btl as a percentage of overall bandwidth
          * (2) copy all btl instances at the highest priority ranking into the
          *     list of btls used for first fragments
          */
-
         for(n_index = 0; n_index < n_size; n_index++) {
             mca_bml_base_btl_t* bml_btl = 
                 mca_bml_base_btl_array_get_index(&bml_endpoint->btl_send, n_index);
@@ -393,16 +427,42 @@ int mca_bml_r2_add_procs(
                 if(bml_endpoint->btl_rdma_offset < bml_btl_rdma->btl_min_rdma_size) {
                     bml_endpoint->btl_rdma_offset = bml_btl_rdma->btl_min_rdma_size;
                 }
-                if(bml_endpoint->btl_rdma_size > btl->btl_max_rdma_size) {
-                   bml_endpoint->btl_rdma_size = btl->btl_max_rdma_size;
-                   bml_endpoint->btl_rdma_align = bml_base_log2(bml_endpoint->btl_rdma_size);
-                }
             }
         }
     }
 
+    /* see if we have a connection to everyone else */
+    for(p=0; p<n_new_procs; p++) {
+        ompi_proc_t *proc = new_procs[p];
+
+        if (NULL == proc->proc_bml) {
+            if (NULL == unreach_proc) {
+                unreach_proc = proc;
+            }
+            ret = OMPI_ERR_UNREACH;
+        }
+    }
+
+    if (mca_bml_r2.show_unreach_errors && 
+        OMPI_ERR_UNREACH == ret) {
+        char *local, *remote;
+
+        orte_ns.get_proc_name_string(&local,
+                                     &(ompi_proc_local_proc->proc_name));
+        orte_ns.get_proc_name_string(&remote,
+                                     &(unreach_proc->proc_name));
+
+        opal_show_help("help-mca-bml-r2",
+                       "unreachable proc",
+                       true, local, remote, NULL);
+
+        free(local);
+        free(remote);
+    }
+
     free(new_procs); 
-    return OMPI_SUCCESS;
+
+    return ret;
 }
 
 /*
@@ -432,7 +492,7 @@ int mca_bml_r2_del_procs(size_t nprocs,
     
     for(p = 0; p < n_del_procs; p++) {
         ompi_proc_t *proc = del_procs[p];
-        mca_bml_base_endpoint_t* bml_endpoint = (mca_bml_base_endpoint_t*) proc->proc_pml;
+        mca_bml_base_endpoint_t* bml_endpoint = (mca_bml_base_endpoint_t*) proc->proc_bml;
         size_t f_index, f_size;
         size_t n_index, n_size;
  
@@ -494,96 +554,134 @@ int mca_bml_r2_finalize( void ) {
 int mca_bml_r2_del_btl(mca_btl_base_module_t* btl)
 {
     ompi_proc_t** procs;
-    size_t p, num_procs;
-
+    size_t i, j, m, p, num_procs;
+    opal_list_item_t* item;
+    mca_btl_base_module_t** modules;
+    mca_btl_base_component_progress_fn_t * btl_progress_new; 
+    
+    
     procs = ompi_proc_all(&num_procs);
     if(NULL == procs)
         return OMPI_SUCCESS;
 
+    /* dont use this btl for any peers */
     for(p=0; p<num_procs; p++) {
-        double total_bandwidth = 0;
         ompi_proc_t* proc = procs[p];
-        mca_bml_base_endpoint_t* ep_old = (mca_bml_base_endpoint_t*)proc->proc_pml;
-        mca_bml_base_endpoint_t* ep_new = OBJ_NEW(mca_bml_base_endpoint_t);
-        size_t b;
-
-        /* initialize */
-        ep_new->super.proc_ompi = proc;
-        ep_new->btl_max_send_size = -1;
-        ep_new->btl_rdma_size = -1;
-        ep_new->btl_rdma_align = 0;
-        ep_new->btl_rdma_offset = 0;
-
-        /* build new eager list */
-        mca_bml_base_btl_array_reserve(&ep_new->btl_eager, mca_bml_base_btl_array_get_size(&ep_old->btl_eager));
-        for(b=0; b< mca_bml_base_btl_array_get_size(&ep_old->btl_eager); b++) {
-            mca_bml_base_btl_t* bml_btl_old = mca_bml_base_btl_array_get_index(&ep_old->btl_eager, b);
-            if(bml_btl_old->btl != btl) {
-                 mca_bml_base_btl_t* bml_btl_new = mca_bml_base_btl_array_insert(&ep_new->btl_eager);
-                 *bml_btl_new = *bml_btl_old;
-            }
-        }
-
-        /* build new send list */
-        total_bandwidth = 0;
-        mca_bml_base_btl_array_reserve(&ep_new->btl_send, mca_bml_base_btl_array_get_size(&ep_old->btl_send));
-        for(b=0; b< mca_bml_base_btl_array_get_size(&ep_old->btl_send); b++) {
-            mca_bml_base_btl_t* bml_btl_old = mca_bml_base_btl_array_get_index(&ep_old->btl_send, b);
-            if(bml_btl_old->btl != btl) {
-                mca_bml_base_btl_t* bml_btl_new = mca_bml_base_btl_array_insert(&ep_new->btl_send);
-                *bml_btl_new = *bml_btl_old;
-                total_bandwidth += bml_btl_new->btl->btl_bandwidth;
-                if (bml_btl_new->btl_max_send_size < ep_new->btl_max_send_size) {
-                    ep_new->btl_max_send_size = bml_btl_new->btl->btl_max_send_size;
-                }
-            }
-        }
-        /* compute weighting factor for this btl */
-        for(b=0; b< mca_bml_base_btl_array_get_size(&ep_new->btl_send); b++) {
-            mca_bml_base_btl_t* bml_btl = mca_bml_base_btl_array_get_index(&ep_old->btl_send, b);
-            if(bml_btl->btl->btl_bandwidth > 0) {
-                bml_btl->btl_weight = bml_btl->btl->btl_bandwidth / total_bandwidth;
-            } else {
-                bml_btl->btl_weight = 1.0 / mca_bml_base_btl_array_get_size(&ep_new->btl_send);
-            }
-        }
-
-        /* build new rdma list */
-        total_bandwidth = 0;
-        mca_bml_base_btl_array_reserve(&ep_new->btl_rdma, mca_bml_base_btl_array_get_size(&ep_old->btl_rdma));
-        for(b=0; b< mca_bml_base_btl_array_get_size(&ep_old->btl_rdma); b++) {
-            mca_bml_base_btl_t* bml_btl_old = mca_bml_base_btl_array_get_index(&ep_old->btl_rdma, b);
-            if(bml_btl_old->btl != btl) {
-                mca_bml_base_btl_t* bml_btl_new = mca_bml_base_btl_array_insert(&ep_new->btl_rdma);
-                *bml_btl_new = *bml_btl_old;
-
-                /* update aggregate endpoint info */
-                total_bandwidth += bml_btl_new->btl->btl_bandwidth;
-                if (ep_new->btl_rdma_offset < bml_btl_new->btl_min_rdma_size) {
-                    ep_new->btl_rdma_offset = bml_btl_new->btl_min_rdma_size;
-                }
-                if (ep_new->btl_rdma_size > bml_btl_new->btl_max_rdma_size) {
-                    ep_new->btl_rdma_size = bml_btl_new->btl_max_rdma_size;
-                    ep_new->btl_rdma_align = bml_base_log2(ep_new->btl_rdma_size);
-                }
-            }
-        }
-
-        /* compute weighting factor for this btl */
-        for(b=0; b< mca_bml_base_btl_array_get_size(&ep_new->btl_rdma); b++) {
-            mca_bml_base_btl_t* bml_btl = mca_bml_base_btl_array_get_index(&ep_old->btl_rdma, b);
-            if(bml_btl->btl->btl_bandwidth > 0) {
-                bml_btl->btl_weight = bml_btl->btl->btl_bandwidth / total_bandwidth;
-            } else {
-                bml_btl->btl_weight = 1.0 / mca_bml_base_btl_array_get_size(&ep_new->btl_rdma);
-            }
-        }
-
-        /* save on proc */
-        proc->proc_pml = (mca_pml_proc_t*)ep_new;
-
-        /* DONT delete old endpoint structure as it may still be in use... */
+        mca_bml_r2_del_proc_btl(proc, btl);
     }
+
+    /* remove from the btl list */
+    for (item =  opal_list_get_first(&mca_btl_base_modules_initialized);
+         item != opal_list_get_end(&mca_btl_base_modules_initialized);
+         item =  opal_list_get_next(item)) {
+        mca_btl_base_selected_module_t *sm = (mca_btl_base_selected_module_t *) item;
+        if(sm->btl_module == btl) {
+            opal_list_remove_item(&mca_btl_base_modules_initialized, item);
+            free(sm);
+            break;
+        }
+    }
+
+    /* remove from bml list */
+    modules = (mca_btl_base_module_t**)malloc(sizeof(mca_btl_base_module_t*) * mca_bml_r2.num_btl_modules-1);
+    for(i=0,m=0; i<mca_bml_r2.num_btl_modules; i++) {
+        if(mca_bml_r2.btl_modules[i] != btl) {
+            modules[m++] = mca_bml_r2.btl_modules[i];
+        }
+    }
+    free(mca_bml_r2.btl_modules);
+    mca_bml_r2.btl_modules = modules;
+    mca_bml_r2.num_btl_modules = m;
+
+    
+    /* remove progress function so btl_progress isn't 
+       called on the failed BTL */ 
+    if(mca_bml_r2.num_btl_progress <= 1) { 
+        /* nothing left to send on! */ 
+        opal_output(0, "%s:%d:%s: only one BTL, can't fail-over!", 
+                    __FILE__, __LINE__, __func__);
+        return OMPI_ERROR;
+    }
+    /* figure out which progress functions to keep */
+    btl_progress_new = (mca_btl_base_component_progress_fn_t*)
+        malloc(sizeof(mca_btl_base_component_progress_fn_t) * 
+               (mca_bml_r2.num_btl_progress - 1));
+    j = 0;
+    for(i = 0; i < mca_bml_r2.num_btl_progress; i++) { 
+        if(btl->btl_component->btl_progress != mca_bml_r2.btl_progress[i]) { 
+            btl_progress_new[j] = mca_bml_r2.btl_progress[i];
+            j++;
+        }
+    }
+    free(mca_bml_r2.btl_progress);
+    mca_bml_r2.btl_progress = btl_progress_new;
+    mca_bml_r2.num_btl_progress--; 
+    
+    /* cleanup */
+    btl->btl_finalize(btl);
+    free(procs);
+    return OMPI_SUCCESS;
+}
+
+int mca_bml_r2_del_proc_btl(ompi_proc_t* proc, mca_btl_base_module_t* btl)
+{
+    mca_bml_base_endpoint_t* ep = (mca_bml_base_endpoint_t*)proc->proc_bml;
+    double total_bandwidth = 0;
+    size_t b;
+
+    /* remove btl from eager list */
+    mca_bml_base_btl_array_remove(&ep->btl_eager, btl);
+    
+    /* remove btl from send list */ 
+    if(mca_bml_base_btl_array_remove(&ep->btl_send, btl)) { 
+    
+        /* compute total_bandwidth and 
+           reset max_send_size to the min of all btl's */
+        total_bandwidth = 0;
+        for(b=0; b< mca_bml_base_btl_array_get_size(&ep->btl_send); b++) {
+            mca_bml_base_btl_t* bml_btl = mca_bml_base_btl_array_get_index(&ep->btl_send, b);
+            total_bandwidth += bml_btl->btl->btl_bandwidth;
+            if (bml_btl->btl_max_send_size < ep->btl_max_send_size) {
+                ep->btl_max_send_size = bml_btl->btl->btl_max_send_size;
+            }
+        }
+        
+        /* compute weighting factor for this btl */
+        for(b=0; b< mca_bml_base_btl_array_get_size(&ep->btl_send); b++) {
+            mca_bml_base_btl_t* bml_btl = mca_bml_base_btl_array_get_index(&ep->btl_send, b);
+            if(bml_btl->btl->btl_bandwidth > 0) {
+                bml_btl->btl_weight = bml_btl->btl->btl_bandwidth / total_bandwidth;
+            } else {
+                bml_btl->btl_weight = 1.0 / mca_bml_base_btl_array_get_size(&ep->btl_send);
+            }
+        }
+    }
+    
+    /* remove btl from RDMA list */
+    if(mca_bml_base_btl_array_remove(&ep->btl_rdma, btl)) { 
+        
+        /* computer total bandwidth */
+        total_bandwidth = 0;
+        for(b=0; b< mca_bml_base_btl_array_get_size(&ep->btl_rdma); b++) {
+            mca_bml_base_btl_t* bml_btl = mca_bml_base_btl_array_get_index(&ep->btl_rdma, b);
+            /* update aggregate endpoint info */
+            total_bandwidth += bml_btl->btl->btl_bandwidth;
+            if (ep->btl_rdma_offset < bml_btl->btl_min_rdma_size) {
+                ep->btl_rdma_offset = bml_btl->btl_min_rdma_size;
+            } 
+        }
+        
+        /* compute weighting factor for this btl */
+        for(b=0; b< mca_bml_base_btl_array_get_size(&ep->btl_rdma); b++) {
+            mca_bml_base_btl_t* bml_btl = mca_bml_base_btl_array_get_index(&ep->btl_rdma, b);
+            if(bml_btl->btl->btl_bandwidth > 0) {
+                bml_btl->btl_weight = bml_btl->btl->btl_bandwidth / total_bandwidth;
+            } else {
+                bml_btl->btl_weight = 1.0 / mca_bml_base_btl_array_get_size(&ep->btl_rdma);
+            }
+        }
+    }
+    
     return OMPI_SUCCESS;
 }
 
@@ -618,6 +716,40 @@ int mca_bml_r2_register(
 }
 
 
+/*
+ *  Register an error handler with/ all active btls
+ *   if they support error handlers..
+ */
+
+int mca_bml_r2_register_error( 
+                        mca_btl_base_module_error_cb_fn_t  cbfunc
+                        )
+{
+    uint32_t  i; 
+    int rc;
+    mca_btl_base_module_t *btl; 
+    uint32_t ver;
+    
+    for(i = 0; i < mca_bml_r2.num_btl_modules; i++) { 
+        btl = mca_bml_r2.btl_modules[i]; 
+        /* this wont work for version numbers greater than 256... seems 
+           reasonable.. */
+        ver = btl->btl_component->btl_version.mca_type_major_version << 16 |
+            btl->btl_component->btl_version.mca_type_minor_version << 8 |
+            btl->btl_component->btl_version.mca_type_release_version;
+        /* is version number greater than or equal to 1.0.1? */
+        if(ver >= ((1 << 16) |  (0 << 8) | 1) &&            
+           NULL != btl->btl_register_error) { 
+            rc = btl->btl_register_error(btl, cbfunc);  
+            if(OMPI_SUCCESS != rc) {
+                return rc;
+            }
+        }
+    }
+    return OMPI_SUCCESS; 
+}
+
+
 int mca_bml_r2_component_fini(void)
 {
     /* FIX */
