@@ -84,7 +84,10 @@ mca_btl_gm_module_t mca_btl_gm_module = {
         mca_btl_gm_put_nl,
         mca_btl_gm_get_nl,
 #endif
-        mca_btl_base_dump
+        mca_btl_base_dump,
+        NULL, /* mpool */
+        mca_btl_gm_register_error_cb
+        
     }
 };
 
@@ -175,6 +178,19 @@ int mca_btl_gm_register(
 }
 
 
+/* 
+ *Register callback function for error handling..
+ */ 
+int mca_btl_gm_register_error_cb(
+                        struct mca_btl_base_module_t* btl, 
+                        mca_btl_base_module_error_cb_fn_t cbfunc)
+{
+    
+    mca_btl_gm_module_t* gm_btl = (mca_btl_gm_module_t*) btl; 
+    gm_btl->error_cb = cbfunc; /* stash for later */
+    return OMPI_SUCCESS;
+}
+
 /**
  * Allocate a segment.
  *
@@ -192,9 +208,18 @@ mca_btl_base_descriptor_t* mca_btl_gm_alloc(
     
     if(size <= btl->btl_eager_limit) { 
         MCA_BTL_GM_FRAG_ALLOC_EAGER(gm_btl, frag, rc); 
+        if(NULL == frag) { 
+            return NULL;
+        }
+        frag->type=MCA_BTL_GM_EAGER;
         frag->segment.seg_len = size;
+        
     } else if(size <=  btl->btl_max_send_size) { 
         MCA_BTL_GM_FRAG_ALLOC_MAX(gm_btl, frag, rc); 
+        if(NULL == frag) { 
+            return NULL;
+        }
+        frag->type=MCA_BTL_GM_SEND;
         frag->segment.seg_len = size; 
     } else { 
         return NULL;
@@ -213,11 +238,16 @@ mca_btl_base_descriptor_t* mca_btl_gm_alloc(
  * Return a segment
  */
 
-int mca_btl_gm_free(
-    struct mca_btl_base_module_t* btl, 
-    mca_btl_base_descriptor_t* des) 
+int mca_btl_gm_free( struct mca_btl_base_module_t* btl, 
+		     mca_btl_base_descriptor_t* des )
 {
-    mca_btl_gm_frag_t* frag = (mca_btl_gm_frag_t*)des; 
+    mca_btl_gm_frag_t* frag = (mca_btl_gm_frag_t*)des;
+
+    if( NULL != frag->registration ) {
+        btl->btl_mpool->mpool_deregister(btl->btl_mpool, (mca_mpool_base_registration_t*) frag->registration);
+        frag->registration = NULL;
+    }
+
     MCA_BTL_GM_FRAG_RETURN(btl, frag); 
     return OMPI_SUCCESS; 
 }
@@ -238,136 +268,95 @@ mca_btl_base_descriptor_t* mca_btl_gm_prepare_src(
     size_t* size
 )
 {
-    mca_btl_gm_frag_t* frag;
+    mca_btl_gm_frag_t *frag = NULL;
     struct iovec iov;
     uint32_t iov_count = 1;
     size_t max_data = *size;
-    int32_t free_after;
     int rc;
 
 #if (OMPI_MCA_BTL_GM_HAVE_RDMA_GET || OMPI_MCA_BTL_GM_HAVE_RDMA_PUT)
-    /*
-     * If the data has already been pinned and is contigous than we can
-     * use it in place.
-    */
-    if (NULL != registration && 0 == ompi_convertor_need_buffers(convertor)) {
-        size_t reg_len;
-        MCA_BTL_GM_FRAG_ALLOC_USER(btl, frag, rc);
-        if(NULL == frag){
-            return NULL;
+    if(ompi_convertor_need_buffers(convertor) == false && 0 == reserve) {
+        if(registration != NULL || max_data > btl->btl_max_send_size) {
+            MCA_BTL_GM_FRAG_ALLOC_USER(btl, frag, rc);
+            if(NULL == frag) {
+                return NULL;
+            }
+
+            /*
+             * just assign it something..
+             * we will assign the real value in put/get
+             */
+            frag->type = MCA_BTL_GM_PUT;
+            iov.iov_len = max_data;
+            iov.iov_base = NULL;
+
+
+            ompi_convertor_pack(convertor, &iov, &iov_count, &max_data);
+
+            *size = max_data;
+
+            if(NULL == registration) {
+                rc = btl->btl_mpool->mpool_register(btl->btl_mpool,
+                        iov.iov_base, max_data, 0, &registration);
+                if(OMPI_SUCCESS != rc || NULL == registration) {
+                    MCA_BTL_GM_FRAG_RETURN(btl, frag);
+                    return NULL;
+                }
+                /* keep track of the registration we did */
+                frag->registration = registration;
+            }
+
+            frag->segment.seg_len = max_data;
+            frag->segment.seg_addr.pval = iov.iov_base;
+
+            frag->base.des_src = &frag->segment;
+            frag->base.des_src_cnt = 1;
+            frag->base.des_dst = NULL;
+            frag->base.des_dst_cnt = 0;
+            frag->base.des_flags = 0;
+
+            return &frag->base;
         }
-        iov.iov_len = max_data;
-        iov.iov_base = NULL;
-
-        ompi_convertor_pack(convertor, &iov, &iov_count, &max_data, &free_after);
-                                                                                                    
-        frag->segment.seg_len = max_data;
-        frag->segment.seg_addr.pval = iov.iov_base;
-
-        reg_len = (unsigned char*)registration->bound - (unsigned char*)iov.iov_base + 1;
-        
-        /* bump reference count as so that the registration
-         * doesn't go away when the operation completes
-         */
-        btl->btl_mpool->mpool_retain(btl->btl_mpool, 
-                                     (mca_mpool_base_registration_t*) registration);
-        
-        frag->registration = registration;
-
-    /*
-     * if the data is not already pinned - but the leave pinned option is set,
-     * then go ahead and pin contigous data. however, if a reserve is required 
-     * then we must allocated a fragment w/ buffer space
-    */
-    } else if (max_data > btl->btl_max_send_size && 
-               ompi_convertor_need_buffers(convertor) == 0 &&
-               reserve == 0) {
-
-        mca_mpool_base_module_t* mpool = btl->btl_mpool;
-        MCA_BTL_GM_FRAG_ALLOC_USER(btl, frag, rc);
-        if(NULL == frag){
-            return NULL;
-        }
-        iov.iov_len = max_data;
-        iov.iov_base = NULL;
-
-        ompi_convertor_pack(convertor, &iov, &iov_count, &max_data, &free_after);
-                                                                                                
-        frag->segment.seg_len = max_data;
-        frag->segment.seg_addr.pval = iov.iov_base;
-
-        rc = mpool->mpool_register(
-                                   mpool,
-                                   iov.iov_base,
-                                   max_data,
-                                   0,
-                                   &registration);
-
-        if(rc != OMPI_SUCCESS) {
-            MCA_BTL_GM_FRAG_RETURN(btl,frag);
-            return NULL;
-        }
-
-        frag->registration = registration;
-    } 
-
-    /*
-     * if we aren't pinning the data and the requested size is less
-     * than the eager limit pack into a fragment from the eager pool
-    */
-    else 
+    }
 #endif
 
-    if (max_data+reserve <= btl->btl_eager_limit) {
-                                                                                                    
+    if (max_data + reserve <= btl->btl_eager_limit) {
+        /* the data is small enough to fit in the eager frag and
+         * memory is not prepinned */
         MCA_BTL_GM_FRAG_ALLOC_EAGER(btl, frag, rc);
-        if(NULL == frag) {
-            return NULL;
+        if(frag != NULL) {
+            frag->type = MCA_BTL_GM_EAGER;
         }
-                                                                                                    
-        iov.iov_len = max_data;
-        iov.iov_base = (unsigned char*) frag->segment.seg_addr.pval + reserve;
-                                                                                                    
-        rc = ompi_convertor_pack(convertor, &iov, &iov_count, &max_data, &free_after);
-        *size  = max_data;
-        if( rc < 0 ) {
-            MCA_BTL_GM_FRAG_RETURN(btl, frag);
-            return NULL;
-        }
-        frag->segment.seg_len = max_data + reserve;
     }
 
-    /* 
-     * otherwise pack as much data as we can into a fragment
-     * that is the max send size.
-     */
-    else {
-                                                                                                    
+    if(NULL == frag) {
+        /* the data doesn't fit into eager frag or eger frag is
+         * not available */
         MCA_BTL_GM_FRAG_ALLOC_MAX(btl, frag, rc);
         if(NULL == frag) {
             return NULL;
         }
-        if(max_data + reserve > btl->btl_max_send_size){
+        frag->type = MCA_BTL_GM_SEND;
+        if(max_data + reserve > btl->btl_max_send_size) {
             max_data = btl->btl_max_send_size - reserve;
         }
-        iov.iov_len = max_data;
-        iov.iov_base = (unsigned char*) frag->segment.seg_addr.pval + reserve;
-                                                                                                    
-        rc = ompi_convertor_pack(convertor, &iov, &iov_count, &max_data, &free_after);
-        *size  = max_data;
-                                                                                                    
-        if( rc < 0 ) {
-            MCA_BTL_GM_FRAG_RETURN(btl, frag);
-            return NULL;
-        }
-        frag->segment.seg_len = max_data + reserve;
     }
 
+    iov.iov_len = max_data;
+    iov.iov_base = (unsigned char*) frag->segment.seg_addr.pval + reserve;
+    rc = ompi_convertor_pack(convertor, &iov, &iov_count, &max_data);
+    if(rc < 0) {
+        MCA_BTL_GM_FRAG_RETURN(btl, frag);
+        return NULL;
+    }
+    *size  = max_data;
+    frag->segment.seg_len = max_data + reserve;
     frag->base.des_src = &frag->segment;
     frag->base.des_src_cnt = 1;
     frag->base.des_dst = NULL;
     frag->base.des_dst_cnt = 0;
     frag->base.des_flags = 0;
+
     return &frag->base;
 }
 
@@ -397,15 +386,27 @@ mca_btl_base_descriptor_t* mca_btl_gm_prepare_dst(
 #if (OMPI_MCA_BTL_GM_HAVE_RDMA_GET || OMPI_MCA_BTL_GM_HAVE_RDMA_PUT)
     mca_btl_gm_frag_t* frag;
     mca_mpool_base_module_t* mpool = btl->btl_mpool;
-    long lb;
+    ptrdiff_t lb;
     int rc;
 
     MCA_BTL_GM_FRAG_ALLOC_USER(btl, frag, rc);
     if(NULL == frag) {
         return NULL;
     }
-
+    /*
+     * just assign it something.. 
+     *  we will assign the real value in put/get 
+     */
+    frag->type = MCA_BTL_GM_PUT;
+    
     ompi_ddt_type_lb(convertor->pDesc, &lb);
+    /* 
+     *  we don't know that this is for a PUT,
+     *  but it doesn't matter.. they belong 
+     *  on the same list eventually anyway
+     */      
+    frag->type = MCA_BTL_GM_PUT; 
+    
     frag->segment.seg_len = *size;
     frag->segment.seg_addr.pval = convertor->pBaseBuf + lb + convertor->bConverted;
 
@@ -414,29 +415,16 @@ mca_btl_base_descriptor_t* mca_btl_gm_prepare_dst(
     frag->base.des_dst = &frag->segment;
     frag->base.des_dst_cnt = 1;
     frag->base.des_flags = 0;
-    if(NULL != registration) {
-        /* bump reference count as so that the registration
-         * doesn't go away when the operation completes
-         */
-        
-        mpool->mpool_retain(mpool, 
-                           (mca_mpool_base_registration_t*) registration); 
-                
-        frag->registration = registration;
-
-    }  else {
-
-        rc = mpool->mpool_register(
-                                   mpool,
-                                   frag->segment.seg_addr.pval,
-                                   frag->segment.seg_len,
-                                   0,
-                                   &registration);
+    if(NULL == registration) {
+        rc = mpool->mpool_register( mpool,
+				    frag->segment.seg_addr.pval,
+				    frag->segment.seg_len,
+				    0,
+				    &registration );
         if(rc != OMPI_SUCCESS) {
             MCA_BTL_GM_FRAG_RETURN(btl,frag);
             return NULL;
         }
-        
         frag->registration = registration;
     }
     return &frag->base;
@@ -539,7 +527,6 @@ static int mca_btl_gm_send_nl(
     frag->btl = gm_btl;
     frag->endpoint = endpoint; 
     frag->hdr->tag = tag;
-    frag->type = MCA_BTL_GM_SEND;
 
     /* queue the descriptor if there are no send tokens */
     MCA_BTL_GM_ACQUIRE_TOKEN_NL(gm_btl, frag);
@@ -547,35 +534,33 @@ static int mca_btl_gm_send_nl(
     /* post the send descriptor */
     if(frag->base.des_flags & MCA_BTL_DES_FLAGS_PRIORITY &&  
        frag->size == mca_btl_gm_component.gm_eager_frag_size) {
-        gm_send_with_callback(
-            gm_btl->port,
-            frag->hdr,
-            mca_btl_gm_component.gm_eager_frag_size,
-            frag->segment.seg_len + sizeof(mca_btl_base_header_t),
-            GM_HIGH_PRIORITY,
-            endpoint->endpoint_addr.node_id,
-            endpoint->endpoint_addr.port_id,
-            mca_btl_gm_send_callback,
-            frag);
+        gm_send_with_callback( gm_btl->port,
+			       frag->hdr,
+			       mca_btl_gm_component.gm_eager_frag_size,
+			       frag->segment.seg_len + sizeof(mca_btl_base_header_t),
+			       GM_HIGH_PRIORITY,
+			       endpoint->endpoint_addr.node_id,
+			       endpoint->endpoint_addr.port_id,
+			       mca_btl_gm_send_callback,
+			       frag );
     } else {
-        gm_send_with_callback(
-            gm_btl->port,
-            frag->hdr,
-            mca_btl_gm_component.gm_max_frag_size,
-            frag->segment.seg_len + sizeof(mca_btl_base_header_t),
-            GM_LOW_PRIORITY,
-            endpoint->endpoint_addr.node_id,
-            endpoint->endpoint_addr.port_id,
-            mca_btl_gm_send_callback,
-            frag);
+        gm_send_with_callback( gm_btl->port,
+			       frag->hdr,
+			       mca_btl_gm_component.gm_max_frag_size,
+			       frag->segment.seg_len + sizeof(mca_btl_base_header_t),
+			       GM_LOW_PRIORITY,
+			       endpoint->endpoint_addr.node_id,
+			       endpoint->endpoint_addr.port_id,
+			       mca_btl_gm_send_callback,
+			       frag );
     }
-
     if(opal_list_get_size(&gm_btl->gm_repost)) {
         mca_btl_gm_frag_t* frag;
         while(NULL != (frag = (mca_btl_gm_frag_t*)opal_list_remove_first(&gm_btl->gm_repost))) {
             gm_provide_receive_buffer(gm_btl->port, frag->hdr, frag->size, frag->priority);
         }
     }
+
     return OMPI_SUCCESS;
 }
 
@@ -602,8 +587,7 @@ int mca_btl_gm_send(
     frag->btl = gm_btl;
     frag->endpoint = endpoint; 
     frag->hdr->tag = tag;
-    frag->type = MCA_BTL_GM_SEND;
-
+    
     /* queue the descriptor if there are no send tokens */
     OPAL_THREAD_LOCK(&mca_btl_gm_component.gm_lock);
     MCA_BTL_GM_ACQUIRE_TOKEN(gm_btl, frag);
@@ -611,27 +595,25 @@ int mca_btl_gm_send(
     /* post the send descriptor */
     if(frag->base.des_flags & MCA_BTL_DES_FLAGS_PRIORITY &&  
        frag->size == mca_btl_gm_component.gm_eager_frag_size) {
-        gm_send_with_callback(
-            gm_btl->port,
-            frag->hdr,
-            mca_btl_gm_component.gm_eager_frag_size,
-            frag->segment.seg_len + sizeof(mca_btl_base_header_t),
-            GM_HIGH_PRIORITY,
-            endpoint->endpoint_addr.node_id,
-            endpoint->endpoint_addr.port_id,
-            mca_btl_gm_send_callback,
-            frag);
+        gm_send_with_callback( gm_btl->port,
+			       frag->hdr,
+			       mca_btl_gm_component.gm_eager_frag_size,
+			       frag->segment.seg_len + sizeof(mca_btl_base_header_t),
+			       GM_HIGH_PRIORITY,
+			       endpoint->endpoint_addr.node_id,
+			       endpoint->endpoint_addr.port_id,
+			       mca_btl_gm_send_callback,
+			       frag );
     } else {
-        gm_send_with_callback(
-            gm_btl->port,
-            frag->hdr,
-            mca_btl_gm_component.gm_max_frag_size,
-            frag->segment.seg_len + sizeof(mca_btl_base_header_t),
-            GM_LOW_PRIORITY,
-            endpoint->endpoint_addr.node_id,
-            endpoint->endpoint_addr.port_id,
-            mca_btl_gm_send_callback,
-            frag);
+        gm_send_with_callback( gm_btl->port,
+			       frag->hdr,
+			       mca_btl_gm_component.gm_max_frag_size,
+			       frag->segment.seg_len + sizeof(mca_btl_base_header_t),
+			       GM_LOW_PRIORITY,
+			       endpoint->endpoint_addr.node_id,
+			       endpoint->endpoint_addr.port_id,
+			       mca_btl_gm_send_callback,
+			       frag );
     }
 
     if(opal_list_get_size(&gm_btl->gm_repost)) {
@@ -734,7 +716,7 @@ static int mca_btl_gm_put_nl(
     /* post the put descriptor */
     gm_put(gm_btl->port,
         des->des_src->seg_addr.pval,
-        des->des_dst->seg_addr.lval,
+        des->des_dst->seg_addr.pval,
         des->des_src->seg_len,
         GM_LOW_PRIORITY,
         endpoint->endpoint_addr.node_id,
@@ -776,7 +758,7 @@ int mca_btl_gm_put(
     /* post the put descriptor */
     gm_put(gm_btl->port,
         des->des_src->seg_addr.pval,
-        des->des_dst->seg_addr.lval,
+        des->des_dst->seg_addr.pval,
         des->des_src->seg_len,
         GM_LOW_PRIORITY,
         endpoint->endpoint_addr.node_id,
@@ -879,7 +861,7 @@ static int mca_btl_gm_get_nl(
 
     /* post get put descriptor */
     gm_get(gm_btl->port,
-        des->des_dst->seg_addr.lval,
+        des->des_dst->seg_addr.pval,
         des->des_src->seg_addr.pval,
         des->des_src->seg_len,
         GM_LOW_PRIORITY,
@@ -922,7 +904,7 @@ int mca_btl_gm_get(
 
     /* post get put descriptor */
     gm_get(gm_btl->port,
-        des->des_dst->seg_addr.lval,
+        des->des_dst->seg_addr.pval,
         des->des_src->seg_addr.pval,
         des->des_src->seg_len,
         GM_LOW_PRIORITY,
