@@ -9,6 +9,8 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -30,6 +32,8 @@
 #include "ompi/mca/pml/base/pml_base_module_exchange.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "ompi/mca/mpool/base/base.h" 
+#include "ompi/mca/common/mx/common_mx.h"
+
 #include "btl_mx.h"
 #include "btl_mx_frag.h"
 #include "btl_mx_endpoint.h" 
@@ -45,7 +49,7 @@ mca_btl_mx_component_t mca_btl_mx_component = {
             /* Indicate that we are a pml v1.0.0 component (which also implies a
                specific MCA version) */
 
-            MCA_BTL_BASE_VERSION_1_0_0,
+            MCA_BTL_BASE_VERSION_1_0_1,
 
             "mx", /* MCA component name */
             OMPI_MAJOR_VERSION,  /* MCA component major version */
@@ -75,6 +79,8 @@ mca_btl_mx_component_t mca_btl_mx_component = {
 
 int mca_btl_mx_component_open(void)
 {
+    int tmp;
+
     /* initialize state */
     mca_btl_mx_component.mx_num_btls = 0;
     mca_btl_mx_component.mx_btls = NULL;
@@ -122,25 +128,30 @@ int mca_btl_mx_component_open(void)
 
     mca_base_param_reg_int( (mca_base_component_t*)&mca_btl_mx_component, "exclusivity",
                             "Priority compared with the others devices (used only when several devices are available",
-                            false, false, 50, (int*)&mca_btl_mx_module.super.btl_exclusivity );
+                            false, false, 50, (int*) &mca_btl_mx_module.super.btl_exclusivity );
     mca_base_param_reg_int( (mca_base_component_t*)&mca_btl_mx_component, "first_frag_size",
                             "Size of the first fragment for the rendez-vous protocol over MX",
-                            true, true, 16*1024 - 20, (int*)&mca_btl_mx_module.super.btl_eager_limit );
+                            true, true, 16*1024 - 20, &tmp);
+    mca_btl_mx_module.super.btl_eager_limit = tmp;
     mca_base_param_reg_int( (mca_base_component_t*)&mca_btl_mx_component, "min_send_size",
                             "Minimum send fragment size ...",
-                            false, false, 32*1024 - 40, (int*)&mca_btl_mx_module.super.btl_min_send_size );
+                            false, false, 32*1024 - 40, &tmp);
+    mca_btl_mx_module.super.btl_min_send_size = tmp;
     mca_base_param_reg_int( (mca_base_component_t*)&mca_btl_mx_component, "max_send_size",
                             "Maximum send fragment size withour RDMA ...",
-                            false, false, 128*1024, (int*)&mca_btl_mx_module.super.btl_max_send_size );
+                            false, false, 128*1024, &tmp);
+    mca_btl_mx_module.super.btl_max_send_size = tmp;
     mca_base_param_reg_int( (mca_base_component_t*)&mca_btl_mx_component, "min_rdma_size",
                             "Minimum size of fragment for the RDMA protocol",
-                            false, false, 1024*1024, (int*)&mca_btl_mx_module.super.btl_min_rdma_size );
+                            false, false, 1024*1024, &tmp);
+    mca_btl_mx_module.super.btl_min_rdma_size = tmp;
     mca_base_param_reg_int( (mca_base_component_t*)&mca_btl_mx_component, "max_rdma_size",
                             "Maximum size of fragment for the RDMA protocol",
-                            false, false, 1024*1024, (int*)&mca_btl_mx_module.super.btl_max_rdma_size );
+                            false, false, 1024*1024, &tmp);
+    mca_btl_mx_module.super.btl_max_rdma_size = tmp;
     mca_base_param_reg_int( (mca_base_component_t*)&mca_btl_mx_component, "flags",
                             "Flags to activate/deactivate the RDMA",
-                            true, false, MCA_BTL_FLAGS_PUT, (int*)&mca_btl_mx_module.super.btl_max_rdma_size );
+                            true, false, MCA_BTL_FLAGS_PUT, (int*)&mca_btl_mx_module.super.btl_flags );
 
     return OMPI_SUCCESS;
 }
@@ -153,32 +164,10 @@ int mca_btl_mx_component_close(void)
 {
     if( NULL == mca_btl_mx_component.mx_btls )
         return OMPI_SUCCESS;
-
-    mx_finalize();
-#if OMPI_ENABLE_DEBUG
-    if (mca_btl_mx_component.mx_send_eager_frags.fl_num_allocated &&
-        mca_btl_mx_component.mx_send_eager_frags.fl_num_allocated !=
-        mca_btl_mx_component.mx_send_eager_frags.super.opal_list_length) {
-        opal_output(0, "mx send eager frags: %d allocated %d returned\n",
-            mca_btl_mx_component.mx_send_eager_frags.fl_num_allocated,
-            mca_btl_mx_component.mx_send_eager_frags.super.opal_list_length);
+    
+    if(OMPI_SUCCESS != ompi_common_mx_finalize()) { 
+        return OMPI_ERROR;
     }
-    if (mca_btl_mx_component.mx_send_user_frags.fl_num_allocated &&
-        mca_btl_mx_component.mx_send_user_frags.fl_num_allocated !=
-        mca_btl_mx_component.mx_send_user_frags.super.opal_list_length) {
-        opal_output(0, "mx send user frags: %d allocated %d returned\n",
-            mca_btl_mx_component.mx_send_user_frags.fl_num_allocated,
-            mca_btl_mx_component.mx_send_user_frags.super.opal_list_length);
-    }
-    /* allow for pre-posted receives */
-    if (mca_btl_mx_component.mx_recv_frags.fl_num_allocated &&
-        mca_btl_mx_component.mx_recv_frags.fl_num_allocated - 3 >
-        mca_btl_mx_component.mx_recv_frags.super.opal_list_length) {
-        opal_output(0, "mx recv frags: %d allocated %d returned\n",
-            mca_btl_mx_component.mx_recv_frags.fl_num_allocated,
-            mca_btl_mx_component.mx_recv_frags.super.opal_list_length);
-    }
-#endif
 
     /* release resources */
     OBJ_DESTRUCT(&mca_btl_mx_component.mx_send_eager_frags);
@@ -220,6 +209,7 @@ static mca_btl_mx_module_t* mca_btl_mx_create(uint64_t addr)
                                NULL, 0, &mx_btl->mx_endpoint);
     if(status != MX_SUCCESS) {
         opal_output(0, "mca_btl_mx_init: mx_open_endpoint() failed with status=%d\n", status);
+        mx_btl->mx_endpoint = NULL;
         mca_btl_mx_finalize( &mx_btl->super );
         return NULL;
     }
@@ -270,14 +260,12 @@ mca_btl_base_module_t** mca_btl_mx_component_init(int *num_btl_modules,
     /*mx_set_error_handler(MX_ERRORS_ARE_FATAL);*/
 
     /* First check if MX is available ... */
-    if( MX_SUCCESS != (status = mx_init()) ) {
-        opal_output( 0, "mca_btl_mx_component_init: mx_init() failed with status = %d (%s)\n",
-                     status, mx_strerror(status) );
+    if(OMPI_SUCCESS!=ompi_common_mx_initialize()) { 
         mca_pml_base_modex_send(&mca_btl_mx_component.super.btl_version, 
                                 NULL, 0);
         return NULL;
     }
-
+        
     /* initialize objects */
     OBJ_CONSTRUCT(&mca_btl_mx_component.mx_send_eager_frags, ompi_free_list_t);
     OBJ_CONSTRUCT(&mca_btl_mx_component.mx_send_user_frags, ompi_free_list_t);
@@ -369,6 +357,10 @@ mca_btl_base_module_t** mca_btl_mx_component_init(int *num_btl_modules,
             OBJ_RELEASE( btl );
             continue;
         }
+
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+        BTL_MX_ADDR_HTON(mx_addrs[i]);
+#endif
         mca_btl_mx_component.mx_btls[count++] = btl;
     }
     size = sizeof(mca_btl_mx_addr_t) * count;
