@@ -31,7 +31,6 @@
 #include "ompi/datatype/datatype.h"
 
 #include "btl_portals.h"
-#include "btl_portals_compat.h"
 #include "btl_portals_endpoint.h"
 #include "btl_portals_recv.h"
 #include "btl_portals_frag.h"
@@ -65,7 +64,9 @@ mca_btl_portals_module_t mca_btl_portals_module = {
         mca_btl_portals_send,
         mca_btl_portals_put,
         mca_btl_portals_get,
-        mca_btl_base_dump
+        mca_btl_base_dump,
+        NULL, /* mpool */
+        NULL /* register error */
     },
 };
 
@@ -89,13 +90,14 @@ mca_btl_portals_add_procs(struct mca_btl_base_module_t* btl_base,
                         "Adding %d procs (%d)", nprocs,
                         mca_btl_portals_module.portals_num_procs);
 
-    /* make sure our environment is fully initialized.  At end of this
-       call, we have a working network handle on our module and
-       portals_procs will have the portals process identifier for each
-       proc (ordered, in theory) */
-    ret = mca_btl_portals_add_procs_compat(&mca_btl_portals_module,
-                                           nprocs, procs, 
-                                           &portals_procs);
+    /* if we havne't already, get our network handle */
+    if (mca_btl_portals_module.portals_ni_h == PTL_INVALID_HANDLE) {
+        ret = ompi_common_portals_ni_initialize(&mca_btl_portals_module.portals_ni_h);
+        if (OMPI_SUCCESS != ret) return ret;
+    }
+
+    portals_procs = malloc(nprocs * sizeof(ptl_process_id_t));
+    ret = ompi_common_portals_get_procs(nprocs, procs, portals_procs);
     if (OMPI_SUCCESS != ret) return ret;
 
     if (0 == mca_btl_portals_module.portals_num_procs) {
@@ -106,6 +108,11 @@ mca_btl_portals_add_procs(struct mca_btl_base_module_t* btl_base,
     for (i= 0; i < nprocs ; ++i) {
         curr_proc = procs[i];
 
+        /* portals doesn't support heterogeneous yet... */
+        if (ompi_proc_local()->proc_arch != curr_proc->proc_arch) {
+            continue;
+        }
+
         peers[i] = malloc(sizeof(mca_btl_base_endpoint_t));
         if (NULL == peers[i]) return OMPI_ERROR;
         *((mca_btl_base_endpoint_t*) peers[i]) = portals_procs[i];
@@ -302,7 +309,6 @@ mca_btl_portals_prepare_src(struct mca_btl_base_module_t* btl_base,
     size_t max_data = *size;
     struct iovec iov;
     uint32_t iov_count = 1;
-    int32_t free_after;
     int ret;
 
     assert(&mca_btl_portals_module == (mca_btl_portals_module_t*) btl_base);
@@ -321,7 +327,7 @@ mca_btl_portals_prepare_src(struct mca_btl_base_module_t* btl_base,
         iov.iov_len = max_data;
         iov.iov_base = (unsigned char*) frag->segments[0].seg_addr.pval + reserve;
         ret = ompi_convertor_pack(convertor, &iov, &iov_count, 
-                                  &max_data, &free_after);
+                                  &max_data );
         *size  = max_data;
         if ( ret < 0 ) {
             return NULL;
@@ -350,8 +356,7 @@ mca_btl_portals_prepare_src(struct mca_btl_base_module_t* btl_base,
         iov.iov_len = max_data;
         iov.iov_base = NULL;
 
-        ompi_convertor_pack(convertor, &iov, &iov_count, &max_data, 
-                            &free_after);
+        ompi_convertor_pack(convertor, &iov, &iov_count, &max_data );
 
         frag->segments[0].seg_len = max_data;
         frag->segments[0].seg_addr.pval = iov.iov_base;
@@ -425,7 +430,7 @@ mca_btl_portals_prepare_dst(struct mca_btl_base_module_t* btl_base,
     ptl_md_t md;
     ptl_handle_me_t me_h;
     int ret;
-    long lb;
+    ptrdiff_t lb;
 
     assert(&mca_btl_portals_module == (mca_btl_portals_module_t*) btl_base);
 
@@ -521,12 +526,16 @@ mca_btl_portals_finalize(struct mca_btl_base_module_t *btl_base)
         for (i = 0 ; i < OMPI_BTL_PORTALS_EQ_SIZE ; ++i) {
             int ptl_ret = PtlEQFree(mca_btl_portals_module.portals_eq_handles[i]);
             if (PTL_OK != ptl_ret) {
+#if OMPI_PORTALS_CRAYXT3
                 if (i != OMPI_BTL_PORTALS_EQ_SEND && PTL_EQ_IN_USE != ptl_ret) {
                     /* The PML isn't great about cleaning up after itself.
                        Ignore related errors. */
+#endif
                     opal_output(mca_btl_portals_component.portals_output,
                             "Error freeing EQ %d: %d", i, ptl_ret);
+#if OMPI_PORTALS_CRAYXT3
                 }
+#endif
             }
         }
 
@@ -538,14 +547,8 @@ mca_btl_portals_finalize(struct mca_btl_base_module_t *btl_base)
     OBJ_DESTRUCT(&mca_btl_portals_module.portals_frag_max);
     OBJ_DESTRUCT(&mca_btl_portals_module.portals_frag_user);
 
-    if (PTL_INVALID_HANDLE != mca_btl_portals_module.portals_ni_h) {
-        ret = PtlNIFini(mca_btl_portals_module.portals_ni_h);
-        if (PTL_OK != ret) {
-            opal_output_verbose(20, mca_btl_portals_component.portals_output,
-                                "PtlNIFini returned %d", ret);
-            return OMPI_ERROR;
-        }
-    }
+    ompi_common_portals_ni_finalize();
+    ompi_common_portals_finalize();
 
     opal_output_verbose(20, mca_btl_portals_component.portals_output,
                         "successfully finalized module");
