@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -106,7 +106,9 @@ mca_btl_sm_t mca_btl_sm[2] = {
         mca_btl_sm_send, 
         NULL,  /* put */
         NULL,  /* get */
-        mca_btl_base_dump
+        mca_btl_base_dump,
+        NULL, /* mpool */
+        mca_btl_sm_register_error_cb /* register error */
         }
     },
     {
@@ -132,7 +134,9 @@ mca_btl_sm_t mca_btl_sm[2] = {
         mca_btl_sm_send,  
         NULL, /* put function */
         NULL, /* get function */
-        mca_btl_base_dump
+        mca_btl_base_dump,
+        NULL, /* mpool */
+        mca_btl_sm_register_error_cb /* register error */
         }
     }
 };
@@ -159,6 +163,7 @@ int mca_btl_sm_add_procs_same_base_addr(
     ssize_t diff;
     volatile char **tmp_ptr;
     volatile int *tmp_int_ptr;
+    bool have_connected_peer = false;
 
     /* initializion */
     for(i=0 ; i < nprocs ; i++ ) {
@@ -197,21 +202,25 @@ int mca_btl_sm_add_procs_same_base_addr(
 #endif
         struct mca_btl_base_endpoint_t *peer;
 
-        /* check to see if this is me */
-        if( my_proc == procs[proc] ) {
-            mca_btl_sm_component.my_smp_rank = n_local_procs;
-        }
-
         /* check to see if this proc can be reached via shmem (i.e.,
            if they're on my local host and in my job) */
-        else if (procs[proc]->proc_name.jobid != my_proc->proc_name.jobid ||
+        if (procs[proc]->proc_name.jobid != my_proc->proc_name.jobid ||
                  0 == (procs[proc]->proc_flags & OMPI_PROC_FLAG_LOCAL)) {
             continue;
         }
 
         /* If we got here, the proc is reachable via sm.  So
            initialize the peers information */
-        peer = peers[proc] = malloc(sizeof(struct mca_btl_base_endpoint_t));
+
+        /* check to see if this is me */
+        if( my_proc == procs[proc] ) {
+            mca_btl_sm_component.my_smp_rank = n_local_procs;
+        } else {
+            /* we have someone to talk to */
+            have_connected_peer = true;
+        }
+
+        peer = peers[proc] = (struct mca_btl_base_endpoint_t*)malloc(sizeof(struct mca_btl_base_endpoint_t));
         if( NULL == peer ){
             return_code=OMPI_ERR_OUT_OF_RESOURCE;
             goto CLEANUP;
@@ -220,7 +229,7 @@ int mca_btl_sm_add_procs_same_base_addr(
             mca_btl_sm_component.num_smp_procs;
         
 #if OMPI_ENABLE_PROGRESS_THREADS == 1
-        sprintf(path, "%s/sm_fifo.%lu", orte_process_info.job_session_dir, 
+        sprintf(path, "%s"OPAL_PATH_SEP"sm_fifo.%lu", orte_process_info.job_session_dir, 
                 (unsigned long)procs[proc]->proc_name.vpid);
         peer->fifo_fd = open(path, O_WRONLY);
         if(peer->fifo_fd < 0) {
@@ -232,7 +241,8 @@ int mca_btl_sm_add_procs_same_base_addr(
         mca_btl_sm_component.sm_proc_connect[proc]=SM_CONNECTED;
     }
 
-    if( n_local_procs == 0) {
+    /* jump out if there's not someone we can talk to */
+    if (!have_connected_peer) {
         return_code = OMPI_SUCCESS;
         goto CLEANUP;
     }
@@ -252,7 +262,7 @@ int mca_btl_sm_add_procs_same_base_addr(
            goto CLEANUP;
         }
         mca_btl_sm_component.sm_mpool_base =
-            mca_btl_sm_component.sm_mpool->mpool_base(mca_btl_sm_component.sm_mpool_base);
+            mca_btl_sm_component.sm_mpool->mpool_base((mca_mpool_base_module_t*)mca_btl_sm_component.sm_mpool_base);
     }
 
     /* make sure that my_smp_rank has been defined */
@@ -315,7 +325,7 @@ int mca_btl_sm_add_procs_same_base_addr(
     if ( !mca_btl_sm[0].btl_inited ) {
         /* set file name */
         len=asprintf(&(mca_btl_sm_component.sm_resouce_ctl_file),
-                "%s/shared_mem_btl_module.%s",orte_process_info.job_session_dir,
+                "%s"OPAL_PATH_SEP"shared_mem_btl_module.%s",orte_process_info.job_session_dir,
                 orte_system_info.nodename);
         if( 0 > len ) {
             goto CLEANUP;
@@ -359,7 +369,7 @@ int mca_btl_sm_add_procs_same_base_addr(
             /* allocate array of ompi_fifo_t* elements -
              * offset relative to base segement is stored, so that
              * this can be used by other procs */
-            mca_btl_sm_component.sm_ctl_header->fifo=
+            mca_btl_sm_component.sm_ctl_header->fifo= (volatile ompi_fifo_t**)
                 mca_btl_sm_component.sm_mpool->mpool_alloc
                 (mca_btl_sm_component.sm_mpool, n_to_allocate*sizeof(ompi_fifo_t *),
                  CACHE_LINE_SIZE, 0, NULL);
@@ -449,18 +459,18 @@ int mca_btl_sm_add_procs_same_base_addr(
             ( (char *)(mca_btl_sm_component.sm_ctl_header->segment_header.
               base_shared_mem_segment)  +
 		      (long )(mca_btl_sm_component.sm_mpool->mpool_base(mca_btl_sm_component.sm_mpool)) );
-        tmp_ptr[mca_btl_sm_component.my_smp_rank]=
+        tmp_ptr[mca_btl_sm_component.my_smp_rank]=(char*)
             mca_btl_sm_component.sm_mpool->mpool_base(mca_btl_sm_component.sm_mpool);
         /* memory barrier to ensure this flag is set before other
          *  flags are set */
         opal_atomic_mb();
 
-	/* Set my flag to 1 (convert from relative address first) */
+		/* Set my flag to 1 (convert from relative address first) */
         tmp_int_ptr=(volatile int *)
             ( ((char *) mca_btl_sm_component.sm_ctl_header->segment_header.
                base_shared_mem_flags) +
               ((long) mca_btl_sm_component.sm_mpool_base));
-	tmp_int_ptr[mca_btl_sm_component.my_smp_rank]=1;
+		tmp_int_ptr[mca_btl_sm_component.my_smp_rank]=1;
 
         /*
          * initialize the array of fifo's "owned" by this process
@@ -478,10 +488,27 @@ int mca_btl_sm_add_procs_same_base_addr(
         }
 
         for( j=0 ; j < n_to_allocate ; j++ ) {
-            my_fifos[j].head=OMPI_CB_FREE;
-            my_fifos[j].tail=OMPI_CB_FREE;
-            opal_atomic_unlock(&(my_fifos[j].head_lock));
-            opal_atomic_unlock(&(my_fifos[j].tail_lock));
+            char *buf;
+            my_fifos[j].head = (ompi_cb_fifo_wrapper_t*)OMPI_CB_FREE;
+            my_fifos[j].tail = (ompi_cb_fifo_wrapper_t*)OMPI_CB_FREE;
+	    if(opal_using_threads()) {
+                /* allocate head and tail locks on different cache lines */
+                buf = (char*)mca_btl_sm_component.sm_mpool->mpool_alloc(
+                    mca_btl_sm_component.sm_mpool,
+                    CACHE_LINE_SIZE*2, CACHE_LINE_SIZE, 0, NULL);
+                if(NULL == buf) {
+                    return_code = OMPI_ERR_OUT_OF_RESOURCE;
+                    goto CLEANUP;
+                }
+            	my_fifos[j].head_lock = (opal_atomic_lock_t*)buf;
+            	my_fifos[j].tail_lock = (opal_atomic_lock_t*)(buf +
+				CACHE_LINE_SIZE);
+            	opal_atomic_init(my_fifos[j].head_lock, OPAL_ATOMIC_UNLOCKED);
+            	opal_atomic_init(my_fifos[j].tail_lock, OPAL_ATOMIC_UNLOCKED);
+	    } else {
+		    my_fifos[j].head_lock = NULL;
+		    my_fifos[j].tail_lock = NULL;
+	    }
         }
         fifo_tmp=(ompi_fifo_t * volatile *)
                 ( (char *)(mca_btl_sm_component.sm_ctl_header->fifo) +
@@ -522,6 +549,22 @@ int mca_btl_sm_add_procs_same_base_addr(
         mca_btl_sm_component.fifo[j]=
             ( ompi_fifo_t *)( (char *)fifo_tmp[j]+diff);
         mca_btl_sm_component.sm_offset[j] = diff;
+
+    }
+
+    for( j=mca_btl_sm_component.num_smp_procs ; j <
+            mca_btl_sm_component.num_smp_procs+n_local_procs ; j++ ) {
+        if(j == mca_btl_sm_component.my_smp_rank)
+            continue;
+        return_code = ompi_fifo_init(mca_btl_sm_component.size_of_cb_queue,
+                mca_btl_sm_component.cb_lazy_free_freq,
+                0,0,0,
+                &mca_btl_sm_component.fifo[j][mca_btl_sm_component.my_smp_rank],
+                mca_btl_sm_component.sm_offset[j],
+                mca_btl_sm_component.sm_mpool);
+
+        if(return_code != OMPI_SUCCESS)
+            goto CLEANUP;
     }
 
     /* initialize some of the free-lists */
@@ -618,6 +661,13 @@ int mca_btl_sm_add_procs_same_base_addr(
         }
     }
 
+    /* make sure we have enough eager fragmnents for each process */
+    return_code = ompi_free_list_resize(&mca_btl_sm_component.sm_frags1,
+                                        (mca_btl_sm_component.num_smp_procs + n_local_procs) * 2);
+    if (OMPI_SUCCESS != return_code) {
+        goto CLEANUP;
+    }
+
     /* update the local smp process count */
     mca_btl_sm_component.num_smp_procs+=n_local_procs;
 
@@ -672,7 +722,7 @@ int mca_btl_sm_add_procs(
             }
 
             /* initialize the peers information */
-            peers[proc]=malloc(sizeof(struct mca_btl_base_endpoint_t));
+            peers[proc] = (struct mca_btl_base_endpoint_t*)malloc(sizeof(struct mca_btl_base_endpoint_t));
             if( NULL == peers[proc] ){
                 return_code=OMPI_ERR_OUT_OF_RESOURCE;
                 goto CLEANUP;
@@ -753,6 +803,17 @@ int mca_btl_sm_register(
     return OMPI_SUCCESS;
 }
                                                                                                                  
+/* 
+ * Register callback function for error handling..
+ */
+int mca_btl_sm_register_error_cb(
+        struct mca_btl_base_module_t* btl,
+        mca_btl_base_module_error_cb_fn_t cbfunc)
+{
+    mca_btl_sm_t *sm_btl = (mca_btl_sm_t *)btl;
+    sm_btl->error_cb = cbfunc;
+    return OMPI_SUCCESS;
+}
 
 /**
  * Allocate a segment.
@@ -773,10 +834,14 @@ extern mca_btl_base_descriptor_t* mca_btl_sm_alloc(
     } else {
         return NULL;
     }
-    frag->segment.seg_len = size;
+
+    if (frag != NULL) {
+        frag->segment.seg_len = size;
+    }
+
     return (mca_btl_base_descriptor_t*)frag;
 }
-                                                                                                                   
+
 /**
  * Return a segment allocated by this BTL.
  *
@@ -811,7 +876,6 @@ struct mca_btl_base_descriptor_t* mca_btl_sm_prepare_src(
     struct iovec iov;
     uint32_t iov_count = 1;
     size_t max_data = *size;
-    int32_t free_after;
     int rc;
 
     MCA_BTL_SM_FRAG_ALLOC2(frag, rc);
@@ -823,9 +887,9 @@ struct mca_btl_base_descriptor_t* mca_btl_sm_prepare_src(
         max_data = frag->size - reserve;
     } 
     iov.iov_len = max_data;
-    iov.iov_base = (void*)(((unsigned char*)(frag+1)) + reserve);
+    iov.iov_base = (IOVBASE_TYPE*)(((unsigned char*)(frag+1)) + reserve);
 
-    rc = ompi_convertor_pack(convertor, &iov, &iov_count, &max_data, &free_after);
+    rc = ompi_convertor_pack(convertor, &iov, &iov_count, &max_data );
     if(rc < 0) {
         MCA_BTL_SM_FRAG_RETURN(frag);
         return NULL;
