@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -49,6 +49,7 @@
 #include "ompi/mca/pml/base/pml_base_module_exchange.h"
 #include "ompi/mca/mpool/base/base.h"
 #include "ompi/mca/common/sm/common_sm_mmap.h"
+#include "ompi/mca/btl/base/btl_base_error.h"
 #include "btl_sm.h"
 #include "btl_sm_frag.h"
 #include "btl_sm_fifo.h"
@@ -66,7 +67,7 @@ mca_btl_sm_component_t mca_btl_sm_component = {
         {
             /* Indicate that we are a pml v1.0.0 component (which also implies a
                specific MCA version) */
-            MCA_BTL_BASE_VERSION_1_0_0,
+            MCA_BTL_BASE_VERSION_1_0_1,
             "sm", /* MCA component name */
             OMPI_MAJOR_VERSION,  /* MCA component major version */
             OMPI_MINOR_VERSION,  /* MCA component minor version */
@@ -125,7 +126,11 @@ int mca_btl_sm_component_open(void)
     mca_btl_sm_component.sm_free_list_max =
         mca_btl_sm_param_register_int("free_list_max", -1);
     mca_btl_sm_component.sm_free_list_inc =
-        mca_btl_sm_param_register_int("free_list_inc", 256);
+        mca_btl_sm_param_register_int("free_list_inc", 64);
+    mca_btl_sm_component.sm_exclusivity =
+        mca_btl_sm_param_register_int("exclusivity", MCA_BTL_EXCLUSIVITY_HIGH-1);
+    mca_btl_sm_component.sm_latency =
+        mca_btl_sm_param_register_int("latency", 100);
     mca_btl_sm_component.sm_max_procs =
         mca_btl_sm_param_register_int("max_procs", -1);
     mca_btl_sm_component.sm_extra_procs =
@@ -133,9 +138,9 @@ int mca_btl_sm_component_open(void)
     mca_btl_sm_component.sm_mpool_name =
         mca_btl_sm_param_register_string("mpool", "sm");
     mca_btl_sm_component.eager_limit =
-        mca_btl_sm_param_register_int("eager_limit", 1024);
+        mca_btl_sm_param_register_int("eager_limit", 4*1024);
     mca_btl_sm_component.max_frag_size =
-        mca_btl_sm_param_register_int("max_frag_size", 8*1024);
+        mca_btl_sm_param_register_int("max_frag_size", 32*1024);
     mca_btl_sm_component.size_of_cb_queue =
         mca_btl_sm_param_register_int("size_of_cb_queue", 128);
     mca_btl_sm_component.cb_lazy_free_freq =
@@ -172,16 +177,21 @@ int mca_btl_sm_component_open(void)
 
 int mca_btl_sm_component_close(void)
 {
-    int return_value=OMPI_SUCCESS;
+    int return_value = OMPI_SUCCESS;
 
     OBJ_DESTRUCT(&mca_btl_sm_component.sm_lock);
-    OBJ_DESTRUCT(&mca_btl_sm_component.sm_frags1);
-    OBJ_DESTRUCT(&mca_btl_sm_component.sm_frags2);
+    /**
+     * We don't have to destroy the fragment lists. They are allocated
+     * directly into the mmapped file, they will auto-magically dissapear
+     * when the file get unmapped.
+     */
+    /*OBJ_DESTRUCT(&mca_btl_sm_component.sm_frags1);*/
+    /*OBJ_DESTRUCT(&mca_btl_sm_component.sm_frags2);*/
 
     /* unmap the shared memory control structure */
     if(mca_btl_sm_component.mmap_file != NULL) {
         return_value = mca_common_sm_mmap_fini( mca_btl_sm_component.mmap_file );
-        if(-1 == return_value) {
+        if( OMPI_SUCCESS != return_value ) {
             return_value=OMPI_ERROR;
             opal_output(0," munmap failed :: file - %s :: errno - %d \n",
                     mca_btl_sm_component.mmap_file->map_addr,
@@ -194,6 +204,7 @@ int mca_btl_sm_component_close(void)
          * to call this, so that in an abnormal termination scenario,
          * this file will still get cleaned up */
         unlink(mca_btl_sm_component.mmap_file->map_path);
+        OBJ_RELEASE(mca_btl_sm_component.mmap_file);
     }
 
 #if OMPI_ENABLE_PROGRESS_THREADS == 1
@@ -240,7 +251,7 @@ mca_btl_base_module_t** mca_btl_sm_component_init(
 #if OMPI_ENABLE_PROGRESS_THREADS == 1
     /* create a named pipe to receive events  */
     sprintf( mca_btl_sm_component.sm_fifo_path, 
-             "%s/sm_fifo.%lu", orte_process_info.job_session_dir,
+             "%s"OPAL_PATH_SEP"sm_fifo.%lu", orte_process_info.job_session_dir,
              (unsigned long)orte_process_info.my_name->vpid );
     if(mkfifo(mca_btl_sm_component.sm_fifo_path, 0660) < 0) {
         opal_output(0, "mca_btl_sm_component_init: mkfifo failed with errno=%d\n",errno);
@@ -260,7 +271,7 @@ mca_btl_base_module_t** mca_btl_sm_component_init(
 
     /* allocate the Shared Memory PTL */
     *num_ptls = 2;
-    ptls = malloc((*num_ptls)*sizeof(mca_btl_base_module_t*));
+    ptls = (mca_btl_base_module_t**)malloc((*num_ptls)*sizeof(mca_btl_base_module_t*));
     if (NULL == ptls) {
         return NULL;
     }
@@ -276,16 +287,17 @@ mca_btl_base_module_t** mca_btl_sm_component_init(
         mca_btl_sm[i].super.btl_max_send_size=mca_btl_sm_component.max_frag_size;
         mca_btl_sm[i].super.btl_min_rdma_size=mca_btl_sm_component.max_frag_size;
         mca_btl_sm[i].super.btl_max_rdma_size=mca_btl_sm_component.max_frag_size;
-        /* Because of the strange behavior of the qsort function on some OSes
-         * when it has to order items with the same value,
-         * we have to make sure that the add_procs functions are called in the
-         * correct order (ie. mca_btl_sm_add_procs_same_base_addr before
-         * mca_btl_sm_add_procs) or we will face segfault as only the first
-         * allocate memory for the mca_btl_sm_component.sm_proc_connect array.
+        /* The order in which the SM modules are initialized is important as only
+         * the first one (the one using the mca_btl_sm_add_procs_same_base_addr)
+         * will setup all the memory for the internal structures (sm_proc_connect).
+         * Therefore, the order in which the two SM module will be after the 
+         * selection is important. We have to make sure they get sorted in the
+         * correct order. The simplest way is to force the exclusivity of the
+         * second module to something lower than the exclusivity of the first one.
          */
-        mca_btl_sm[i].super.btl_exclusivity=MCA_BTL_EXCLUSIVITY_HIGH-1-i;  /* always use this btl */
-        mca_btl_sm[i].super.btl_latency=100;      /* lowest latency */
-        mca_btl_sm[i].super.btl_bandwidth=900; /* not really used now since exclusivity is set to the highest value */
+        mca_btl_sm[i].super.btl_exclusivity = mca_btl_sm_component.sm_exclusivity - i;
+        mca_btl_sm[i].super.btl_latency     = mca_btl_sm_component.sm_latency; /* lowest latency */
+        mca_btl_sm[i].super.btl_bandwidth   = 900; /* not really used now since exclusivity is set to the highest value */
     }
 
     /* initialize some PTL data */
@@ -332,7 +344,7 @@ int mca_btl_sm_component_progress(void)
     ompi_fifo_t *fifo = NULL;
     int my_smp_rank=mca_btl_sm_component.my_smp_rank;
     int proc;
-    int rc = 0;
+    int rc = 0, btl = 0;
 
     /* send progress is made by the PML */
 
@@ -348,7 +360,7 @@ int mca_btl_sm_component_progress(void)
             ; proc++ ) 
     {
         peer_smp_rank= mca_btl_sm_component.list_smp_procs_same_base_addr[proc];
-        fifo=&(mca_btl_sm_component.fifo[peer_smp_rank][my_smp_rank]);
+        fifo=&(mca_btl_sm_component.fifo[my_smp_rank][peer_smp_rank]);
 
         /* if fifo is not yet setup - continue - not data has been sent*/
         if(OMPI_CB_FREE == fifo->tail){
@@ -357,7 +369,7 @@ int mca_btl_sm_component_progress(void)
 
         /* aquire thread lock */
         if( opal_using_threads() ) {
-            opal_atomic_lock( &(fifo->tail_lock) );
+            opal_atomic_lock(fifo->tail_lock);
         }
 
         /* get pointer - pass in offset to change queue pointer
@@ -365,12 +377,11 @@ int mca_btl_sm_component_progress(void)
          * that we have the same base address as the sender, so no
          * translation is necessary when accessing the fifo.  Hence,
          * we use the _same_base_addr varient. */
-        frag = (mca_btl_sm_frag_t *)
-          ompi_fifo_read_from_tail_same_base_addr( fifo );
+        frag = (mca_btl_sm_frag_t *)ompi_fifo_read_from_tail(fifo);
 
         /* release thread lock */
         if( opal_using_threads() ) {
-            opal_atomic_unlock(&(fifo->tail_lock));
+            opal_atomic_unlock(fifo->tail_lock);
         }
 
         if( OMPI_CB_FREE == frag ) {
@@ -394,7 +405,7 @@ int mca_btl_sm_component_progress(void)
                 MCA_BTL_SM_FIFO_WRITE( mca_btl_sm_component.sm_peers[peer_smp_rank],
                                        my_smp_rank, peer_smp_rank, frag, rc );
                 if(OMPI_SUCCESS != rc)
-                    return rc;
+                    goto err;
                 break;
             }
             default:
@@ -405,7 +416,7 @@ int mca_btl_sm_component_progress(void)
                 MCA_BTL_SM_FIFO_WRITE( mca_btl_sm_component.sm_peers[peer_smp_rank],
                                        my_smp_rank, peer_smp_rank, frag, rc );
                 if(OMPI_SUCCESS != rc)
-                    return rc;
+                    goto err;
                 break;
             }
         }
@@ -413,13 +424,14 @@ int mca_btl_sm_component_progress(void)
     }  /* end peer_local_smp_rank loop */
 
 
+    btl = 1;
     /* loop over fifo's - procs with different base shared memory 
      * virtual address as this process */
     for( proc=0 ; proc < mca_btl_sm_component.num_smp_procs_different_base_addr
             ; proc++ ) 
     {
         peer_smp_rank= mca_btl_sm_component.list_smp_procs_different_base_addr[proc];
-        fifo=&(mca_btl_sm_component.fifo[peer_smp_rank][my_smp_rank]);
+        fifo=&(mca_btl_sm_component.fifo[my_smp_rank][peer_smp_rank]);
 
         /* if fifo is not yet setup - continue - not data has been sent*/
         if(OMPI_CB_FREE == fifo->tail){
@@ -428,7 +440,7 @@ int mca_btl_sm_component_progress(void)
 
         /* aquire thread lock */
         if( opal_using_threads() ) {
-            opal_atomic_lock(&(fifo->tail_lock));
+            opal_atomic_lock(fifo->tail_lock);
         }
 
         /* get pointer - pass in offset to change queue pointer
@@ -437,19 +449,19 @@ int mca_btl_sm_component_progress(void)
          * translate every access into the fifo to be relevant to our
          * memory space.  Hence, we do *not* use the _same_base_addr
          * variant. */
-        frag=(mca_btl_sm_frag_t *)ompi_fifo_read_from_tail( fifo,
-                mca_btl_sm_component.sm_offset[peer_smp_rank]);
+        frag = (mca_btl_sm_frag_t *)ompi_fifo_read_from_tail( fifo );
+        
         if( OMPI_CB_FREE == frag ) {
             /* release thread lock */
             if( opal_using_threads() ) {
-                opal_atomic_unlock(&(fifo->tail_lock));
+                opal_atomic_unlock(fifo->tail_lock);
             }
             continue;
         }
 
         /* release thread lock */
         if( opal_using_threads() ) {
-            opal_atomic_unlock(&(fifo->tail_lock));
+            opal_atomic_unlock(fifo->tail_lock);
         }
 
         /* change the address from address relative to the shared
@@ -486,7 +498,7 @@ int mca_btl_sm_component_progress(void)
                 MCA_BTL_SM_FIFO_WRITE( mca_btl_sm_component.sm_peers[peer_smp_rank],
                                        my_smp_rank, peer_smp_rank, frag, rc );
                 if(OMPI_SUCCESS != rc)
-                    return rc;
+                    goto err;
                 break;
             }
             default:
@@ -497,11 +509,15 @@ int mca_btl_sm_component_progress(void)
                 MCA_BTL_SM_FIFO_WRITE( mca_btl_sm_component.sm_peers[peer_smp_rank],
                                        my_smp_rank, peer_smp_rank, frag, rc );
                 if(OMPI_SUCCESS != rc)
-                    return rc;
+                    goto err;
                 break;
             }
         }
         rc++;
     }  /* end peer_local_smp_rank loop */
     return rc;
+err:
+    BTL_ERROR(("SM faild to send message due to shortage of shared memory.\n"));
+    mca_btl_sm[btl].error_cb(&mca_btl_sm[btl].super, MCA_BTL_ERROR_FLAGS_FATAL);
+    return rc;
 }
