@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -43,12 +43,8 @@ mca_coll_basic_reduce_scatter_intra(void *sbuf, void *rbuf, int *rcounts,
                                     struct ompi_op_t *op,
                                     struct ompi_communicator_t *comm)
 {
-    int i;
-    int err;
-    int rank;
-    int size;
-    int count;
-    long true_lb, true_extent, lb, extent;
+    int i, err, rank, size, count;
+    ptrdiff_t true_lb, true_extent, lb, extent;
     int *disps = NULL;
     char *free_buffer = NULL;
     char *pml_buffer = NULL;
@@ -68,7 +64,7 @@ mca_coll_basic_reduce_scatter_intra(void *sbuf, void *rbuf, int *rcounts,
     }
 
     if (0 == rank) {
-        disps = malloc((unsigned) size * sizeof(int));
+        disps = (int*)malloc((unsigned) size * sizeof(int));
         if (NULL == disps) {
             return OMPI_ERR_OUT_OF_RESOURCE;
         }
@@ -79,7 +75,7 @@ mca_coll_basic_reduce_scatter_intra(void *sbuf, void *rbuf, int *rcounts,
         ompi_ddt_get_extent(dtype, &lb, &extent);
         ompi_ddt_get_true_extent(dtype, &true_lb, &true_extent);
 
-        free_buffer = malloc(true_extent + (count - 1) * extent);
+        free_buffer = (char*)malloc(true_extent + (count - 1) * extent);
         if (NULL == free_buffer) {
             free(disps);
             return OMPI_ERR_OUT_OF_RESOURCE;
@@ -138,12 +134,9 @@ mca_coll_basic_reduce_scatter_inter(void *sbuf, void *rbuf, int *rcounts,
                                     struct ompi_op_t *op,
                                     struct ompi_communicator_t *comm)
 {
-    int err, i;
-    int rank;
-    int root = 0;
-    int rsize;
+    int err, i, rank, root = 0, rsize;
     int totalcounts, tcount;
-    long lb, extent;
+    ptrdiff_t lb, extent;
     char *tmpbuf = NULL, *tmpbuf2 = NULL, *tbuf = NULL;
     ompi_request_t *req;
     ompi_request_t **reqs = comm->c_coll_basic_data->mccb_reqs;
