@@ -11,9 +11,12 @@
 #include "mpe.h"
 #endif
 
+#include "limits.h"
+
 /* prototypes of functions used for collective writes only. */
 static void ADIOI_Exch_and_write(ADIO_File fd, void *buf, MPI_Datatype
-                         datatype, int nprocs, int myrank, ADIOI_Access
+                         datatype, int nprocs, int myrank, 
+                         int interleave_count, ADIOI_Access
                          *others_req, ADIO_Offset *offset_list,
                          int *len_list, int contig_access_count, ADIO_Offset
                          min_st_offset, ADIO_Offset fd_size,
@@ -75,7 +78,17 @@ void ADIOI_GEN_WriteStridedColl(ADIO_File fd, void *buf, int count,
     ADIO_Offset *offset_list = NULL, *st_offsets = NULL, *fd_start = NULL,
 	*fd_end = NULL, *end_offsets = NULL;
     int *buf_idx = NULL, *len_list = NULL;
+    char value[MPI_MAX_INFO_VAL];
+    int info_flag, ompi_parallel_opts = 0;
+    unsigned long long min_pe_request = ULONG_MAX;
+    unsigned long long max_pe_request =         0;
+    unsigned long long min_rd_request = ULONG_MAX;
+    unsigned long long max_rd_request =         0;
+    int old_error;
 
+    MPI_Info_get(fd->info, "ompi_enable_parallel_optimizations", MPI_MAX_INFO_VAL, value, 
+                 &info_flag);
+    if (info_flag) ompi_parallel_opts = 1;
 
 #ifdef PROFILE
 	MPE_Log_event(13, 0, "start computation");
@@ -105,14 +118,77 @@ void ADIOI_GEN_WriteStridedColl(ADIO_File fd, void *buf, int count,
 	/* each process communicates its start and end offsets to other 
 	   processes. The result is an array each of start and end offsets stored
 	   in order of process rank. */ 
-    
+
 	st_offsets = (ADIO_Offset *) ADIOI_Malloc(nprocs*sizeof(ADIO_Offset));
 	end_offsets = (ADIO_Offset *) ADIOI_Malloc(nprocs*sizeof(ADIO_Offset));
 
-	MPI_Allgather(&start_offset, 1, ADIO_OFFSET, st_offsets, 1,
-		      ADIO_OFFSET, fd->comm);
-	MPI_Allgather(&end_offset, 1, ADIO_OFFSET, end_offsets, 1,
-		      ADIO_OFFSET, fd->comm);
+        if (ompi_parallel_opts) {
+            /* OMPI: reduce the collectives calls from 2 to 1, to improve scaling */
+            ADIO_Offset *stend_offsets, min_rd_st_offset, max_rd_end_offset, total_rd_size;
+            ADIO_Offset my_offsets[2];
+            int nprocs_for_creq;
+
+            stend_offsets = (ADIO_Offset *) ADIOI_Malloc(2*nprocs*sizeof(ADIO_Offset));
+            my_offsets[0] = start_offset;
+            my_offsets[1] = end_offset;
+
+            MPI_Allgather(my_offsets, 2, ADIO_OFFSET, stend_offsets, 2, ADIO_OFFSET, fd->comm);
+            min_rd_st_offset  = stend_offsets[0];
+            max_rd_end_offset = stend_offsets[1]; 
+            for (i=0; i<nprocs; i++) 
+                {
+                    st_offsets [i]    = stend_offsets[i*2  ];
+                    end_offsets[i]    = stend_offsets[i*2+1]; 
+                    min_rd_st_offset  = ADIOI_MIN(st_offsets [i],min_rd_st_offset);
+                    max_rd_end_offset = ADIOI_MAX(end_offsets[i],max_rd_end_offset);
+                    min_pe_request    = ADIOI_MIN((ADIO_Offset) min_pe_request,end_offsets[i]-st_offsets[i]+1);
+                    max_pe_request    = ADIOI_MAX((ADIO_Offset) max_pe_request,end_offsets[i]-st_offsets[i]+1);
+                }
+            min_rd_request    = ADIOI_MIN((ADIO_Offset) min_rd_request, max_rd_end_offset-min_rd_st_offset+1);
+            max_rd_request    = ADIOI_MAX((ADIO_Offset) max_rd_request, max_rd_end_offset-min_rd_st_offset+1);
+            ADIOI_Free(stend_offsets);
+
+            MPI_Info_get(fd->info, "ompi_cb_nodes_runtime_override", MPI_MAX_INFO_VAL, value, 
+                         &info_flag);
+            if (info_flag) {
+                /* ------------------------------------------------------------------ */
+                /* OMPI: swh@lanl.gov (Steve Hodson):                                 */
+                /* If user has not specified cb_nodes then calculate it as follows:   */
+                /* 1)nprocs_for_coll depends initially on the collective request size.*/
+                /*   For larger requests the denominator is directly proportional to  */
+                /*   the number of times the collective buffer is reused per request. */
+                /* 2)nprocs_for_coll limited to 1/4      the number of processes      */
+                /* 3)nprocs_for_coll is at least to 1/32 the number of processes      */
+                /* 4)nprocs_for_coll limited to range 1-32. Need at least 1,          */
+                /*   but don't exceed expected number of disks in use at a time       */
+                /* 5)nprocs_for_coll even workaround                                  */
+                /* 6)nprocs_for_coll at least 2 for more than 15 processes,           */
+                /*   regardless of how small collective request is.                   */
+                /* Caveat:                                                            */
+                /* The preceeding recipe was arrived at empirically for the           */
+                /* Panasas file system on Flash. Applicability to other file systems  */
+                /* needs to be demonstrated.                                          */
+                /* Caution: Care must be taken below to make sure that nprocs_for_coll*/ 
+                /* NEVER exceeds the default aggregator configuration list build once */
+                /* in open: ADIOI_cb_config_list_parse. Since nprocs_for_coll is      */
+                /* usually less that this number, only a subset of the previously     */
+                /* allocated aggregators will be used.                                */
+                /* ------------------------------------------------------------------ */
+                total_rd_size   = max_rd_end_offset - min_rd_st_offset + 1;
+                nprocs_for_creq = (int)(total_rd_size / ( 8 * 1024 * 1024 ));
+                nprocs_for_coll = ADIOI_MIN(nprocs_for_creq, nprocs/ 4);
+                nprocs_for_coll = ADIOI_MAX(nprocs_for_coll, nprocs/32);
+                nprocs_for_coll = ADIOI_MAX(nprocs_for_coll,  1);
+                nprocs_for_coll = ADIOI_MIN(nprocs_for_coll, 32);
+                if (nprocs_for_coll > 1 && nprocs_for_coll%2 ) nprocs_for_coll--;
+                if ( nprocs > 15 ) nprocs_for_coll = ADIOI_MAX(nprocs_for_coll, 2 );
+            }
+        } else {
+            MPI_Allgather(&start_offset, 1, ADIO_OFFSET, st_offsets, 1,
+                          ADIO_OFFSET, fd->comm);
+            MPI_Allgather(&end_offset, 1, ADIO_OFFSET, end_offsets, 1,
+                          ADIO_OFFSET, fd->comm);
+        }
 
 	/* are the accesses of different processes interleaved? */
 	for (i=1; i<nprocs; i++)
@@ -196,10 +272,39 @@ void ADIOI_GEN_WriteStridedColl(ADIO_File fd, void *buf, int count,
 
 /* exchange data and write in sizes of no more than coll_bufsize. */
     ADIOI_Exch_and_write(fd, buf, datatype, nprocs, myrank,
+                        interleave_count,
                         others_req, offset_list,
 			len_list, contig_access_count, min_st_offset,
 			fd_size, fd_start, fd_end, buf_idx, error_code);
 
+    /* If this collective write is followed by an independent write,
+     * it's possible to have those subsequent writes on other processes
+     * race ahead and sneak in before the read-modify-write completes.
+     * We carry out a collective communication at the end here so no one
+     * can start independent i/o before collective I/O completes. 
+     *
+     * optimization: if only one process performing i/o, we can perform
+     * a less-expensive Bcast 
+     *
+     * need to do some gymnastics with the error codes so that if something
+     * went wrong, all processes report error, but if a process has a more
+     * specific error code, we can still have that process report the
+     * additional information */
+
+    old_error = *error_code;
+    if (*error_code != MPI_SUCCESS) *error_code = MPI_ERR_IO;
+
+    if (fd->hints->cb_nodes == 1) 
+	    MPI_Bcast(error_code, 1, MPI_INT, 
+			    fd->hints->ranklist[0], fd->comm);
+    else 
+	    MPI_Allreduce(MPI_IN_PLACE, error_code, 1, MPI_INT, 
+			    MPI_MAX, fd->comm);
+
+    if ( (old_error != MPI_SUCCESS) && (old_error != MPI_ERR_IO) )
+	    *error_code = old_error;
+
+
     if (!buftype_is_contig) ADIOI_Delete_flattened(datatype);
 
 /* free all memory allocated for collective I/O */
@@ -243,7 +348,8 @@ void ADIOI_GEN_WriteStridedColl(ADIO_File fd, void *buf, int count,
  */
 static void ADIOI_Exch_and_write(ADIO_File fd, void *buf, MPI_Datatype
 				 datatype, int nprocs, int myrank,
-				 ADIOI_Access
+				 int interleave_count, 
+                                 ADIOI_Access
 				 *others_req, ADIO_Offset *offset_list,
 				 int *len_list, int contig_access_count,
 				 ADIO_Offset
@@ -260,6 +366,7 @@ static void ADIOI_Exch_and_write(ADIO_File fd, void *buf, MPI_Datatype
    array to a file, where each local array is 8Mbytes, requiring
    at least another 8Mbytes of temp space is unacceptable. */
 
+    /* TODO: 'hole' not used outside of ADIOI_W_Exchange_data */
     int hole, i, j, m, size=0, ntimes, max_ntimes, buftype_is_contig;
     ADIO_Offset st_loc=-1, end_loc=-1, off, done, req_off;
     char *write_buf=NULL;
@@ -476,12 +583,21 @@ static void ADIOI_Exch_and_write(ADIO_File fd, void *buf, MPI_Datatype
 	    if (count[i]) flag = 1;
 
 	if (flag) {
+            /* no interleaving means we might have to recompute size in the
+	     * case where hints have steered us into this path even though we
+	     * are non-overlapped.  However, if there is a "hole" inbetween
+	     * regions, we'll do a read-modify-write and W_Exchange_data will
+	     * have read in excess data */
+	    if(!interleave_count && !hole) {
+	        for (size=0, i=0; i<contig_access_count; i++)
+		    size += len_list[i];
+	    }
 	    ADIO_WriteContig(fd, write_buf, size, MPI_BYTE, ADIO_EXPLICIT_OFFSET, 
                         off, &status, error_code);
 	    if (*error_code != MPI_SUCCESS) return;
 	}
 
-	off += size;
+        off += (int) (ADIOI_MIN(coll_bufsize, end_loc-st_loc+1-done)); 
 	done += size;
     }
 
