@@ -1,7 +1,7 @@
 /*
  * Copyright (c) 2004-2005 The Trustees of Indiana University.
  *                         All rights reserved.
- * Copyright (c) 2004-2005 The Trustees of the University of Tennessee.
+ * Copyright (c) 2004-2006 The Trustees of the University of Tennessee.
  *                         All rights reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
  *                         University of Stuttgart.  All rights reserved.
@@ -24,26 +24,27 @@
 #include "osc_pt2pt_header.h"
 #include "osc_pt2pt_obj_convert.h"
 #include "osc_pt2pt_data_move.h"
+#include "osc_pt2pt_buffer.h"
 
 #include "opal/threads/mutex.h"
 #include "ompi/info/info.h"
 #include "ompi/communicator/communicator.h"
 #include "ompi/mca/osc/osc.h"
-#include "ompi/mca/btl/btl.h"
-#include "ompi/mca/bml/bml.h"
-#include "ompi/mca/bml/base/base.h"
+#include "ompi/mca/osc/base/base.h"
+#include "ompi/mca/pml/pml.h"
 #include "ompi/datatype/dt_arch.h"
 
 static int ompi_osc_pt2pt_component_open(void);
+static void ompi_osc_pt2pt_component_fragment_cb(struct ompi_osc_pt2pt_buffer_t *buffer);
 
 ompi_osc_pt2pt_component_t mca_osc_pt2pt_component = {
     { /* ompi_osc_base_component_t */
         { /* ompi_base_component_t */
             OMPI_OSC_BASE_VERSION_1_0_0,
             "pt2pt",
-            1,
-            0,
-            0,
+            OMPI_MAJOR_VERSION,  /* MCA component major version */
+            OMPI_MINOR_VERSION,  /* MCA component minor version */
+            OMPI_RELEASE_VERSION,  /* MCA component release version */
             ompi_osc_pt2pt_component_open,
             NULL
         },
@@ -79,12 +80,6 @@ ompi_osc_pt2pt_module_t ompi_osc_pt2pt_module_template = {
     }
 };
 
-
-void ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
-                                          mca_btl_base_tag_t tag,
-                                          mca_btl_base_descriptor_t *descriptor,
-                                          void *cbdata);
-
 /* look up parameters for configuring this window.  The code first
    looks in the info structure passed by the user, then through mca
    parameters. */
@@ -100,7 +95,7 @@ check_config_value_bool(char *key, ompi_info_t *info)
     if (flag == 0) goto info_not_found;
     value_len++;
 
-    value_string = malloc(sizeof(char) * value_len);
+    value_string = (char*)malloc(sizeof(char) * value_len);
     if (NULL == value_string) goto info_not_found;
 
     ret = ompi_info_get(info, key, value_len, value_string, &flag);
@@ -121,36 +116,27 @@ check_config_value_bool(char *key, ompi_info_t *info)
     ret = mca_base_param_lookup_int(param, &flag);
     if (OMPI_SUCCESS != ret) return false;
 
-    result = flag;
-
-    return result;
+    return OPAL_INT_TO_BOOL(flag);
 }
 
 
-static int fence_sync_index;
-
-
 static int
 ompi_osc_pt2pt_component_open(void)
 {
-    fence_sync_index = 
-        mca_base_param_reg_string(&mca_osc_pt2pt_component.super.osc_version,
-                                  "fence_sync_method",
-                                  "How to synchronize fence: reduce_scatter, allreduce, alltoall",
-                                  false, false, "reduce_scatter", NULL);
+    int tmp;
 
-        mca_base_param_reg_int(&mca_osc_pt2pt_component.super.osc_version,
-                               "eager_send",
-                               "Attempt to start data movement during communication call, "
-                               "instead of at synchrnoization time.  "
-                               "Info key of same name overrides this value, "
-                               "if info key given.",
-                               false, false, 0, NULL);
+    mca_base_param_reg_int(&mca_osc_pt2pt_component.super.osc_version,
+                           "no_locks",
+                           "Enable optimizations available only if MPI_LOCK is not used.",
+                           false, false, 0, NULL);
 
-        mca_base_param_reg_int(&mca_osc_pt2pt_component.super.osc_version,
-                               "no_locks",
-                               "Enable optimizations available only if MPI_LOCK is not used.",
-                               false, false, 0, NULL);
+    mca_base_param_reg_int(&mca_osc_pt2pt_component.super.osc_version,
+                           "eager_limit",
+                           "Max size of eagerly sent data",
+                           false, false, 16 * 1024, 
+                           &tmp);
+
+    mca_osc_pt2pt_component.p2p_c_eager_size = tmp;
 
     return OMPI_SUCCESS;
 }
@@ -160,8 +146,6 @@ int
 ompi_osc_pt2pt_component_init(bool enable_progress_threads,
                              bool enable_mpi_threads)
 {
-    int ret;
-
     /* we can run with either threads or not threads (may not be able
        to do win locks)... */
     mca_osc_pt2pt_component.p2p_c_have_progress_threads = 
@@ -191,11 +175,14 @@ ompi_osc_pt2pt_component_init(bool enable_progress_threads,
                         OBJ_CLASS(ompi_osc_pt2pt_longreq_t),
                         1, -1, 1);
 
-    ret = mca_bml.bml_register(MCA_BTL_TAG_OSC_PT2PT,
-                               ompi_osc_pt2pt_component_fragment_cb,
-                               NULL);
+    OBJ_CONSTRUCT(&mca_osc_pt2pt_component.p2p_c_buffers, opal_free_list_t);
+    opal_free_list_init(&mca_osc_pt2pt_component.p2p_c_buffers,
+                        mca_osc_pt2pt_component.p2p_c_eager_size +
+                        sizeof(ompi_osc_pt2pt_buffer_t),
+                        OBJ_CLASS(ompi_osc_pt2pt_buffer_t),
+                        1, -1, 1);
 
-    return ret;
+    return OMPI_SUCCESS;
 }
 
 
@@ -204,14 +191,15 @@ ompi_osc_pt2pt_component_finalize(void)
 {
     size_t num_modules;
 
-    if (0 != 
+    if (0 !=
         (num_modules = opal_hash_table_get_size(&mca_osc_pt2pt_component.p2p_c_modules))) {
-        opal_output(0, "WARNING: There were %d Windows created but not freed.",
+        opal_output(ompi_osc_base_output,
+                    "WARNING: There were %d Windows created but not freed.",
                     num_modules);
+        opal_progress_unregister(ompi_osc_pt2pt_progress);
     }
 
-    mca_bml.bml_register(MCA_BTL_TAG_OSC_PT2PT, NULL, NULL);
-
+    OBJ_DESTRUCT(&mca_osc_pt2pt_component.p2p_c_buffers);
     OBJ_DESTRUCT(&mca_osc_pt2pt_component.p2p_c_longreqs);
     OBJ_DESTRUCT(&mca_osc_pt2pt_component.p2p_c_replyreqs);
     OBJ_DESTRUCT(&mca_osc_pt2pt_component.p2p_c_sendreqs);
@@ -228,7 +216,7 @@ ompi_osc_pt2pt_component_query(ompi_win_t *win,
                               ompi_communicator_t *comm)
 {
     /* we can always run - return a low priority */
-    return 10;
+    return 5;
 }
 
 
@@ -239,10 +227,11 @@ ompi_osc_pt2pt_component_select(ompi_win_t *win,
 {
     ompi_osc_pt2pt_module_t *module;
     int ret, i;
-    char *sync_string;
+    ompi_osc_pt2pt_buffer_t *buffer;
+    opal_free_list_item_t *item;
 
     /* create module structure */
-    module = malloc(sizeof(ompi_osc_pt2pt_module_t));
+    module = (ompi_osc_pt2pt_module_t*)malloc(sizeof(ompi_osc_pt2pt_module_t));
     if (NULL == module) return OMPI_ERROR;
 
     /* fill in the function pointer part */
@@ -263,9 +252,13 @@ ompi_osc_pt2pt_component_select(ompi_win_t *win,
         return ret;
     }
 
+    module->p2p_cb_request = NULL;
+
+    OBJ_CONSTRUCT(&module->p2p_pending_control_sends, opal_list_t);
+
     OBJ_CONSTRUCT(&module->p2p_pending_sendreqs, opal_list_t);
-    module->p2p_num_pending_sendreqs = malloc(sizeof(short) * 
-                                              ompi_comm_size(module->p2p_comm));
+    module->p2p_num_pending_sendreqs = (unsigned int*)malloc(sizeof(unsigned int) * 
+                                                      ompi_comm_size(module->p2p_comm));
     if (NULL == module->p2p_num_pending_sendreqs) {
         OBJ_DESTRUCT(&module->p2p_pending_sendreqs);
         ompi_comm_free(&comm);
@@ -275,17 +268,19 @@ ompi_osc_pt2pt_component_select(ompi_win_t *win,
         return ret;
     }
     memset(module->p2p_num_pending_sendreqs, 0, 
-           sizeof(short) * ompi_comm_size(module->p2p_comm));
+           sizeof(unsigned int) * ompi_comm_size(module->p2p_comm));
 
     module->p2p_num_pending_out = 0;
     module->p2p_num_pending_in = 0;
+    module->p2p_num_post_msgs = 0;
+    module->p2p_num_complete_msgs = 0;
     module->p2p_tag_counter = 0;
 
     OBJ_CONSTRUCT(&(module->p2p_long_msgs), opal_list_t);
 
     OBJ_CONSTRUCT(&(module->p2p_copy_pending_sendreqs), opal_list_t);
-    module->p2p_copy_num_pending_sendreqs = malloc(sizeof(short) * 
-                                                   ompi_comm_size(module->p2p_comm));
+    module->p2p_copy_num_pending_sendreqs = (unsigned int*)malloc(sizeof(unsigned int) * 
+                                                           ompi_comm_size(module->p2p_comm));
     if (NULL == module->p2p_copy_num_pending_sendreqs) {
         OBJ_DESTRUCT(&module->p2p_copy_pending_sendreqs);
         OBJ_DESTRUCT(&module->p2p_long_msgs);
@@ -298,13 +293,11 @@ ompi_osc_pt2pt_component_select(ompi_win_t *win,
         return ret;
     }
     memset(module->p2p_num_pending_sendreqs, 0, 
-           sizeof(short) * ompi_comm_size(module->p2p_comm));
-
-    module->p2p_eager_send = check_config_value_bool("eager_send", info);
+           sizeof(unsigned int) * ompi_comm_size(module->p2p_comm));
 
     /* fence data */
-    module->p2p_fence_coll_counts = malloc(sizeof(int) * 
-                                           ompi_comm_size(module->p2p_comm));
+    module->p2p_fence_coll_counts = (int*)malloc(sizeof(int) * 
+                                                 ompi_comm_size(module->p2p_comm));
     if (NULL == module->p2p_fence_coll_counts) {
         free(module->p2p_copy_num_pending_sendreqs);
         OBJ_DESTRUCT(&module->p2p_copy_pending_sendreqs);
@@ -321,9 +314,9 @@ ompi_osc_pt2pt_component_select(ompi_win_t *win,
         module->p2p_fence_coll_counts[i] = 1;
     }
 
-    module->p2p_fence_coll_results = malloc(sizeof(int) * 
-                                            ompi_comm_size(module->p2p_comm));
-    if (NULL == module->p2p_fence_coll_counts) {
+    module->p2p_fence_coll_results = (unsigned int*)malloc(sizeof(unsigned short) * 
+                                                    ompi_comm_size(module->p2p_comm));
+    if (NULL == module->p2p_fence_coll_results) {
         free(module->p2p_fence_coll_counts);
         free(module->p2p_copy_num_pending_sendreqs);
         OBJ_DESTRUCT(&module->p2p_copy_pending_sendreqs);
@@ -337,22 +330,42 @@ ompi_osc_pt2pt_component_select(ompi_win_t *win,
         return ret;
     }
 
-    /* figure out what sync method to use */
-    mca_base_param_lookup_string(fence_sync_index, &sync_string);
-    if (0 == strcmp(sync_string, "reduce_scatter")) {
-        module->p2p_fence_sync_type = OSC_SYNC_REDUCE_SCATTER;
-    } else if (0 == strcmp(sync_string, "allreduce")) {
-        module->p2p_fence_sync_type = OSC_SYNC_ALLREDUCE;
-    } else if (0 == strcmp(sync_string, "alltoall")) {
-        module->p2p_fence_sync_type = OSC_SYNC_ALLTOALL;
-    } else {
-        opal_output(0, "invalid value for fence_sync_method parameter: %s\n", sync_string);
-        return OMPI_ERROR;
-    }
-
     /* pwsc data */
     module->p2p_pw_group = NULL;
     module->p2p_sc_group = NULL;
+    module->p2p_sc_remote_active_ranks =
+        (bool*)malloc(sizeof(bool) * ompi_comm_size(module->p2p_comm));
+    if (NULL == module->p2p_sc_remote_active_ranks) {
+        free(module->p2p_fence_coll_results);
+        free(module->p2p_fence_coll_counts);
+        free(module->p2p_copy_num_pending_sendreqs);
+        OBJ_DESTRUCT(&module->p2p_copy_pending_sendreqs);
+        OBJ_DESTRUCT(&module->p2p_long_msgs);
+        free(module->p2p_num_pending_sendreqs);
+        OBJ_DESTRUCT(&module->p2p_pending_sendreqs);
+        ompi_comm_free(&comm);
+        OBJ_DESTRUCT(&(module->p2p_acc_lock));
+        OBJ_DESTRUCT(&(module->p2p_lock));
+        free(module);
+        return OMPI_ERROR;
+    }
+    module->p2p_sc_remote_ranks =
+        (int*)malloc(sizeof(int) * ompi_comm_size(module->p2p_comm));
+    if (NULL == module->p2p_sc_remote_ranks) {
+        free(module->p2p_sc_remote_active_ranks);
+        free(module->p2p_fence_coll_results);
+        free(module->p2p_fence_coll_counts);
+        free(module->p2p_copy_num_pending_sendreqs);
+        OBJ_DESTRUCT(&module->p2p_copy_pending_sendreqs);
+        OBJ_DESTRUCT(&module->p2p_long_msgs);
+        free(module->p2p_num_pending_sendreqs);
+        OBJ_DESTRUCT(&module->p2p_pending_sendreqs);
+        ompi_comm_free(&comm);
+        OBJ_DESTRUCT(&(module->p2p_acc_lock));
+        OBJ_DESTRUCT(&(module->p2p_lock));
+        free(module);
+        return OMPI_ERROR;
+    }
 
     /* lock data */
     module->p2p_lock_status = 0;
@@ -365,6 +378,11 @@ ompi_osc_pt2pt_component_select(ompi_win_t *win,
     opal_hash_table_set_value_uint32(&mca_osc_pt2pt_component.p2p_c_modules,
                                      module->p2p_comm->c_contextid,
                                      module);
+
+    if (1 == opal_hash_table_get_size(&mca_osc_pt2pt_component.p2p_c_modules)) {
+        /* start progress thread */
+        opal_progress_register(ompi_osc_pt2pt_progress);
+    }
     OPAL_THREAD_UNLOCK(&mca_osc_pt2pt_component.p2p_c_lock);
 
     /* fill in window information */
@@ -376,47 +394,91 @@ ompi_osc_pt2pt_component_select(ompi_win_t *win,
     /* sync memory - make sure all initialization completed */
     opal_atomic_mb();
 
-    /* register to receive fragment callbacks */
-    ret = mca_bml.bml_register(MCA_BTL_TAG_OSC_PT2PT,
-                               ompi_osc_pt2pt_component_fragment_cb,
-                               NULL);
-
-
-    if (module->p2p_eager_send) {
-        /* need to barrier if eager sending or we can receive before the
-           other side has been fully setup, causing much gnashing of
-           teeth. */
-        module->p2p_comm->c_coll.coll_barrier(module->p2p_comm);
+    /* start up receive for protocol headers */
+    OPAL_FREE_LIST_GET(&mca_osc_pt2pt_component.p2p_c_buffers,
+                       item, ret);
+    if (NULL == item) {
+        free(module->p2p_sc_remote_ranks);
+        free(module->p2p_sc_remote_active_ranks);
+        free(module->p2p_fence_coll_results);
+        free(module->p2p_fence_coll_counts);
+        free(module->p2p_copy_num_pending_sendreqs);
+        OBJ_DESTRUCT(&module->p2p_copy_pending_sendreqs);
+        OBJ_DESTRUCT(&module->p2p_long_msgs);
+        free(module->p2p_num_pending_sendreqs);
+        OBJ_DESTRUCT(&module->p2p_pending_sendreqs);
+        ompi_comm_free(&comm);
+        OBJ_DESTRUCT(&(module->p2p_acc_lock));
+        OBJ_DESTRUCT(&(module->p2p_lock));
+        free(module);
+        return OMPI_ERROR;
     }
+    buffer = (ompi_osc_pt2pt_buffer_t*) item;
+    buffer->cbfunc = ompi_osc_pt2pt_component_fragment_cb;
+    buffer->cbdata = (void*) module;
+    
+    ret = MCA_PML_CALL(irecv(buffer->payload,
+                             mca_osc_pt2pt_component.p2p_c_eager_size,
+                             MPI_BYTE,
+                             MPI_ANY_SOURCE,
+                             CONTROL_MSG_TAG,
+                             module->p2p_comm,
+                             &buffer->request));
+    opal_list_append(&module->p2p_pending_control_sends, 
+                     &buffer->super.super);
 
     return ret;
 }
 
 
-
 /* dispatch for callback on message completion */
-void
-ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
-                                mca_btl_base_tag_t tag,
-                                mca_btl_base_descriptor_t *descriptor,
-                                void *cbdata)
+static void
+ompi_osc_pt2pt_component_fragment_cb(struct ompi_osc_pt2pt_buffer_t *pt2pt_buffer)
 {
     int ret;
+    void *payload, *buffer;
+    size_t buffer_len;
     ompi_osc_pt2pt_module_t *module;
-    void *payload;
+    ompi_osc_pt2pt_buffer_t *new_pt2pt_buffer;
+    opal_free_list_item_t *item;
 
-    assert(descriptor->des_dst[0].seg_len >= 
+    buffer = pt2pt_buffer->payload;
+    buffer_len = pt2pt_buffer->status._count;
+    module = pt2pt_buffer->cbdata;
+
+    /* post a new receive message */
+
+    /* start up receive for protocol headers */
+    OPAL_FREE_LIST_GET(&mca_osc_pt2pt_component.p2p_c_buffers,
+                       item, ret);
+    assert(NULL != item);
+    new_pt2pt_buffer = (ompi_osc_pt2pt_buffer_t*) item;
+    new_pt2pt_buffer->cbfunc = ompi_osc_pt2pt_component_fragment_cb;
+    new_pt2pt_buffer->cbdata = (void*) module;
+    
+    ret = MCA_PML_CALL(irecv(new_pt2pt_buffer->payload,
+                             mca_osc_pt2pt_component.p2p_c_eager_size,
+                             MPI_BYTE,
+                             MPI_ANY_SOURCE,
+                             CONTROL_MSG_TAG,
+                             module->p2p_comm,
+                             &new_pt2pt_buffer->request));
+    assert(OMPI_SUCCESS == ret);
+    opal_list_append(&module->p2p_pending_control_sends, 
+                     &new_pt2pt_buffer->super.super);
+
+    assert(buffer_len >=
            sizeof(ompi_osc_pt2pt_base_header_t));
 
     /* handle message */
-    switch (((ompi_osc_pt2pt_base_header_t*) descriptor->des_dst[0].seg_addr.pval)->hdr_type) {
+    switch (((ompi_osc_pt2pt_base_header_t*) buffer)->hdr_type) {
     case OMPI_OSC_PT2PT_HDR_PUT:
         {
             ompi_osc_pt2pt_send_header_t *header;
 
             /* get our header and payload */
             header = (ompi_osc_pt2pt_send_header_t*) 
-                descriptor->des_dst[0].seg_addr.pval;
+                buffer;
             payload = (void*) (header + 1);
 
 #if !defined(WORDS_BIGENDIAN) && OMPI_ENABLE_HETEROGENEOUS_SUPPORT
@@ -425,9 +487,17 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
             }
 #endif
 
-            /* get our module pointer */
-            module = ompi_osc_pt2pt_windx_to_module(header->hdr_windx);
-            if (NULL == module) return;
+            assert(module == ompi_osc_pt2pt_windx_to_module(header->hdr_windx));
+
+            if (!ompi_win_exposure_epoch(module->p2p_win)) {
+                if (OMPI_WIN_FENCE & ompi_win_get_mode(module->p2p_win)) {
+                    /* well, we're definitely in an access epoch now */
+                    ompi_win_set_mode(module->p2p_win, 
+                                      OMPI_WIN_FENCE | 
+                                      OMPI_WIN_ACCESS_EPOCH |
+                                      OMPI_WIN_EXPOSE_EPOCH);
+                }
+            }
 
             ret = ompi_osc_pt2pt_sendreq_recv_put(module, header, payload);
         }
@@ -439,7 +509,7 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
 
             /* get our header and payload */
             header = (ompi_osc_pt2pt_send_header_t*) 
-                descriptor->des_dst[0].seg_addr.pval;
+                buffer;
             payload = (void*) (header + 1);
 
 #if !defined(WORDS_BIGENDIAN) && OMPI_ENABLE_HETEROGENEOUS_SUPPORT
@@ -448,9 +518,17 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
             }
 #endif
 
-            /* get our module pointer */
-            module = ompi_osc_pt2pt_windx_to_module(header->hdr_windx);
-            if (NULL == module) return;
+            assert(module == ompi_osc_pt2pt_windx_to_module(header->hdr_windx));
+
+            if (!ompi_win_exposure_epoch(module->p2p_win)) {
+                if (OMPI_WIN_FENCE & ompi_win_get_mode(module->p2p_win)) {
+                    /* well, we're definitely in an access epoch now */
+                    ompi_win_set_mode(module->p2p_win, 
+                                      OMPI_WIN_FENCE | 
+                                      OMPI_WIN_ACCESS_EPOCH |
+                                      OMPI_WIN_EXPOSE_EPOCH);
+                }
+            }
 
             /* receive into temporary buffer */
             ret = ompi_osc_pt2pt_sendreq_recv_accum(module, header, payload);
@@ -466,7 +544,7 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
 
             /* get our header and payload */
             header = (ompi_osc_pt2pt_send_header_t*) 
-                descriptor->des_dst[0].seg_addr.pval;
+                buffer;
             payload = (void*) (header + 1);
 
 #if !defined(WORDS_BIGENDIAN) && OMPI_ENABLE_HETEROGENEOUS_SUPPORT
@@ -475,12 +553,20 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
             }
 #endif
 
-            /* get our module pointer */
-            module = ompi_osc_pt2pt_windx_to_module(header->hdr_windx);
-            if (NULL == module) return;
+            assert(module == ompi_osc_pt2pt_windx_to_module(header->hdr_windx));
+
+            if (!ompi_win_exposure_epoch(module->p2p_win)) {
+                if (OMPI_WIN_FENCE & ompi_win_get_mode(module->p2p_win)) {
+                    /* well, we're definitely in an access epoch now */
+                    ompi_win_set_mode(module->p2p_win, 
+                                      OMPI_WIN_FENCE | 
+                                      OMPI_WIN_ACCESS_EPOCH |
+                                      OMPI_WIN_EXPOSE_EPOCH);
+                }
+            }
 
             /* create or get a pointer to our datatype */
-            proc = module->p2p_comm->c_pml_procs[header->hdr_origin]->proc_ompi;
+            proc = ompi_comm_peer_lookup( module->p2p_comm, header->hdr_origin );
             datatype = ompi_osc_pt2pt_datatype_create(proc, &payload);
 
             /* create replyreq sendreq */
@@ -507,7 +593,7 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
 
             /* get our header and payload */
             header = (ompi_osc_pt2pt_reply_header_t*) 
-                descriptor->des_dst[0].seg_addr.pval;
+                buffer;
             payload = (void*) (header + 1);
 
 #if !defined(WORDS_BIGENDIAN) && OMPI_ENABLE_HETEROGENEOUS_SUPPORT
@@ -528,7 +614,7 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
         {
             ompi_osc_pt2pt_control_header_t *header = 
                 (ompi_osc_pt2pt_control_header_t*) 
-                descriptor->des_dst[0].seg_addr.pval;
+                buffer;
 
 #if !defined(WORDS_BIGENDIAN) && OMPI_ENABLE_HETEROGENEOUS_SUPPORT
             if (header->hdr_base.hdr_flags & OMPI_OSC_PT2PT_HDR_FLAG_NBO) {
@@ -536,18 +622,16 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
             }
 #endif
 
-            /* get our module pointer */
-            module = ompi_osc_pt2pt_windx_to_module(header->hdr_windx);
-            if (NULL == module) return;
+            assert(module == ompi_osc_pt2pt_windx_to_module(header->hdr_windx));
 
-            OPAL_THREAD_ADD32(&(module->p2p_num_pending_in), -1);
+            OPAL_THREAD_ADD32(&(module->p2p_num_post_msgs), -1);
         }
         break;
     case OMPI_OSC_PT2PT_HDR_COMPLETE:
         {
             ompi_osc_pt2pt_control_header_t *header = 
                 (ompi_osc_pt2pt_control_header_t*) 
-                descriptor->des_dst[0].seg_addr.pval;
+                buffer;
 
 #if !defined(WORDS_BIGENDIAN) && OMPI_ENABLE_HETEROGENEOUS_SUPPORT
             if (header->hdr_base.hdr_flags & OMPI_OSC_PT2PT_HDR_FLAG_NBO) {
@@ -555,13 +639,11 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
             }
 #endif
 
-            /* get our module pointer */
-            module = ompi_osc_pt2pt_windx_to_module(header->hdr_windx);
-            if (NULL == module) return;
+            assert(module = ompi_osc_pt2pt_windx_to_module(header->hdr_windx));
 
             /* we've heard from one more place, and have value reqs to
                process */
-            OPAL_THREAD_ADD32(&(module->p2p_num_pending_out), -1);
+            OPAL_THREAD_ADD32(&(module->p2p_num_complete_msgs), -1);
             OPAL_THREAD_ADD32(&(module->p2p_num_pending_in), header->hdr_value[0]);
         }
         break;
@@ -570,7 +652,7 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
         {
             ompi_osc_pt2pt_control_header_t *header = 
                 (ompi_osc_pt2pt_control_header_t*) 
-                descriptor->des_dst[0].seg_addr.pval;
+                buffer;
 
 #if !defined(WORDS_BIGENDIAN) && OMPI_ENABLE_HETEROGENEOUS_SUPPORT
             if (header->hdr_base.hdr_flags & OMPI_OSC_PT2PT_HDR_FLAG_NBO) {
@@ -578,9 +660,7 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
             }
 #endif
 
-            /* get our module pointer */
-            module = ompi_osc_pt2pt_windx_to_module(header->hdr_windx);
-            if (NULL == module) return;
+            assert(module == ompi_osc_pt2pt_windx_to_module(header->hdr_windx));
 
             if (header->hdr_value[1] > 0) {
                 ompi_osc_pt2pt_passive_lock(module, header->hdr_value[0], 
@@ -595,7 +675,7 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
         {
             ompi_osc_pt2pt_control_header_t *header = 
                 (ompi_osc_pt2pt_control_header_t*) 
-                descriptor->des_dst[0].seg_addr.pval;
+                buffer;
 
 #if !defined(WORDS_BIGENDIAN) && OMPI_ENABLE_HETEROGENEOUS_SUPPORT
             if (header->hdr_base.hdr_flags & OMPI_OSC_PT2PT_HDR_FLAG_NBO) {
@@ -603,18 +683,105 @@ ompi_osc_pt2pt_component_fragment_cb(struct mca_btl_base_module_t *btl,
             }
 #endif
 
-            /* get our module pointer */
-            module = ompi_osc_pt2pt_windx_to_module(header->hdr_windx);
-            if (NULL == module) return;
+            assert(module == ompi_osc_pt2pt_windx_to_module(header->hdr_windx));
 
             ompi_osc_pt2pt_passive_unlock(module, header->hdr_value[0],
                                           header->hdr_value[1]);
         }
         break;
+    case OMPI_OSC_PT2PT_HDR_UNLOCK_REPLY:
+        {
+            ompi_osc_pt2pt_control_header_t *header = 
+                (ompi_osc_pt2pt_control_header_t*) 
+                buffer;
 
+#if !defined(WORDS_BIGENDIAN) && OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+            if (header->hdr_base.hdr_flags & OMPI_OSC_PT2PT_HDR_FLAG_NBO) {
+                OMPI_OSC_PT2PT_CONTROL_HDR_NTOH(*header);
+            }
+#endif
+
+            assert(module == ompi_osc_pt2pt_windx_to_module(header->hdr_windx));
+            OPAL_THREAD_ADD32(&(module->p2p_num_pending_out), -1);
+        }
+        break;
     default:
-        /* BWB - FIX ME - this sucks */
-        opal_output(0, "received packet for Window with unknown type");
-        abort();
+        opal_output_verbose(5, ompi_osc_base_output,
+                            "received packet for Window with unknown type");
    }
+
+    item = &(pt2pt_buffer->super);
+    OPAL_FREE_LIST_RETURN(&mca_osc_pt2pt_component.p2p_c_buffers,
+                          item);
+}
+
+
+
+int
+ompi_osc_pt2pt_request_test(ompi_request_t ** rptr,
+                            int *completed,
+                            ompi_status_public_t * status )
+{
+    ompi_request_t *request = *rptr;
+    int ret = OMPI_SUCCESS;
+
+#if OMPI_ENABLE_PROGRESS_THREADS == 0
+    if (request->req_state == OMPI_REQUEST_INACTIVE ||
+        request->req_complete) {
+        ret = ompi_request_test(rptr, completed, status);
+    } else {
+        *completed = 0;
+    }
+#else
+    ret = ompi_request_test(rptr, completed, status);
+#endif
+
+    return ret;
+}
+
+int
+ompi_osc_pt2pt_progress(void)
+{
+    int ret, done, count = 0;
+    void *node;
+    uint32_t key;
+    ompi_osc_pt2pt_module_t *module;
+    opal_list_item_t *item;
+
+    ret = opal_hash_table_get_first_key_uint32(&mca_osc_pt2pt_component.p2p_c_modules,
+                                               &key,
+                                               (void**) &module,
+                                               &node);
+    if (OMPI_SUCCESS != ret) return 0;
+
+    do {
+        /* loop through pending requests */
+        for (item = opal_list_get_first(&module->p2p_pending_control_sends) ;
+             item != opal_list_get_end(&module->p2p_pending_control_sends) ;
+             item = opal_list_get_next(item)) {
+            ompi_osc_pt2pt_buffer_t *buffer = 
+                (ompi_osc_pt2pt_buffer_t*) item;
+
+            ret = ompi_osc_pt2pt_request_test(&buffer->request, &done, &buffer->status);
+            if (OMPI_SUCCESS == ret && done) {
+                item = opal_list_remove_item(&module->p2p_pending_control_sends, 
+                                             item);
+                buffer->cbfunc(buffer);
+                /* it's possible that cbfunc is going to do something
+                   that calls progress, which means our loop is
+                   probably hosed up because it's possible that the
+                   list changed under us.  It's either exit the loop
+                   through the list or start all over again.  I'm
+                   going with exit. */
+                break;
+            }
+        }
+    } while (OMPI_SUCCESS ==
+             opal_hash_table_get_next_key_uint32(&mca_osc_pt2pt_component.p2p_c_modules,
+                                                 &key,
+                                                 (void**) &module,
+                                                 node,
+                                                 &node));
+
+    return count;
 }
