@@ -21,6 +21,6 @@
 #include "opal/class/opal_list.h"
 
 
-OBJ_CLASS_INSTANCE(ompi_osc_pt2pt_longreq_t, opal_list_item_t,
+OBJ_CLASS_INSTANCE(ompi_osc_pt2pt_longreq_t, opal_free_list_item_t,
                    NULL, NULL);
 
