@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -36,34 +36,47 @@
 #include "ompi/datatype/dt_arch.h"
 #include "ompi/peruse/peruse-internal.h"
 
-OBJ_CLASS_INSTANCE(
-    mca_pml_ob1_buffer_t,
-    ompi_free_list_item_t,
-    NULL,
-    NULL
-);
+OBJ_CLASS_INSTANCE( mca_pml_ob1_buffer_t,
+                    ompi_free_list_item_t,
+                    NULL,
+                    NULL );
 
-OBJ_CLASS_INSTANCE(
-    mca_pml_ob1_recv_frag_t,
-    opal_list_item_t,
-    NULL,
-    NULL
-);
+OBJ_CLASS_INSTANCE( mca_pml_ob1_recv_frag_t,
+                    opal_list_item_t,
+                    NULL,
+                    NULL );
 
+/**
+ * Static functions.
+ */
 
+/**
+ * Match incoming recv_frags against posted receives.  
+ * Supports out of order delivery.
+ * 
+ * @param frag_header (IN)          Header of received recv_frag.
+ * @param frag_desc (IN)            Received recv_frag descriptor.
+ * @param match_made (OUT)          Flag indicating wether a match was made.
+ * @param additional_matches (OUT)  List of additional matches 
+ * @return                          OMPI_SUCCESS or error status on failure.
+ */
+static int mca_pml_ob1_recv_frag_match( mca_btl_base_module_t *btl, 
+                                        mca_pml_ob1_match_hdr_t *hdr,
+                                        mca_btl_base_segment_t* segments,
+                                        size_t num_segments );
 
 /**
  *  Callback from BTL on receive.
  */
                                                                                                                       
-void mca_pml_ob1_recv_frag_callback(
-                                    mca_btl_base_module_t* btl, 
-                                    mca_btl_base_tag_t tag,
-                                    mca_btl_base_descriptor_t* des,
-                                    void* cbdata)
+void mca_pml_ob1_recv_frag_callback( mca_btl_base_module_t* btl, 
+                                     mca_btl_base_tag_t tag,
+                                     mca_btl_base_descriptor_t* des,
+                                     void* cbdata )
 {
     mca_btl_base_segment_t* segments = des->des_dst;
     mca_pml_ob1_hdr_t* hdr = (mca_pml_ob1_hdr_t*)segments->seg_addr.pval;
+
     if(segments->seg_len < sizeof(mca_pml_ob1_common_hdr_t)) {
         return;
     }
@@ -109,11 +122,17 @@ void mca_pml_ob1_recv_frag_callback(
                 MCA_PML_OB1_ACK_HDR_NTOH(hdr->hdr_ack);
             }
 #endif
-            sendreq = (mca_pml_ob1_send_request_t*)
-                hdr->hdr_ack.hdr_src_req.pval;
+            sendreq = (mca_pml_ob1_send_request_t*)hdr->hdr_ack.hdr_src_req.pval;
             sendreq->req_recv = hdr->hdr_ack.hdr_dst_req;
-            sendreq->req_rdma_offset = hdr->hdr_ack.hdr_rdma_offset;
-            MCA_PML_OB1_SEND_REQUEST_ADVANCE(sendreq);
+            sendreq->req_rdma_offset = (size_t)hdr->hdr_ack.hdr_rdma_offset;
+            if(OPAL_THREAD_ADD32(&sendreq->req_state, 1) == 2 &&
+                    sendreq->req_bytes_delivered >=
+                    sendreq->req_send.req_bytes_packed) {
+                MCA_PML_OB1_SEND_REQUEST_PML_COMPLETE(sendreq);
+            } else {
+                mca_pml_ob1_send_request_schedule(sendreq);
+            }
+                
             break;
         }
     case MCA_PML_OB1_HDR_TYPE_FRAG:
@@ -124,8 +143,7 @@ void mca_pml_ob1_recv_frag_callback(
                 MCA_PML_OB1_FRAG_HDR_NTOH(hdr->hdr_frag);
             }
 #endif
-            recvreq = (mca_pml_ob1_recv_request_t*)
-                hdr->hdr_frag.hdr_dst_req.pval;
+            recvreq = (mca_pml_ob1_recv_request_t*)hdr->hdr_frag.hdr_dst_req.pval;
             mca_pml_ob1_recv_request_progress(recvreq,btl,segments,des->des_dst_cnt);
             break;
         }
@@ -138,8 +156,7 @@ void mca_pml_ob1_recv_frag_callback(
                we remember if we ever change the bml. */
             assert(0 == (hdr->hdr_common.hdr_flags & MCA_PML_OB1_HDR_FLAGS_NBO));
 #endif
-            sendreq = (mca_pml_ob1_send_request_t*)
-                hdr->hdr_rdma.hdr_req.pval;
+            sendreq = (mca_pml_ob1_send_request_t*)hdr->hdr_rdma.hdr_req.pval;
             mca_pml_ob1_send_request_put(sendreq,btl,&hdr->hdr_rdma);
             break;
         }
@@ -151,8 +168,7 @@ void mca_pml_ob1_recv_frag_callback(
                 MCA_PML_OB1_FIN_HDR_NTOH(hdr->hdr_fin);
             }
 #endif
-            rdma = (mca_btl_base_descriptor_t*)
-                hdr->hdr_fin.hdr_des.pval;
+            rdma = (mca_btl_base_descriptor_t*)hdr->hdr_fin.hdr_des.pval;
             rdma->des_cbfunc(btl, NULL, rdma, OMPI_SUCCESS);
             break;
         }
@@ -176,35 +192,50 @@ void mca_pml_ob1_recv_frag_callback(
  * set by the upper level routine.
  */
 #define MCA_PML_OB1_MATCH_GENERIC_RECEIVES(hdr,generic_receives,proc,return_match) \
-do { \
-    /* local variables */ \
-    mca_pml_ob1_recv_request_t *generic_recv; \
-    int recv_tag, frag_tag = hdr->hdr_tag; \
-\
-    /* Loop over the receives. */ \
-    for( generic_recv = (mca_pml_ob1_recv_request_t *)  \
-            opal_list_get_first(generic_receives); \
-            generic_recv != (mca_pml_ob1_recv_request_t *) \
-            opal_list_get_end(generic_receives); \
-            generic_recv = (mca_pml_ob1_recv_request_t *)  \
-            ((opal_list_item_t *)generic_recv)->opal_list_next) { \
-        /* Check for a match */ \
-        recv_tag = generic_recv->req_recv.req_base.req_tag; \
-        if ( (frag_tag == recv_tag) || \
-             ( (recv_tag == OMPI_ANY_TAG) && (0 <= frag_tag) ) ) { \
- \
-            generic_recv->req_recv.req_base.req_proc = proc->proc_ompi; \
-            /* Match made */ \
-            return_match = generic_recv; \
- \
-            /* remove descriptor from posted specific ireceive list */ \
-            opal_list_remove_item(generic_receives, \
-                    (opal_list_item_t *)generic_recv); \
- \
-            break; \
-        } \
-    } \
-} while(0)
+    do {                                                                           \
+        /* local variables */                                                      \
+        mca_pml_ob1_recv_request_t *generic_recv = (mca_pml_ob1_recv_request_t *)  \
+                     opal_list_get_first(generic_receives);                        \
+        mca_pml_ob1_recv_request_t *last_recv = (mca_pml_ob1_recv_request_t *)     \
+            opal_list_get_end(generic_receives);                                   \
+        register int recv_tag, frag_tag = hdr->hdr_tag;                            \
+                                                                                   \
+        /* Loop over the receives. If the received tag is less than zero  */       \
+        /* enter in a special mode, where we match only our internal tags */       \
+        /* (such as those used by the collectives.*/                               \
+        if( 0 <= frag_tag ) {                                                      \
+            for( ; generic_recv != last_recv;                                      \
+                 generic_recv = (mca_pml_ob1_recv_request_t *)                     \
+                     ((opal_list_item_t *)generic_recv)->opal_list_next) {         \
+                /* Check for a match */                                            \
+                recv_tag = generic_recv->req_recv.req_base.req_tag;                \
+                if ( (frag_tag == recv_tag) || (recv_tag == OMPI_ANY_TAG) ) {      \
+                    break;                                                         \
+                }                                                                  \
+            }                                                                      \
+        } else {                                                                   \
+            for( ; generic_recv != last_recv;                                      \
+                 generic_recv = (mca_pml_ob1_recv_request_t *)                     \
+                     ((opal_list_item_t *)generic_recv)->opal_list_next) {         \
+                /* Check for a match */                                            \
+                recv_tag = generic_recv->req_recv.req_base.req_tag;                \
+                if(frag_tag == recv_tag) {                        \
+                    break;                                                         \
+                }                                                                  \
+            }                                                                      \
+        }                                                                          \
+        if( generic_recv != (mca_pml_ob1_recv_request_t *)                         \
+            opal_list_get_end(generic_receives) ) {                                \
+                                                                                   \
+            /* Match made */                                                       \
+            return_match = generic_recv;                                           \
+                                                                                   \
+            /* remove descriptor from posted specific ireceive list */             \
+            opal_list_remove_item(generic_receives,                                \
+                                  (opal_list_item_t *)generic_recv);               \
+                                                                                   \
+        }                                                                          \
+    } while(0)
 
 /**
  * Try and match the incoming message fragment to the list of
@@ -267,137 +298,120 @@ do { \
  * set by the upper level routine.
  */
 
-#define MCA_PML_OB1_CHECK_SPECIFIC_AND_WILD_RECEIVES_FOR_MATCH( \
-    hdr,comm,proc,return_match) \
-do {  \
-    /* local variables */  \
-    mca_pml_ob1_recv_request_t *specific_recv, *wild_recv; \
-    mca_pml_sequence_t wild_recv_seq, specific_recv_seq;  \
-    int frag_tag, wild_recv_tag, specific_recv_tag;  \
-  \
-    /* initialization */  \
-    frag_tag=hdr->hdr_tag;  \
-  \
-    /*  \
-     * We know that when this is called, both specific and wild irecvs  \
-     *  have been posted.  \
-     */  \
-    specific_recv = (mca_pml_ob1_recv_request_t *)   \
-            opal_list_get_first(&(proc)->specific_receives);  \
-    wild_recv = (mca_pml_ob1_recv_request_t *)  \
-            opal_list_get_first(&comm->wild_receives);  \
-  \
-    specific_recv_seq = specific_recv->req_recv.req_base.req_sequence;  \
-    wild_recv_seq = wild_recv->req_recv.req_base.req_sequence;  \
-  \
-    while (true) {  \
-        if (wild_recv_seq < specific_recv_seq) {  \
-            /*  \
-             * wild recv is earlier than the specific one.  \
-             */  \
-            /*  \
-             * try and match  \
-             */  \
-            wild_recv_tag = wild_recv->req_recv.req_base.req_tag;  \
-            if ( (frag_tag == wild_recv_tag) ||  \
-                 ( (wild_recv_tag == OMPI_ANY_TAG) && (0 <= frag_tag) ) ) {  \
-                /*  \
-                 * Match made  \
-                 */  \
-                return_match=wild_recv;  \
-  \
-                /* remove this recv from the wild receive queue */  \
-                opal_list_remove_item(&comm->wild_receives,  \
-                        (opal_list_item_t *)wild_recv);  \
-\
-                PERUSE_TRACE_COMM_EVENT (PERUSE_COMM_REQ_REMOVE_FROM_POSTED_Q, \
-                                         &(wild_recv->req_recv.req_base), \
-                                         PERUSE_RECV); \
-\
-                break;  \
-            }  \
-  \
-            /*  \
-             * No match, go to the next.  \
-             */  \
-            wild_recv=(mca_pml_ob1_recv_request_t *)  \
-                ((opal_list_item_t *)wild_recv)->opal_list_next;  \
-  \
-            /*  \
-             * If that was the last wild one, just look at the  \
-             * rest of the specific ones.  \
-             */  \
-            if (wild_recv == (mca_pml_ob1_recv_request_t *)  \
-                    opal_list_get_end(&comm->wild_receives) )   \
-            {   \
-                MCA_PML_OB1_CHECK_SPECIFIC_RECEIVES_FOR_MATCH(hdr, comm, proc, return_match);  \
-                break;  \
-            }  \
-  \
-            /*  \
-             * Get the sequence number for this recv, and go  \
-             * back to the top of the loop.  \
-             */  \
-            wild_recv_seq = wild_recv->req_recv.req_base.req_sequence;  \
-  \
-        } else {  \
-            /*  \
-             * specific recv is earlier than the wild one.  \
-             */  \
-            specific_recv_tag=specific_recv->req_recv.req_base.req_tag;  \
-            if ( (frag_tag == specific_recv_tag) || \
-                 ( (specific_recv_tag == OMPI_ANY_TAG) && (0<=frag_tag)) )   \
-            {  \
-                /*  \
-                 * Match made  \
-                 */  \
-                return_match = specific_recv;  \
-                /* remove descriptor from specific receive list */  \
-                opal_list_remove_item(&(proc)->specific_receives,  \
-                    (opal_list_item_t *)specific_recv);  \
-\
-                PERUSE_TRACE_COMM_EVENT (PERUSE_COMM_REQ_REMOVE_FROM_POSTED_Q, \
-                                         &(specific_recv->req_recv.req_base), \
-                                         PERUSE_RECV); \
-\
-                break; \
-            }  \
-  \
-            /*  \
-             * No match, go on to the next specific irecv.  \
-             */  \
-            specific_recv = (mca_pml_ob1_recv_request_t *)  \
-                ((opal_list_item_t *)specific_recv)->opal_list_next;  \
-  \
-            /*  \
-             * If that was the last specific irecv, process the  \
-             * rest of the wild ones.  \
-             */  \
-            if (specific_recv == (mca_pml_ob1_recv_request_t *)  \
-                    opal_list_get_end(&(proc)->specific_receives))  \
-            {  \
-                MCA_PML_OB1_CHECK_WILD_RECEIVES_FOR_MATCH(hdr, comm, proc, return_match);  \
-                break; \
-            }  \
-            /*  \
-             * Get the sequence number for this recv, and go  \
-             * back to the top of the loop.  \
-             */ \
-            specific_recv_seq = specific_recv->req_recv.req_base.req_sequence;  \
-        }  \
-    }  \
-} while(0)
+#define MCA_PML_OB1_CHECK_SPECIFIC_AND_WILD_RECEIVES_FOR_MATCH( hdr,comm,proc,return_match) \
+    do {                                                                \
+        /* local variables */                                           \
+        mca_pml_ob1_recv_request_t *specific_recv, *wild_recv;          \
+        mca_pml_sequence_t wild_recv_seq, specific_recv_seq;            \
+        int frag_tag, wild_recv_tag, specific_recv_tag;                 \
+                                                                        \
+        /* initialization */                                            \
+        frag_tag=hdr->hdr_tag;                                          \
+                                                                        \
+        /*                                                              \
+         * We know that when this is called, both specific and wild irecvs \
+         *  have been posted.                                           \
+         */                                                             \
+        specific_recv = (mca_pml_ob1_recv_request_t *)                  \
+            opal_list_get_first(&(proc)->specific_receives);            \
+        wild_recv = (mca_pml_ob1_recv_request_t *)                      \
+            opal_list_get_first(&comm->wild_receives);                  \
+                                                                        \
+        specific_recv_seq = specific_recv->req_recv.req_base.req_sequence; \
+        wild_recv_seq = wild_recv->req_recv.req_base.req_sequence;      \
+                                                                        \
+        while (true) {                                                  \
+            if (wild_recv_seq < specific_recv_seq) {                    \
+                /* wild recv is earlier than the specific one. */       \
+                /* try and match */                                     \
+                wild_recv_tag = wild_recv->req_recv.req_base.req_tag;   \
+                if ( (frag_tag == wild_recv_tag) ||                     \
+                     ( (wild_recv_tag == OMPI_ANY_TAG) && (0 <= frag_tag) ) ) { \
+                    /* Match made */                                    \
+                    return_match=wild_recv;                             \
+                                                                        \
+                    /* remove this recv from the wild receive queue */  \
+                    opal_list_remove_item(&comm->wild_receives,         \
+                                          (opal_list_item_t *)wild_recv); \
+                                                                        \
+                    PERUSE_TRACE_COMM_EVENT (PERUSE_COMM_REQ_REMOVE_FROM_POSTED_Q, \
+                                             &(wild_recv->req_recv.req_base), \
+                                             PERUSE_RECV);              \
+                                                                        \
+                    break;                                              \
+                }                                                       \
+                                                                        \
+                /* No match, go to the next */                          \
+                wild_recv=(mca_pml_ob1_recv_request_t *)                \
+                    ((opal_list_item_t *)wild_recv)->opal_list_next;    \
+                                                                        \
+                /*                                                      \
+                 * If that was the last wild one, just look at the      \
+                 * rest of the specific ones.                           \
+                 */                                                     \
+                if (wild_recv == (mca_pml_ob1_recv_request_t *)         \
+                    opal_list_get_end(&comm->wild_receives) )           \
+                    {                                                   \
+                        MCA_PML_OB1_CHECK_SPECIFIC_RECEIVES_FOR_MATCH(hdr, comm, proc, return_match); \
+                        break;                                          \
+                    }                                                   \
+                                                                        \
+                /*                                                      \
+                 * Get the sequence number for this recv, and go        \
+                 * back to the top of the loop.                         \
+                 */                                                     \
+                wild_recv_seq = wild_recv->req_recv.req_base.req_sequence; \
+                                                                        \
+            } else {                                                    \
+                /* specific recv is earlier than the wild one. */       \
+                specific_recv_tag=specific_recv->req_recv.req_base.req_tag; \
+                if ( (frag_tag == specific_recv_tag) ||                 \
+                     ( (specific_recv_tag == OMPI_ANY_TAG) && (0<=frag_tag)) ) \
+                    {                                                   \
+                        /* Match made */                                \
+                        return_match = specific_recv;                   \
+                        /* remove descriptor from specific receive list */ \
+                        opal_list_remove_item(&(proc)->specific_receives, \
+                                              (opal_list_item_t *)specific_recv); \
+                                                                        \
+                        PERUSE_TRACE_COMM_EVENT (PERUSE_COMM_REQ_REMOVE_FROM_POSTED_Q, \
+                                                 &(specific_recv->req_recv.req_base), \
+                                                 PERUSE_RECV);          \
+                                                                        \
+                        break;                                          \
+                    }                                                   \
+                                                                        \
+                /* No match, go on to the next specific irecv. */       \
+                specific_recv = (mca_pml_ob1_recv_request_t *)          \
+                    ((opal_list_item_t *)specific_recv)->opal_list_next; \
+                                                                        \
+                /*                                                      \
+                 * If that was the last specific irecv, process the     \
+                 * rest of the wild ones.                               \
+                 */                                                     \
+                if (specific_recv == (mca_pml_ob1_recv_request_t *)     \
+                    opal_list_get_end(&(proc)->specific_receives))      \
+                    {                                                   \
+                        MCA_PML_OB1_CHECK_WILD_RECEIVES_FOR_MATCH(hdr, comm, proc, return_match); \
+                        break;                                          \
+                    }                                                   \
+                /*                                                      \
+                 * Get the sequence number for this recv, and go        \
+                 * back to the top of the loop.                         \
+                 */                                                     \
+                specific_recv_seq = specific_recv->req_recv.req_base.req_sequence; \
+            }                                                           \
+        }                                                               \
+    } while(0)
 
 
 /*
  * Specialized matching routines for internal use only.
  */
 
-static bool mca_pml_ob1_check_cantmatch_for_match(
-    opal_list_t *additional_matches,
-    mca_pml_ob1_comm_t* comm,
-    mca_pml_ob1_comm_proc_t *proc);
-
+static bool mca_pml_ob1_check_cantmatch_for_match( opal_list_t *additional_matches,
+                                                   mca_pml_ob1_comm_t* comm,
+                                                   mca_pml_ob1_comm_proc_t *proc );
 
 /**
  * RCS/CTS receive side matching
@@ -429,11 +443,10 @@ static bool mca_pml_ob1_check_cantmatch_for_match(
  *   - fragments may be corrupt
  *   - this routine may be called simultaneously by more than one thread
  */
-int mca_pml_ob1_recv_frag_match(
-                                mca_btl_base_module_t *btl, 
-                                mca_pml_ob1_match_hdr_t *hdr,
-                                mca_btl_base_segment_t* segments,
-                                size_t num_segments)
+static int mca_pml_ob1_recv_frag_match( mca_btl_base_module_t *btl, 
+                                        mca_pml_ob1_match_hdr_t *hdr,
+                                        mca_btl_base_segment_t* segments,
+                                        size_t num_segments )
 {
     /* local variables */
     uint16_t next_msg_seq_expected, frag_msg_seq;
@@ -473,7 +486,7 @@ int mca_pml_ob1_recv_frag_match(
 
     /* get sequence number of next message that can be processed */
     next_msg_seq_expected = (uint16_t)proc->expected_sequence;
-    if (frag_msg_seq == next_msg_seq_expected) {
+    if(frag_msg_seq == next_msg_seq_expected) {
 
         /*
          * This is the sequence number we were expecting,
@@ -518,7 +531,8 @@ rematch:
         }
 
         /* if match found, process data */
-        if (match) {
+        if(match) {
+            match->req_recv.req_base.req_proc = proc->ompi_proc;
 
             /*
              * update delivered sequence number information, if needed.
@@ -589,11 +603,9 @@ rematch:
 
     if(match != NULL) {
         mca_pml_ob1_recv_request_progress(match,btl,segments,num_segments);
-#if OMPI_WANT_PERUSE
     } else {
         PERUSE_TRACE_MSG_EVENT( PERUSE_COMM_MSG_INSERT_IN_UNEX_Q, comm_ptr,
                                 hdr->hdr_src, hdr->hdr_tag, PERUSE_RECV);
-#endif  /* OMPI_WANT_PERUSE */
     } 
     if(additional_match) {
         opal_list_item_t* item;
@@ -623,10 +635,9 @@ rematch:
  * set by the upper level routine.
  */
 
-static bool mca_pml_ob1_check_cantmatch_for_match(
-    opal_list_t *additional_matches,
-    mca_pml_ob1_comm_t* comm,
-    mca_pml_ob1_comm_proc_t *proc)
+static bool mca_pml_ob1_check_cantmatch_for_match( opal_list_t *additional_matches,
+                                                   mca_pml_ob1_comm_t* comm,
+                                                   mca_pml_ob1_comm_proc_t *proc )
 {
     /* local parameters */
     int match_found;
@@ -704,7 +715,8 @@ rematch:
                 }
 
                 /* if match found, process data */
-                if (match) {
+                if(match) {
+                    match->req_recv.req_base.req_proc = proc->ompi_proc;
 
                     /*
                      * If this was a probe need to queue fragment on unexpected list
