@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -45,7 +46,7 @@ int MPI_Gather(void *sendbuf, int sendcount, MPI_Datatype sendtype,
             return OMPI_ERRHANDLER_INVOKE(MPI_COMM_WORLD, MPI_ERR_COMM, 
                                           FUNC_NAME);
 	} else if ((ompi_comm_rank(comm) != root && MPI_IN_PLACE == sendbuf) ||
-                   MPI_IN_PLACE == recvbuf) {
+                   (ompi_comm_rank(comm) == root && MPI_IN_PLACE == recvbuf)) {
             return OMPI_ERRHANDLER_INVOKE(comm, MPI_ERR_ARG, FUNC_NAME);
         }
 
@@ -112,12 +113,14 @@ int MPI_Gather(void *sendbuf, int sendcount, MPI_Datatype sendtype,
         }
     }
 
-    /* Can we optimize?  Everyone had to give the same send signature,
-       which means that everyone must have given a sendcount > 0 if
-       there's anything to send. */
+    /* Do we need to do anything? */
 
-    if (sendcount == 0) {
-      return MPI_SUCCESS;
+    if ((0 == sendcount &&
+         (ompi_comm_rank(comm) != root ||
+          (ompi_comm_rank(comm) == root && MPI_IN_PLACE != sendbuf))) ||
+        (ompi_comm_rank(comm) == root && MPI_IN_PLACE == sendbuf && 
+         0 == recvcount)) {
+        return MPI_SUCCESS;
     }
 
     /* Invoke the coll component to perform the back-end operation */
