@@ -9,6 +9,7 @@
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -20,6 +21,7 @@
 
 #include "ompi/mpi/c/bindings.h"
 #include "ompi/mca/pml/pml.h"
+#include "ompi/request/request.h"
 
 #if OMPI_HAVE_WEAK_SYMBOLS && OMPI_PROFILING_DEFINES
 #pragma weak MPI_Startall = PMPI_Startall
@@ -34,6 +36,8 @@ static const char FUNC_NAME[] = "MPI_Startall";
 
 int MPI_Startall(int count, MPI_Request *requests) 
 {
+    int i;
+
     if ( MPI_PARAM_CHECK ) {
         int rc = MPI_SUCCESS;
         OMPI_ERR_INIT_FINALIZE(FUNC_NAME);
@@ -42,6 +46,13 @@ int MPI_Startall(int count, MPI_Request *requests)
         } else if (count < 0) {
             rc = MPI_ERR_ARG;
         }
+        for (i = 0; i < count; ++i) {
+            if (OMPI_REQUEST_PML != requests[i]->req_type &&
+                OMPI_REQUEST_NOOP != requests[i]->req_type) {
+                rc = MPI_ERR_REQUEST;
+                break;
+            }
+        }
         OMPI_ERRHANDLER_CHECK(rc, MPI_COMM_WORLD, rc, FUNC_NAME);
     }
     return MCA_PML_CALL(start(count, requests));
