@@ -10,6 +10,7 @@
 //                         University of Stuttgart.  All rights reserved.
 // Copyright (c) 2004-2005 The Regents of the University of California.
 //                         All rights reserved.
+// Copyright (c) 2007      Cisco Systems, Inc.  All rights reserved.
 // $COPYRIGHT$
 // 
 // Additional copyrights may follow
@@ -123,7 +124,7 @@ MPI::Comm::Iprobe(int source, int tag, MPI::Status & status) const
 {
   int t;
   (void)MPI_Iprobe(source, tag, mpi_comm, &t, &status.mpi_status);
-  return (bool) t;
+  return OPAL_INT_TO_BOOL(t);
 }
   
 inline bool
@@ -131,7 +132,7 @@ MPI::Comm::Iprobe(int source, int tag) const
 {
   int t;
   (void)MPI_Iprobe(source, tag, mpi_comm, &t, MPI_STATUS_IGNORE);
-  return (bool) t;
+  return OPAL_INT_TO_BOOL(t);
 }
 
 inline void
@@ -283,13 +284,18 @@ MPI::Comm::Compare(const MPI::Comm & comm1,
   return result;
 }
 
-  
+inline void
+MPI::Comm::Free(void) 
+{
+    (void)MPI_Comm_free(&mpi_comm);
+}
+
 inline bool
 MPI::Comm::Is_inter() const
 {
   int t;
   (void)MPI_Comm_test_inter(mpi_comm, &t);
-  return (bool) t;
+  return OPAL_INT_TO_BOOL(t);
 }
 
 
@@ -539,19 +545,83 @@ MPI::Comm::Create_errhandler(MPI::Comm::_MPI2CPP_ERRHANDLERFN_* function)
 {
   MPI_Errhandler errhandler;
   // $%%@#%# AIX/POE 2.3.0.0 makes us put in this cast here
-  (void)MPI_Errhandler_create((MPI_Handler_function*) ompi_mpi_cxx_errhandler_intercept, 
+  (void)MPI_Comm_create_errhandler((MPI_Handler_function*) ompi_mpi_cxx_comm_errhandler_intercept, 
 			      &errhandler);
   MPI::Errhandler temp(errhandler);
-  temp.handler_fn = (void(*)(MPI::Comm&, int*, ...))function;
+  temp.comm_handler_fn = (void(*)(MPI::Comm&, int*, ...))function;
   return temp;
 }
 
+// 1) original Create_keyval that takes the first 2 arguments as C++
+//    functions
+inline int
+MPI::Comm::Create_keyval(MPI::Comm::Copy_attr_function* comm_copy_attr_fn,
+                         MPI::Comm::Delete_attr_function* comm_delete_attr_fn, 
+                         void* extra_state)
+{
+    // Back-end function does the heavy lifting
+    return do_create_keyval(NULL, NULL, 
+                            comm_copy_attr_fn, comm_delete_attr_fn,
+                            extra_state);
+}
+
+// 2) overload Create_keyval to take the first 2 arguments as C
+//    functions
+inline int
+MPI::Comm::Create_keyval(MPI_Comm_copy_attr_function* comm_copy_attr_fn,
+                         MPI_Comm_delete_attr_function* comm_delete_attr_fn, 
+                         void* extra_state)
+{
+    // Back-end function does the heavy lifting
+    return do_create_keyval(comm_copy_attr_fn, comm_delete_attr_fn,
+                            NULL, NULL,
+                            extra_state);
+}
+
+// 3) overload Create_keyval to take the first 2 arguments as C++ & C
+//    functions
+inline int
+MPI::Comm::Create_keyval(MPI::Comm::Copy_attr_function* comm_copy_attr_fn,
+                         MPI_Comm_delete_attr_function* comm_delete_attr_fn,
+                         void* extra_state)
+{
+    // Back-end function does the heavy lifting
+    return do_create_keyval(NULL, comm_delete_attr_fn,
+                            comm_copy_attr_fn, NULL,
+                            extra_state);
+}
+
+// 4) overload Create_keyval to take the first 2 arguments as C & C++
+//    functions
+inline int
+MPI::Comm::Create_keyval(MPI_Comm_copy_attr_function* comm_copy_attr_fn,
+                         MPI::Comm::Delete_attr_function* comm_delete_attr_fn,
+                         void* extra_state)
+{
+    // Back-end function does the heavy lifting
+    return do_create_keyval(comm_copy_attr_fn, NULL,
+                            NULL, comm_delete_attr_fn,
+                            extra_state);
+}
+
+inline void
+MPI::Comm::Free_keyval(int& comm_keyval)
+{
+    (void) MPI_Keyval_free(&comm_keyval);
+}
+
+inline void
+MPI::Comm::Set_attr(int comm_keyval, const void* attribute_val) const
+{
+    (void)MPI_Attr_put(mpi_comm, comm_keyval, (void*) attribute_val);
+}
+
 inline bool
 MPI::Comm::Get_attr(int comm_keyval, void* attribute_val) const
 {
   int flag;
   (void)MPI_Attr_get(mpi_comm, comm_keyval, attribute_val, &flag);
-  return (bool)flag;
+  return OPAL_INT_TO_BOOL(flag);
 }
 
 inline void
@@ -584,7 +654,7 @@ MPI::Comm::NULL_COPY_FN(const MPI::Comm& oldcomm, int comm_keyval,
     MPI_Copy_function *stupid_compiler = MPI_NULL_COPY_FN;
     ret = stupid_compiler(oldcomm, comm_keyval, extra_state, attribute_val_in,
                           attribute_val_out, &f);
-    flag = (bool)f;
+    flag = OPAL_INT_TO_BOOL(f);
   } else {
     ret = MPI_SUCCESS;
     flag = true;
@@ -613,7 +683,7 @@ MPI::Comm::DUP_FN(const MPI::Comm& oldcomm, int comm_keyval,
   int ret;
   ret = MPI_DUP_FN(oldcomm, comm_keyval, extra_state, attribute_val_in,
 		   attribute_val_out, &f);
-  flag = (bool) f;
+  flag = OPAL_INT_TO_BOOL(f);
   return ret;
 #else
   return MPI_DUP_FN(oldcomm, comm_keyval, extra_state, attribute_val_in,
