@@ -23,6 +23,7 @@
 
 #include "opal/threads/mutex.h"
 #include "opal/util/output.h"
+#include "opal/util/show_help.h"
 #include "orte/util/sys_info.h"
 #include "orte/dss/dss.h"
 #include "orte/mca/oob/oob.h"
@@ -35,6 +36,7 @@
 #include "ompi/datatype/dt_arch.h"
 #include "ompi/datatype/convertor.h"
 #include "ompi/runtime/params.h"
+#include "ompi/runtime/mpiruntime.h"
 
 static opal_list_t  ompi_proc_list;
 static opal_mutex_t ompi_proc_lock;
@@ -55,6 +57,7 @@ OBJ_CLASS_INSTANCE(
 
 void ompi_proc_construct(ompi_proc_t* proc)
 {
+    proc->proc_bml = NULL;
     proc->proc_pml = NULL;
     proc->proc_modex = NULL;
     OBJ_CONSTRUCT(&proc->proc_lock, opal_mutex_t);
@@ -103,7 +106,7 @@ void ompi_proc_destruct(ompi_proc_t* proc)
 int ompi_proc_init(void)
 {
     orte_process_name_t *peers;
-    size_t i, npeers, self, num_tokens;
+    orte_std_cntr_t i, npeers, num_tokens;
     orte_jobid_t jobid;
     char *segment, **tokens;
     orte_data_value_t value = { {OBJ_CLASS(orte_data_value_t),0}, ORTE_NULL, NULL};
@@ -114,7 +117,7 @@ int ompi_proc_init(void)
     OBJ_CONSTRUCT(&ompi_proc_lock, opal_mutex_t);
 
     /* get all peers in this job */
-    if(ORTE_SUCCESS != (rc = orte_ns.get_peers(&peers, &npeers, &self))) {
+    if(ORTE_SUCCESS != (rc = orte_ns.get_peers(&peers, &npeers, NULL))) {
         opal_output(0, "ompi_proc_init: get_peers failed with errno=%d", rc);
         return rc;
     }
@@ -123,7 +126,7 @@ int ompi_proc_init(void)
     for( i = 0; i < npeers; i++ ) {
         ompi_proc_t *proc = OBJ_NEW(ompi_proc_t);
         proc->proc_name = peers[i];
-        if( i == self ) {
+        if(peers[i].vpid == ORTE_PROC_MY_NAME->vpid ) {
             ompi_proc_local_proc = proc;
             proc->proc_flags |= OMPI_PROC_FLAG_LOCAL;
         }
@@ -149,10 +152,7 @@ int ompi_proc_init(void)
         return rc;
     }
 
-    if (ORTE_SUCCESS != (rc = orte_ns.get_jobid(&jobid, orte_process_info.my_name))) {
-        ORTE_ERROR_LOG(rc);
-        return rc;
-    }
+    jobid = ORTE_PROC_MY_NAME->jobid;
 
     /* find the job segment on the registry */
     if (ORTE_SUCCESS != (rc = orte_schema.get_job_segment_name(&segment, jobid))) {
@@ -222,7 +222,7 @@ ompi_proc_t** ompi_proc_world(size_t *size)
     for (proc =  (ompi_proc_t*)opal_list_get_first(&ompi_proc_list);
          proc != (ompi_proc_t*)opal_list_get_end(&ompi_proc_list);
          proc =  (ompi_proc_t*)opal_list_get_next(proc)) {
-        if (0 == orte_ns.compare(mask, &proc->proc_name, &my_name)) {
+        if (ORTE_EQUAL == orte_ns.compare_fields(mask, &proc->proc_name, &my_name)) {
             ++count;
         }
     }
@@ -238,7 +238,7 @@ ompi_proc_t** ompi_proc_world(size_t *size)
     for (proc =  (ompi_proc_t*)opal_list_get_first(&ompi_proc_list);
          proc != (ompi_proc_t*)opal_list_get_end(&ompi_proc_list);
          proc =  (ompi_proc_t*)opal_list_get_next(proc)) {
-        if (0 == orte_ns.compare(mask, &proc->proc_name, &my_name)) {
+        if (ORTE_EQUAL == orte_ns.compare_fields(mask, &proc->proc_name, &my_name)) {
             procs[count++] = proc;
         }
     }
@@ -291,25 +291,26 @@ ompi_proc_t * ompi_proc_find ( const orte_process_name_t * name )
     orte_ns_cmp_bitmask_t mask;
 
     /* return the proc-struct which matches this jobid+process id */
-
     mask = ORTE_NS_CMP_CELLID | ORTE_NS_CMP_JOBID | ORTE_NS_CMP_VPID;
     OPAL_THREAD_LOCK(&ompi_proc_lock);
     for(proc =  (ompi_proc_t*)opal_list_get_first(&ompi_proc_list);
         proc != (ompi_proc_t*)opal_list_get_end(&ompi_proc_list);
         proc =  (ompi_proc_t*)opal_list_get_next(proc)) {
-        if (0 == orte_ns.compare(mask, &proc->proc_name, name)) {
+        if (ORTE_EQUAL == orte_ns.compare_fields(mask, &proc->proc_name, name)) {
             rproc = proc;
             break;
         }
     }
     OPAL_THREAD_UNLOCK(&ompi_proc_lock);
+
     return rproc;
 }
 
 
-ompi_proc_t * ompi_proc_find_and_add ( const orte_process_name_t * name, bool* isnew )
+static ompi_proc_t *
+ompi_proc_find_and_add(const orte_process_name_t * name, bool* isnew)
 {
-    ompi_proc_t *proc, *rproc=NULL;
+    ompi_proc_t *proc, *rproc = NULL;
     orte_ns_cmp_bitmask_t mask;
 
     /* return the proc-struct which matches this jobid+process id */
@@ -318,29 +319,47 @@ ompi_proc_t * ompi_proc_find_and_add ( const orte_process_name_t * name, bool* i
     for(proc =  (ompi_proc_t*)opal_list_get_first(&ompi_proc_list);
         proc != (ompi_proc_t*)opal_list_get_end(&ompi_proc_list);
         proc =  (ompi_proc_t*)opal_list_get_next(proc)) {
-        if (0 == orte_ns.compare(mask, &proc->proc_name, name)) {
-            *isnew = false;
+        if (ORTE_EQUAL == orte_ns.compare_fields(mask, &proc->proc_name, name)) {
             rproc = proc;
+            *isnew = false;
             break;
         }
     }
+
+    if (NULL == rproc) {
+        *isnew = true;
+        rproc = OBJ_NEW(ompi_proc_t);
+        if (NULL != rproc) {
+            rproc->proc_name = *name;
+        }
+        /* caller had better fill in the rest of the proc, or there's
+           going to be pain later... */
+    }
+
     OPAL_THREAD_UNLOCK(&ompi_proc_lock);
 
-    if ( NULL == rproc ) {
-        ompi_proc_t *tproc = OBJ_NEW(ompi_proc_t);
-        rproc = tproc;
-        rproc->proc_name = *name;
-        *isnew = true;
-    }
     return rproc;
 }
 
-int ompi_proc_get_namebuf ( ompi_proc_t **proclist, int proclistsize, orte_buffer_t* buf)
+
+int
+ompi_proc_pack(ompi_proc_t **proclist, int proclistsize, orte_buffer_t* buf)
 {
-    int i;
+    int i, rc;
+
     OPAL_THREAD_LOCK(&ompi_proc_lock);
     for (i=0; i<proclistsize; i++) {
-        int rc = orte_dss.pack(buf, &(proclist[i]->proc_name), 1, ORTE_NAME);
+        rc = orte_dss.pack(buf, &(proclist[i]->proc_name), 1, ORTE_NAME);
+        if(rc != ORTE_SUCCESS) {
+            OPAL_THREAD_UNLOCK(&ompi_proc_lock);
+            return rc;
+        }
+        rc = orte_dss.pack(buf, &(proclist[i]->proc_arch), 1, ORTE_UINT32);
+        if(rc != ORTE_SUCCESS) {
+            OPAL_THREAD_UNLOCK(&ompi_proc_lock);
+            return rc;
+        }
+        rc = orte_dss.pack(buf, &(proclist[i]->proc_hostname), 1, ORTE_STRING);
         if(rc != ORTE_SUCCESS) {
             OPAL_THREAD_UNLOCK(&ompi_proc_lock);
             return rc;
@@ -351,12 +370,12 @@ int ompi_proc_get_namebuf ( ompi_proc_t **proclist, int proclistsize, orte_buffe
 }
 
 
-int ompi_proc_get_proclist (orte_buffer_t* buf, int proclistsize, ompi_proc_t ***proclist)
+int
+ompi_proc_unpack(orte_buffer_t* buf, int proclistsize, ompi_proc_t ***proclist)
 {
     int i;
-    ompi_proc_t **plist=NULL;
-    orte_process_name_t name;
-    bool isnew = false;
+    size_t newprocs_len = 0;
+    ompi_proc_t **plist=NULL, **newprocs = NULL;
 
     /* do not free plist *ever*, since it is used in the remote group
        structure of a communicator */
@@ -364,18 +383,64 @@ int ompi_proc_get_proclist (orte_buffer_t* buf, int proclistsize, ompi_proc_t **
     if ( NULL == plist ) {
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
+    /* free this on the way out */
+    newprocs = (ompi_proc_t **) calloc (proclistsize, sizeof (ompi_proc_t *));
+    if (NULL == newprocs) {
+        return OMPI_ERR_OUT_OF_RESOURCE;
+    }
 
     for ( i=0; i<proclistsize; i++ ){
-        size_t count=1;
-        int rc = orte_dss.unpack(buf, &name, &count, ORTE_NAME);
-        if(rc != ORTE_SUCCESS) {
+        orte_std_cntr_t count=1;
+        orte_process_name_t new_name;
+        uint32_t new_arch;
+        char *new_hostname;
+        bool isnew = false;
+        int rc;
+
+        rc = orte_dss.unpack(buf, &new_name, &count, ORTE_NAME);
+        if (rc != ORTE_SUCCESS) {
             return rc;
         }
-        plist[i] = ompi_proc_find_and_add ( &name, &isnew );
-        if(isnew) {
-            MCA_PML_CALL(add_procs(&plist[i], 1));
+        rc = orte_dss.unpack(buf, &new_arch, &count, ORTE_UINT32);
+        if (rc != ORTE_SUCCESS) {
+            return rc;
+        }
+        rc = orte_dss.unpack(buf, &new_hostname, &count, ORTE_STRING);
+        if (rc != ORTE_SUCCESS) {
+            return rc;
+        }
+
+        plist[i] = ompi_proc_find_and_add(&new_name, &isnew);
+        if (isnew) {
+            newprocs[newprocs_len++] = plist[i];
+
+            plist[i]->proc_arch = new_arch;
+
+            /* if arch is different than mine, create a new convertor for this proc */
+            if (plist[i]->proc_arch != ompi_mpi_local_arch) {
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
+                OBJ_RELEASE(plist[i]->proc_convertor);
+                plist[i]->proc_convertor = ompi_convertor_create(plist[i]->proc_arch, 0);
+#else
+                opal_show_help("help-mpi-runtime",
+                               "heterogeneous-support-unavailable",
+                               true, orte_system_info.nodename, 
+                               new_hostname == NULL ? "<hostname unavailable>" :
+                                   new_hostname);
+                return OMPI_ERR_NOT_SUPPORTED;
+#endif
+            }
+
+            /* Save the hostname */
+            if (ompi_mpi_keep_peer_hostnames) {
+                plist[i]->proc_hostname = new_hostname;
+            }
         }
     }
+
+    if (newprocs_len > 0) MCA_PML_CALL(add_procs(newprocs, newprocs_len));
+    if (newprocs != NULL) free(newprocs);
+
     *proclist = plist;
     return OMPI_SUCCESS;
 }
@@ -394,10 +459,7 @@ static int setup_registry_callback(void)
     orte_gpr_subscription_id_t id;
     orte_jobid_t jobid;
 
-    if (ORTE_SUCCESS != (rc = orte_ns.get_jobid(&jobid, &local->proc_name))) {
-        ORTE_ERROR_LOG(rc);
-        return rc;
-    }
+    jobid = local->proc_name.jobid;
 
     /* find the job segment on the registry */
     if (ORTE_SUCCESS !=
@@ -438,7 +500,7 @@ static int setup_registry_callback(void)
 
     if (ORTE_SUCCESS != (rc = orte_gpr.subscribe_N(&id, trig_name, sub_name,
                                 ORTE_GPR_NOTIFY_DELETE_AFTER_TRIG,
-                                ORTE_GPR_TOKENS_OR | ORTE_GPR_KEYS_OR,
+                                ORTE_GPR_TOKENS_OR | ORTE_GPR_KEYS_OR | ORTE_GPR_STRIPPED,
                                 segment,
                                 NULL,  /* wildcard - look at all containers */
                                 3, keys,
@@ -467,7 +529,7 @@ CLEANUP:
  */
 static void callback(orte_gpr_notify_data_t *data, void *cbdata)
 {
-    size_t i, j, k;
+    orte_std_cntr_t i, j, k;
     char *str = NULL;
     uint32_t arch = 0, *ui32;
     bool found_name, found_arch;
@@ -512,7 +574,7 @@ static void callback(orte_gpr_notify_data_t *data, void *cbdata)
                     if (NULL != str) {
                         free(str);
                     }
-                    str = strdup(keyval[j]->value->data);
+                    str = strdup((const char*)keyval[j]->value->data);
                 } else if (strcmp(keyval[j]->key, OMPI_PROC_ARCH) == 0) {
                     if (ORTE_SUCCESS != (rc = orte_dss.get((void**)&ui32, keyval[j]->value, ORTE_UINT32))) {
                         ORTE_ERROR_LOG(rc);
@@ -533,19 +595,31 @@ static void callback(orte_gpr_notify_data_t *data, void *cbdata)
                     /* find the associated proc entry and update its
                        arch flag.  If the nodename of this info is
                        my local host, also set the LOCAL flag. */
-                    if (0 == orte_ns.compare(mask, &name, &proc->proc_name)) {
+                    if (ORTE_EQUAL == orte_ns.compare_fields(mask, &name, &proc->proc_name)) {
                         proc->proc_arch = arch;
                         if (0 == strcmp(str, orte_system_info.nodename)) {
                             proc->proc_flags |= OMPI_PROC_FLAG_LOCAL;
                         }
-                        /* if arch is different than mine, create a new convertor for this proc */
+
+                        /* if arch is different than mine, create a
+                           new convertor for this proc in
+                           heterogeneous mode or abort in
+                           non-heterogeneous mode. */
                         if (proc->proc_arch != ompi_mpi_local_arch) {
+#if OMPI_ENABLE_HETEROGENEOUS_SUPPORT
                             OBJ_RELEASE(proc->proc_convertor);
                             proc->proc_convertor = ompi_convertor_create(proc->proc_arch, 0);
+#else
+                            opal_show_help("help-mpi-runtime",
+                                           "proc:heterogeneous-support-unavailable",
+                                           true, orte_system_info.nodename, str);
+                            /* we can't return an error, so abort. */
+                            ompi_mpi_abort(MPI_COMM_WORLD, OMPI_ERR_NOT_SUPPORTED, false);
+#endif
                         }
 
                         /* Save the hostname */
-                        if (ompi_mpi_keep_peer_hostnames) {
+                        if (ompi_mpi_keep_peer_hostnames && NULL == proc->proc_hostname) {
                             proc->proc_hostname = str;
                             str = NULL;
                         }
