@@ -2,7 +2,7 @@
 // Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
 //                         University Research and Technology
 //                         Corporation.  All rights reserved.
-// Copyright (c) 2004-2005 The University of Tennessee and The University
+// Copyright (c) 2004-2006 The University of Tennessee and The University
 //                         of Tennessee Research Foundation.  All rights
 //                         reserved.
 // Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -32,7 +32,7 @@
 #include <netdb.h>
 #endif
 
-#include "opal/install_dirs.h"
+#include "opal/mca/installdirs/installdirs.h"
 #include "opal/class/opal_value_array.h"
 #include "opal/util/printf.h"
 #include "opal/util/show_help.h"
@@ -254,29 +254,29 @@ void ompi_info::do_path(bool want_all, opal_cmd_line_t *cmd_line)
   string scope;
 
   if (want_all) {
-    show_path(path_prefix, OPAL_PREFIX);
-    show_path(path_bindir, OPAL_BINDIR);
-    show_path(path_libdir, OPAL_LIBDIR);
-    show_path(path_incdir, OPAL_INCLUDEDIR);
-    show_path(path_pkglibdir, OPAL_PKGLIBDIR);
-    show_path(path_sysconfdir, OPAL_SYSCONFDIR);
+    show_path(path_prefix, opal_install_dirs.prefix);
+    show_path(path_bindir, opal_install_dirs.bindir);
+    show_path(path_libdir, opal_install_dirs.libdir);
+    show_path(path_incdir, opal_install_dirs.includedir);
+    show_path(path_pkglibdir, opal_install_dirs.pkglibdir);
+    show_path(path_sysconfdir, opal_install_dirs.sysconfdir);
   } else {
     count = opal_cmd_line_get_ninsts(cmd_line, "path");
     for (i = 0; i < count; ++i) {
       scope = opal_cmd_line_get_param(cmd_line, "path", i, 0);
 
       if (path_prefix == scope)
-        show_path(path_prefix, OPAL_PREFIX);
+        show_path(path_prefix, opal_install_dirs.prefix);
       else if (path_bindir == scope)
-        show_path(path_bindir, OPAL_BINDIR);
+        show_path(path_bindir, opal_install_dirs.bindir);
       else if (path_libdir == scope)
-        show_path(path_libdir, OPAL_LIBDIR);
+        show_path(path_libdir, opal_install_dirs.libdir);
       else if (path_incdir == scope)
-        show_path(path_incdir, OPAL_INCLUDEDIR);
+        show_path(path_incdir, opal_install_dirs.includedir);
       else if (path_pkglibdir == scope)
-        show_path(path_pkglibdir, OPAL_PKGLIBDIR);
+        show_path(path_pkglibdir, opal_install_dirs.pkglibdir);
       else if (path_sysconfdir == scope)
-        show_path(path_sysconfdir, OPAL_SYSCONFDIR);
+        show_path(path_sysconfdir, opal_install_dirs.sysconfdir);
       else {
           char *usage = opal_cmd_line_get_usage_msg(cmd_line);
           opal_show_help("help-ompi_info.txt", "usage", true, usage);
@@ -333,6 +333,7 @@ void ompi_info::do_config(bool want_all)
                        "double underscore")))) + string(")"): "no");
   const string f90(OMPI_WANT_F90_BINDINGS ? "yes" : "no");
   const string f90_size(OMPI_F90_BUILD_SIZE);
+  const string heterogeneous(OMPI_ENABLE_HETEROGENEOUS_SUPPORT ? "yes" : "no");
   const string memprofile(OMPI_ENABLE_MEM_PROFILE ? "yes" : "no");
   const string memdebug(OMPI_ENABLE_MEM_DEBUG ? "yes" : "no");
   const string debug(OMPI_ENABLE_DEBUG ? "yes" : "no");
@@ -348,6 +349,8 @@ void ompi_info::do_config(bool want_all)
                           1 == MPI_PARAM_CHECK ? "always" : "runtime");
   string threads;
   const string want_libltdl(OMPI_WANT_LIBLTDL ? "yes" : "no");
+  const string mpirun_prefix_by_default(ORTE_WANT_ORTERUN_PREFIX_BY_DEFAULT ?
+                                        "yes" : "no");
   
   if (OMPI_HAVE_SOLARIS_THREADS || OMPI_HAVE_POSIX_THREADS) {
       threads = OMPI_HAVE_SOLARIS_THREADS ? "solaris" :
@@ -579,4 +582,7 @@ void ompi_info::do_config(bool want_all)
   out("Memory profiling support", "option:mem-profile", memprofile);
   out("Memory debugging support", "option:mem-debug", memdebug);
   out("libltdl support", "option:dlopen", want_libltdl);
+  out("Heterogeneous support", "options:heterogeneous", heterogeneous);
+  out("mpirun default --prefix", "mpirun:prefix_by_default", 
+      mpirun_prefix_by_default);
 }
