@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -80,13 +80,13 @@ extern "C" {
  *
  * The class for the list container.
  */
-OMPI_DECLSPEC OBJ_CLASS_DECLARATION(opal_list_t);
+OPAL_DECLSPEC OBJ_CLASS_DECLARATION(opal_list_t);
 /**
  * \internal
  *
  * Base class for items that are put in list (opal_list_t) containers.
  */
-OMPI_DECLSPEC OBJ_CLASS_DECLARATION(opal_list_item_t);
+OPAL_DECLSPEC OBJ_CLASS_DECLARATION(opal_list_item_t);
 
 
 /**
@@ -146,10 +146,8 @@ struct opal_list_t
 {
     opal_object_t       super;
     /**< Generic parent class for all Open MPI objects */
-    opal_list_item_t    opal_list_head;
-    /**< Head item of the list */
-    opal_list_item_t    opal_list_tail;
-    /**< Tail item of the list */
+    opal_list_item_t    opal_list_sentinel;
+    /**< Head and tail item of the list */
     volatile size_t     opal_list_length;
     /**< Quick reference to the number of items in the list */
 };
@@ -173,8 +171,8 @@ typedef struct opal_list_t opal_list_t;
  */
 static inline bool opal_list_is_empty(opal_list_t* list)
 {
-    return (list->opal_list_head.opal_list_next == 
-            &(list->opal_list_tail));
+    return (list->opal_list_sentinel.opal_list_next == 
+        &(list->opal_list_sentinel) ? true : false);
 }
 
 
@@ -194,7 +192,7 @@ static inline bool opal_list_is_empty(opal_list_t* list)
  */
 static inline opal_list_item_t* opal_list_get_first(opal_list_t* list)
 {
-    opal_list_item_t* item = (opal_list_item_t *)list->opal_list_head.opal_list_next;
+    opal_list_item_t* item = (opal_list_item_t*)list->opal_list_sentinel.opal_list_next;
 #if OMPI_ENABLE_DEBUG
     /* Spot check: ensure that the first item is only on one list */
 
@@ -221,7 +219,7 @@ static inline opal_list_item_t* opal_list_get_first(opal_list_t* list)
  */
 static inline opal_list_item_t* opal_list_get_last(opal_list_t* list)
 {
-    opal_list_item_t* item = (opal_list_item_t *)list->opal_list_tail.opal_list_prev;
+    opal_list_item_t* item = (opal_list_item_t *)list->opal_list_sentinel.opal_list_prev;
 #if OMPI_ENABLE_DEBUG
     /* Spot check: ensure that the last item is only on one list */
 
@@ -251,7 +249,7 @@ static inline opal_list_item_t* opal_list_get_last(opal_list_t* list)
  */
 static inline opal_list_item_t* opal_list_get_begin(opal_list_t* list)
 {
-    return &(list->opal_list_head);
+    return &(list->opal_list_sentinel);
 }
 
 /**
@@ -273,7 +271,7 @@ static inline opal_list_item_t* opal_list_get_begin(opal_list_t* list)
  */
 static inline opal_list_item_t* opal_list_get_end(opal_list_t* list)
 {
-    return &(list->opal_list_tail);
+    return &(list->opal_list_sentinel);
 }
 
 
@@ -413,30 +411,31 @@ _opal_list_append(l,i)
 
 static inline void _opal_list_append(opal_list_t *list, opal_list_item_t *item
 #if OMPI_ENABLE_DEBUG
-                                     , char* FILE, int LINENO
+                                     , char* FILE_NAME, int LINENO
 #endif  /* OMPI_ENABLE_DEBUG */
                                      )
 {
+    opal_list_item_t* sentinel = &(list->opal_list_sentinel);
 #if OMPI_ENABLE_DEBUG
   /* Spot check: ensure that this item is previously on no lists */
 
   assert(0 == item->opal_list_item_refcount);
   assert( NULL == item->opal_list_item_belong_to );
-  item->super.cls_init_file_name = FILE;
+  item->super.cls_init_file_name = FILE_NAME;
   item->super.cls_init_lineno    = LINENO;
 #endif
 
   /* set new element's previous pointer */
-  item->opal_list_prev=list->opal_list_tail.opal_list_prev;
+  item->opal_list_prev = sentinel->opal_list_prev;
 
   /* reset previous pointer on current last element */
-  list->opal_list_tail.opal_list_prev->opal_list_next=item;
+  sentinel->opal_list_prev->opal_list_next = item;
 
   /* reset new element's next pointer */
-  item->opal_list_next=&(list->opal_list_tail);
+  item->opal_list_next = sentinel;
 
   /* reset the list's tail element previous pointer */
-  list->opal_list_tail.opal_list_prev = item;
+  sentinel->opal_list_prev = item;
 
   /* increment list element counter */
   list->opal_list_length++;
@@ -468,6 +467,7 @@ static inline void _opal_list_append(opal_list_t *list, opal_list_item_t *item
 static inline void opal_list_prepend(opal_list_t *list, 
                                      opal_list_item_t *item) 
 {
+    opal_list_item_t* sentinel = &(list->opal_list_sentinel);
 #if OMPI_ENABLE_DEBUG
   /* Spot check: ensure that this item is previously on no lists */
 
@@ -476,16 +476,16 @@ static inline void opal_list_prepend(opal_list_t *list,
 #endif
 
   /* reset item's next pointer */
-  item->opal_list_next = list->opal_list_head.opal_list_next;
+  item->opal_list_next = sentinel->opal_list_next;
   
   /* reset item's previous pointer */
-  item->opal_list_prev = &(list->opal_list_head);
+  item->opal_list_prev = sentinel;
   
   /* reset previous first element's previous poiner */
-  list->opal_list_head.opal_list_next->opal_list_prev = item;
+  sentinel->opal_list_next->opal_list_prev = item;
   
   /* reset head's next pointer */
-  list->opal_list_head.opal_list_next = item;
+  sentinel->opal_list_next = item;
   
   /* increment list element counter */
   list->opal_list_length++;
@@ -531,20 +531,20 @@ static inline opal_list_item_t *opal_list_remove_first(opal_list_t *list)
 #if OMPI_ENABLE_DEBUG
   /* Spot check: ensure that the first item is only on this list */
 
-  assert(1 == list->opal_list_head.opal_list_next->opal_list_item_refcount);
+  assert(1 == list->opal_list_sentinel.opal_list_next->opal_list_item_refcount);
 #endif
 
   /* reset list length counter */
   list->opal_list_length--;
   
   /* get pointer to first element on the list */
-  item = list->opal_list_head.opal_list_next;
+  item = list->opal_list_sentinel.opal_list_next;
   
   /* reset previous pointer of next item on the list */
-  item->opal_list_next->opal_list_prev=item->opal_list_prev;
+  item->opal_list_next->opal_list_prev = item->opal_list_prev;
   
   /* reset the head next pointer */
-  list->opal_list_head.opal_list_next=item->opal_list_next;
+  list->opal_list_sentinel.opal_list_next = item->opal_list_next;
   
 #if OMPI_ENABLE_DEBUG
   assert( list == item->opal_list_item_belong_to );
@@ -587,26 +587,26 @@ static inline opal_list_item_t *opal_list_remove_last(opal_list_t *list)
   */
   volatile opal_list_item_t  *item;
   if ( 0 == list->opal_list_length ) {
-    return (opal_list_item_t *)NULL;
+      return (opal_list_item_t *)NULL;
   }
   
 #if OMPI_ENABLE_DEBUG
   /* Spot check: ensure that the first item is only on this list */
 
-  assert(1 == list->opal_list_tail.opal_list_prev->opal_list_item_refcount);
+  assert(1 == list->opal_list_sentinel.opal_list_prev->opal_list_item_refcount);
 #endif
 
   /* reset list length counter */
   list->opal_list_length--;
   
   /* get item */
-  item = list->opal_list_tail.opal_list_prev;
+  item = list->opal_list_sentinel.opal_list_prev;
   
   /* reset previous pointer on next to last pointer */
-  item->opal_list_prev->opal_list_next=item->opal_list_next;
+  item->opal_list_prev->opal_list_next = item->opal_list_next;
   
   /* reset tail's previous pointer */
-  list->opal_list_tail.opal_list_prev=item->opal_list_prev;
+  list->opal_list_sentinel.opal_list_prev = item->opal_list_prev;
   
 #if OMPI_ENABLE_DEBUG
   assert( list == item->opal_list_item_belong_to );
@@ -682,8 +682,8 @@ static inline void opal_list_insert_pos(opal_list_t *list, opal_list_item_t *pos
    * If index is greater than the length of the list, no action is
    * performed and false is returned.
    */
-  bool opal_list_insert(opal_list_t *list, opal_list_item_t *item, 
-                        long long idx);
+  OPAL_DECLSPEC bool opal_list_insert(opal_list_t *list, opal_list_item_t *item, 
+                                      long long idx);
 
 
     /**
@@ -703,8 +703,8 @@ static inline void opal_list_insert_pos(opal_list_t *list, opal_list_item_t *pos
      * containers remain valid, including those that point to elements
      * in \c xlist.
      */
-    void opal_list_join(opal_list_t *thislist, opal_list_item_t *pos, 
-                        opal_list_t *xlist);
+    OPAL_DECLSPEC void opal_list_join(opal_list_t *thislist, opal_list_item_t *pos, 
+                                      opal_list_t *xlist);
 
 
     /**
@@ -730,9 +730,9 @@ static inline void opal_list_insert_pos(opal_list_t *list, opal_list_item_t *pos
      * This is an O(N) operation because the length of both lists must
      * be recomputed.
      */
-    void opal_list_splice(opal_list_t *thislist, opal_list_item_t *pos,
-                          opal_list_t *xlist, opal_list_item_t *first,
-                          opal_list_item_t *last);
+    OPAL_DECLSPEC void opal_list_splice(opal_list_t *thislist, opal_list_item_t *pos,
+                                        opal_list_t *xlist, opal_list_item_t *first,
+                                        opal_list_item_t *last);
 
     /**
      * Comparison function for opal_list_sort(), below.
@@ -793,7 +793,7 @@ static inline void opal_list_insert_pos(opal_list_t *list, opal_list_item_t *pos
      * whatever the underlying type is).  See the documentation of
      * opal_list_item_compare_fn_t for an example).
      */
-    int opal_list_sort(opal_list_t* list, opal_list_item_compare_fn_t compare);
+    OPAL_DECLSPEC int opal_list_sort(opal_list_t* list, opal_list_item_compare_fn_t compare);
 
 #if defined(c_plusplus) || defined(__cplusplus)
 }
