@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -39,7 +39,7 @@ extern "C" {
     /**
      * Globally exported variable
      */
-    OMPI_COMP_EXPORT extern const opal_maffinity_base_component_1_0_0_t
+    OPAL_MODULE_DECLSPEC extern const opal_maffinity_base_component_1_0_0_t
         mca_maffinity_first_use_component;
 
 
