@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -41,15 +41,22 @@
 #include <arpa/inet.h>
 #endif
 #ifdef HAVE_NET_IF_H
-#if defined(__DARWIN_ALIGN_POWER) && __DARWIN_ALIGN_POWER
-/* Apple engineering suggested this as a workaround for a bug in OS X
- * 10.4 (Tiger) that prevented ioctl(..., SIOCGIFCONF, ...) from
- * working properly in 64 bit mode */
-#pragma options align=power
+#if defined(__APPLE__) && defined(_LP64)
+/* Apple engineering suggested using options align=power as a
+   workaround for a bug in OS X 10.4 (Tiger) that prevented ioctl(...,
+   SIOCGIFCONF, ...) from working properly in 64 bit mode on Power PC.
+   It turns out that the underlying issue is the size of struct
+   ifconf, which the kernel expects to be 12 and natural 64 bit
+   alignment would make 16.  The same bug appears in 64 bit mode on
+   Intel macs, but align=power is a no-op there, so instead, use the
+   pack pragma to instruct the compiler to pack on 4 byte words, which
+   has the same effect as align=power for our needs and works on both
+   Intel and Power PC Macs. */
+#pragma pack(push,4)
 #endif
 #include <net/if.h>
-#if defined(__DARWIN_ALIGN_POWER) && __DARWIN_ALIGN_POWER
-#pragma options align=reset
+#if defined(__APPLE__) && defined(_LP64)
+#pragma pack(pop)
 #endif
 #endif
 #ifdef HAVE_NETDB_H
@@ -103,6 +110,7 @@ static opal_list_t opal_if_list;
 static bool already_done = false;
 
 #define DEFAULT_NUMBER_INTERFACES 10
+#define MAX_IFCONF_SIZE 10 * 1024 * 1024
 
 /*
  *  Discover the list of configured interfaces. Don't care about any
@@ -117,6 +125,7 @@ static int opal_ifinit(void)
     char *ptr;
     struct ifconf ifconf;
     int ifc_len;
+    bool successful_locate = false;
 
     if (already_done) {
         return OPAL_SUCCESS;
@@ -178,6 +187,7 @@ static int opal_ifinit(void)
                call to ioctl, try again with a bigger buffer.  else stop */
             if (ifconf.ifc_len == lastlen && ifconf.ifc_len > 0) {
                 /* we didn't expand.  we're done */
+                successful_locate = true;
                 break;
             }
             lastlen = ifconf.ifc_len;
@@ -187,7 +197,11 @@ static int opal_ifinit(void)
            back around and try again with a bigger buffer */
         free(ifconf.ifc_req);
         ifc_len = (ifc_len == 0) ? 1 : ifc_len * 2;
-    } while (1);
+    } while (ifc_len < MAX_IFCONF_SIZE);
+    if (!successful_locate) {
+        opal_output(0, "opal_ifinit: unable to find network interfaces.");
+        return OPAL_ERR_FATAL;
+    }
 
 
     /* 
@@ -234,6 +248,12 @@ static int opal_ifinit(void)
         }
         if ((ifr->ifr_flags & IFF_UP) == 0) 
             continue;
+#ifdef IFF_SLAVE
+        /* Is this a slave to a load balancer or bonded channel?
+           If so, don't use it -- pick up the master instead */
+        if ((ifr->ifr_flags & IFF_SLAVE) != 0)
+           continue;
+#endif
 #if 0
         if ((ifr->ifr_flags & IFF_LOOPBACK) != 0)
             continue;
@@ -297,13 +317,11 @@ static int opal_ifinit(void)
      */
 
     #define MAX_INTERFACES 10 /* Anju: for now assume there are no more than this */
-    int ret;
     SOCKET sd; 
     INTERFACE_INFO if_list[MAX_INTERFACES];
     int num_interfaces;
     unsigned long num_bytes_returned;
     int i;
-    SOCKADDR_IN *sock_address;
     unsigned int interface_counter = 0;
     opal_if_t intf;
     opal_if_t *intf_ptr;
@@ -360,7 +378,7 @@ static int opal_ifinit(void)
             intf.if_flags = if_list[i].iiFlags;
 
             /* fill in the index in the table */
-            intf.if_index = opal_list_get_size(&opal_if_list)+1;
+            intf.if_index = (int)opal_list_get_size(&opal_if_list)+1;
 
             /* generate the interface name on your own ....
                loopback: lo
@@ -373,7 +391,7 @@ static int opal_ifinit(void)
             }
 
             /* copy all this into a persistent form and store it in the list */
-            intf_ptr = malloc(sizeof(opal_if_t));
+            intf_ptr = (opal_if_t*)malloc(sizeof(opal_if_t));
             if (NULL == intf_ptr) {
                 opal_output (0,"opal_ifinit: Unable to malloc %d bytes",sizeof(opal_list_t));
                 return OPAL_ERR_OUT_OF_RESOURCE;
@@ -509,7 +527,7 @@ int opal_ifcount(void)
 {
     if(opal_ifinit() != OPAL_SUCCESS)
         return (-1);
-    return opal_list_get_size(&opal_if_list);
+    return (int)opal_list_get_size(&opal_if_list);
 }
 
 
@@ -633,6 +651,32 @@ int opal_ifindextoname(int if_index, char* if_name, int length)
     return OPAL_ERROR;
 }
 
+
+bool
+opal_ifislocalhost(struct sockaddr *addr)
+{
+    switch (addr->sa_family) {
+    case AF_INET:
+        {
+            struct sockaddr_in *inaddr = (struct sockaddr_in*) addr;
+            /* if it's in the 127. domain, it shouldn't be routed
+               (0x7f == 127) */
+            if (0x7F000000 == (0x7F000000 & ntohl(inaddr->sin_addr.s_addr))) {
+                return true;
+            }
+            return false;
+        }
+        break;
+
+    default:
+        opal_output(0, "unhandled sa_family %d passed to opal_ifislocalhost",
+                    addr->sa_family);
+        return false;
+        break;
+    }
+}
+
+
 #define ADDRLEN 100
 bool
 opal_ifislocal(char *hostname)
@@ -721,6 +765,12 @@ opal_ifislocal(char *hostname)
     return false;
 }
 
+bool
+opal_ifislocalhost(struct sockaddr *addr)
+{
+    return false;
+}
+
 int
 opal_iffinalize(void)
 {
