@@ -43,12 +43,12 @@
 #include "orte/mca/gpr/replica/communications/gpr_replica_comm.h"
 #include "gpr_replica_fn.h"
 
-static int orte_gpr_replica_get_callback_data(orte_gpr_value_t ***values, size_t *num_vals,
+static int orte_gpr_replica_get_callback_data(orte_gpr_value_t ***values, orte_std_cntr_t *num_vals,
                                               orte_gpr_replica_subscription_t *sub);
 
 static int orte_gpr_replica_store_value_in_trigger_msg(orte_gpr_replica_subscription_t *sub,
                                                        orte_gpr_notify_message_t *msg,
-                                                       size_t cnt,
+                                                       orte_std_cntr_t cnt,
                                                        orte_gpr_value_t **values);
 
 int orte_gpr_replica_process_callbacks(void)
@@ -57,7 +57,7 @@ int orte_gpr_replica_process_callbacks(void)
     orte_gpr_replica_trigger_t **trigs;
     orte_gpr_replica_subscription_t **subs;
     orte_gpr_replica_requestor_t **reqs;
-    size_t i, j, k, m;
+    orte_std_cntr_t i, j, k, m;
     int rc;
 
     /* check and set flag indicating callbacks being processed */
@@ -150,8 +150,8 @@ int orte_gpr_replica_register_callback(orte_gpr_replica_subscription_t *sub,
     orte_gpr_replica_callbacks_t *cb;
     orte_gpr_replica_requestor_t **reqs;
     orte_gpr_value_t **values;
-    size_t cnt;
-    size_t i, j;
+    orte_std_cntr_t cnt;
+    orte_std_cntr_t i, j;
     bool cleanup_reqd;
     int rc=ORTE_SUCCESS;
 
@@ -213,7 +213,7 @@ int orte_gpr_replica_register_callback(orte_gpr_replica_subscription_t *sub,
              * subscription id, combining data where the id's match
              */
             if (ORTE_SUCCESS != (rc = orte_gpr_replica_store_value_in_msg(reqs[i],
-                                            cb->message, cnt, values))) {
+                                            cb->message, sub->name, cnt, values))) {
                 ORTE_ERROR_LOG(rc);
                 goto CLEANUP;
             }
@@ -242,7 +242,7 @@ int orte_gpr_replica_register_trigger_callback(orte_gpr_replica_trigger_t *trig)
     orte_gpr_replica_counter_t **cntr;
     orte_gpr_replica_subscription_t **subs;
     orte_gpr_value_t **values, *value;
-    size_t i, j, k, cnt;
+    orte_std_cntr_t i, j, k, cnt;
     int rc;
 
     /* set the callback's message
@@ -387,9 +387,7 @@ int orte_gpr_replica_define_callback(orte_gpr_notify_msg_type_t msg_type,
          if (((NULL == recipient && NULL == cb->requestor) &&
               (msg_type == cb->message->msg_type)) ||
              (((NULL != recipient && NULL != cb->requestor) &&
-              (0 == orte_ns.compare(ORTE_NS_CMP_ALL,
-                                    recipient,
-                                    cb->requestor))) &&
+              (ORTE_EQUAL == orte_dss.compare(recipient, cb->requestor, ORTE_NAME))) &&
                (msg_type == cb->message->msg_type))) {
              /* okay, a callback has been registered to send data to this
               * recipient - return this location
@@ -421,7 +419,7 @@ int orte_gpr_replica_define_callback(orte_gpr_notify_msg_type_t msg_type,
     if (NULL == recipient) {
         cb->requestor = NULL;
     } else {
-        if (ORTE_SUCCESS != (rc = orte_ns.copy_process_name(&(cb->requestor), recipient))) {
+        if (ORTE_SUCCESS != (rc = orte_dss.copy((void**)&(cb->requestor), recipient, ORTE_NAME))) {
             ORTE_ERROR_LOG(rc);
             return rc;
         }
@@ -436,10 +434,11 @@ int orte_gpr_replica_define_callback(orte_gpr_notify_msg_type_t msg_type,
 
 int orte_gpr_replica_store_value_in_msg(orte_gpr_replica_requestor_t *req,
                                         orte_gpr_notify_message_t *msg,
-                                        size_t cnt,
+                                        char *sub_name,
+                                        orte_std_cntr_t cnt,
                                         orte_gpr_value_t **values)
 {
-    size_t i, j, k, index;
+    orte_std_cntr_t i, j, k, index;
     orte_gpr_notify_data_t **data, *dptr;
 
     /* check to see if this data is going to the same place as
@@ -482,6 +481,10 @@ int orte_gpr_replica_store_value_in_msg(orte_gpr_replica_requestor_t *req,
         ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
         return ORTE_ERR_OUT_OF_RESOURCE;
     }
+    /* set the name of the subscription, if provided */
+    if (NULL != sub_name) {
+        dptr->target = strdup(sub_name);
+    }
     dptr->id = req->idtag;
     if (0 > orte_pointer_array_add(&index, msg->data, dptr)) {
         ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
@@ -509,10 +512,10 @@ int orte_gpr_replica_store_value_in_msg(orte_gpr_replica_requestor_t *req,
 
 static int orte_gpr_replica_store_value_in_trigger_msg(orte_gpr_replica_subscription_t *sub,
                                                        orte_gpr_notify_message_t *msg,
-                                                       size_t cnt,
+                                                       orte_std_cntr_t cnt,
                                                        orte_gpr_value_t **values)
 {
-    size_t i, j, k, index;
+    orte_std_cntr_t i, j, k, index;
     orte_gpr_notify_data_t **data, *dptr;
 
     /* check to see if this data is going to the same place as
@@ -526,7 +529,7 @@ static int orte_gpr_replica_store_value_in_trigger_msg(orte_gpr_replica_subscrip
         if (NULL != data[i]) {
             k++;
             if ((NULL == data[i]->target && NULL == sub) ||
-                 (NULL != data[i]->target &&
+                 (NULL != data[i]->target && NULL != sub->name &&
                  0 == strcmp(data[i]->target, sub->name))) { /* going to the same place */
                 for (j=0; j < cnt; j++) {
                     if (0 > orte_pointer_array_add(&index, data[i]->values, values[j])) {
@@ -557,7 +560,7 @@ static int orte_gpr_replica_store_value_in_trigger_msg(orte_gpr_replica_subscrip
         ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
         return ORTE_ERR_OUT_OF_RESOURCE;
     }
-    if (NULL != sub) {
+    if (NULL != sub && NULL != sub->name) {
         dptr->target = strdup(sub->name);
     }
     if (0 > orte_pointer_array_add(&index, msg->data, dptr)) {
@@ -584,12 +587,12 @@ static int orte_gpr_replica_store_value_in_trigger_msg(orte_gpr_replica_subscrip
     return ORTE_SUCCESS;
 }
 
-static int orte_gpr_replica_get_callback_data(orte_gpr_value_t ***ret_values, size_t *cnt,
+static int orte_gpr_replica_get_callback_data(orte_gpr_value_t ***ret_values, orte_std_cntr_t *cnt,
                                               orte_gpr_replica_subscription_t *sub)
 {
     orte_gpr_value_t **vals, **values;
     orte_gpr_replica_ivalue_t **ivals;
-    size_t i, j, k, num_tokens, num_keys, interim, count;
+    orte_std_cntr_t i, j, k, num_tokens, num_keys, interim, count;
     int rc;
 
     /* setup default error returns */
