@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
@@ -50,8 +50,6 @@ typedef struct {
     orte_data_value_t value;
 } orte_gpr_replica_ival_list_t;
 
-OBJ_CLASS_DECLARATION(orte_gpr_replica_ival_list_t);
-
 /* constructor */
 static void orte_gpr_replica_ival_list_constructor(orte_gpr_replica_ival_list_t* ptr)
 {
@@ -82,8 +80,6 @@ typedef struct {
     opal_list_t *ival_list;             /* list of ival_list_t of values found by get */
 } orte_gpr_replica_get_list_t;
 
-OBJ_CLASS_DECLARATION(orte_gpr_replica_get_list_t);
-
 /* constructor */
 static void orte_gpr_replica_get_list_constructor(orte_gpr_replica_get_list_t* ptr)
 {
@@ -118,8 +114,8 @@ OBJ_CLASS_INSTANCE(
 
 int orte_gpr_replica_put_fn(orte_gpr_addr_mode_t addr_mode,
                             orte_gpr_replica_segment_t *seg,
-                            orte_gpr_replica_itag_t *token_itags, size_t num_tokens,
-                            size_t cnt, orte_gpr_keyval_t **keyvals)
+                            orte_gpr_replica_itag_t *token_itags, orte_std_cntr_t num_tokens,
+                            orte_std_cntr_t cnt, orte_gpr_keyval_t **keyvals)
 {
     orte_gpr_replica_container_t **cptr, *cptr2;
     orte_gpr_replica_itag_t itag;
@@ -127,7 +123,7 @@ int orte_gpr_replica_put_fn(orte_gpr_addr_mode_t addr_mode,
     orte_gpr_replica_itagval_t *iptr, **iptrs;
     bool overwrite, overwritten;
     int rc;
-    size_t i, j, k, m, n, index;
+    orte_std_cntr_t i, j, k, m, n, index;
 
     OPAL_TRACE(2);
 
@@ -157,7 +153,7 @@ int orte_gpr_replica_put_fn(orte_gpr_addr_mode_t addr_mode,
     if (addr_mode & ORTE_GPR_OVERWRITE) {
         overwrite = true;
     }
-    tok_mode = 0x004f & addr_mode;
+    tok_mode = ORTE_GPR_REPLICA_TOKMODE(addr_mode);
     if (0x00 == tok_mode) {  /* default tokens addressing mode to AND */
         tok_mode = ORTE_GPR_REPLICA_AND;
     }
@@ -289,8 +285,8 @@ int orte_gpr_replica_put_fn(orte_gpr_addr_mode_t addr_mode,
 
 int orte_gpr_replica_put_nb_fn(orte_gpr_addr_mode_t addr_mode,
                 orte_gpr_replica_segment_t *seg,
-                orte_gpr_replica_itag_t *token_itags, size_t num_tokens,
-                size_t cnt, orte_gpr_keyval_t **keyvals,
+                orte_gpr_replica_itag_t *token_itags, orte_std_cntr_t num_tokens,
+                orte_std_cntr_t cnt, orte_gpr_keyval_t **keyvals,
                 orte_gpr_notify_cb_fn_t cbfunc, void *user_tag)
 {
     OPAL_TRACE(2);
@@ -301,9 +297,9 @@ int orte_gpr_replica_put_nb_fn(orte_gpr_addr_mode_t addr_mode,
 
 int orte_gpr_replica_get_fn(orte_gpr_addr_mode_t addr_mode,
                             orte_gpr_replica_segment_t *seg,
-                            orte_gpr_replica_itag_t *tokentags, size_t num_tokens,
-                            orte_gpr_replica_itag_t *keytags, size_t num_keys,
-                            size_t *cnt, orte_gpr_value_t ***values)
+                            orte_gpr_replica_itag_t *tokentags, orte_std_cntr_t num_tokens,
+                            orte_gpr_replica_itag_t *keytags, orte_std_cntr_t num_keys,
+                            orte_std_cntr_t *cnt, orte_gpr_value_t ***values)
 {
     opal_list_t get_list;
     orte_gpr_replica_get_list_t *gptr;
@@ -313,7 +309,8 @@ int orte_gpr_replica_get_fn(orte_gpr_addr_mode_t addr_mode,
     orte_gpr_keyval_t **kptr;
     orte_gpr_replica_addr_mode_t tokmode, keymode;
     int rc;
-    size_t i, j, k, m;
+    orte_std_cntr_t i, j, k, m;
+    bool stripped;
 
     OPAL_TRACE(2);
 
@@ -353,15 +350,22 @@ int orte_gpr_replica_get_fn(orte_gpr_addr_mode_t addr_mode,
     *cnt = 0;
     *values = NULL;
 
-    tokmode = 0x004f & addr_mode;
+    tokmode = ORTE_GPR_REPLICA_TOKMODE(addr_mode);
     if (0x00 == tokmode) {  /* default token addressing mode to AND */
         tokmode = ORTE_GPR_REPLICA_AND;
     }
-    keymode = ((0x4f00 & addr_mode) >> 8) & 0x004f;
+    keymode = ORTE_GPR_REPLICA_KEYMODE(addr_mode);
     if (0x00 == keymode) {  /* default key addressing mode to OR */
         keymode = ORTE_GPR_REPLICA_OR;
     }
 
+    /* set the stripped flag - do they want descriptive info in result or not */
+    if (ORTE_GPR_REPLICA_STRIPPED(addr_mode)) {
+        stripped = true;
+    } else {
+        stripped = false;
+    }
+
     /* find all containers that meet search criteria for tokens */
     if (ORTE_SUCCESS != (rc = orte_gpr_replica_find_containers(seg, tokmode,
                                     tokentags, num_tokens))) {
@@ -402,10 +406,15 @@ int orte_gpr_replica_get_fn(orte_gpr_addr_mode_t addr_mode,
                         }
                         ival_list->itag = iptr[j]->itag;
                         ival_list->value.type = iptr[j]->value->type;
-                        if (ORTE_SUCCESS != (rc = orte_dss.copy(&((ival_list->value).data), iptr[j]->value->data, iptr[j]->value->type))) {
-                            ORTE_ERROR_LOG(rc);
-                            OBJ_RELEASE(ival_list);
-                            return rc;
+                        /* it is okay for the data to be NULL as we may not have stored a value yet
+                         * or we may be dealing with an UNDEF type
+                         */
+                        if (NULL != iptr[j]->value->data) {
+                            if (ORTE_SUCCESS != (rc = orte_dss.copy(&((ival_list->value).data), iptr[j]->value->data, iptr[j]->value->type))) {
+                                ORTE_ERROR_LOG(rc);
+                                OBJ_RELEASE(ival_list);
+                                return rc;
+                            }
                         }
                         opal_list_append(gptr->ival_list, &ival_list->item);
                     }
@@ -435,38 +444,30 @@ int orte_gpr_replica_get_fn(orte_gpr_addr_mode_t addr_mode,
             rc = ORTE_ERROR;
             goto CLEANUP;
         }
-        (*values)[i] = OBJ_NEW(orte_gpr_value_t);
-        if (NULL == (*values)[i]) {
-            ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
-            *cnt = 0;
-            rc = ORTE_ERR_OUT_OF_RESOURCE;
-            goto CLEANUP;
-        }
-        (*values)[i]->addr_mode = addr_mode;
-        (*values)[i]->segment = strdup(seg->name);
-        (*values)[i]->cnt = opal_list_get_size(gptr->ival_list);
-        cptr2 = gptr->cptr;
-        (*values)[i]->num_tokens = cptr2->num_itags;
-        (*values)[i]->tokens = (char **)malloc(cptr2->num_itags * sizeof(char*));
-        if (NULL == (*values)[i]->tokens) {
-            ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
-            rc = ORTE_ERR_OUT_OF_RESOURCE;
-            goto CLEANUP;
-        }
-        for (j=0; j < cptr2->num_itags; j++) {
-            if (ORTE_SUCCESS != (rc = orte_gpr_replica_dict_reverse_lookup(
-                                        &((*values)[i]->tokens[j]), seg, cptr2->itags[j]))) {
+        if (stripped) {
+            if (ORTE_SUCCESS != (rc = orte_gpr_base_create_value(&((*values)[i]), addr_mode, NULL,
+                                                                 (orte_std_cntr_t)opal_list_get_size(gptr->ival_list),
+                                                                 0))) {
                 ORTE_ERROR_LOG(rc);
+                *cnt = 0;
                 goto CLEANUP;
             }
+        } else {
+            cptr2 = gptr->cptr;
+            if (ORTE_SUCCESS != (rc = orte_gpr_base_create_value(&((*values)[i]), addr_mode, seg->name,
+                                                                 (orte_std_cntr_t)opal_list_get_size(gptr->ival_list),
+                                                                 cptr2->num_itags))) {
+                ORTE_ERROR_LOG(rc);
+                *cnt = 0;
+                goto CLEANUP;
+            }
+            for (j=0; j < cptr2->num_itags; j++) {
+                if (ORTE_SUCCESS != (rc = orte_gpr_replica_dict_reverse_lookup(&((*values)[i]->tokens[j]), seg, cptr2->itags[j]))) {
+                    ORTE_ERROR_LOG(rc);
+                    goto CLEANUP;
+                }
+            }
         }
-        (*values)[i]->keyvals = (orte_gpr_keyval_t**)malloc((*values)[i]->cnt * sizeof(orte_gpr_keyval_t*));
-        if (NULL == (*values)[i]->keyvals) {
-            ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
-            rc = ORTE_ERR_OUT_OF_RESOURCE;
-            goto CLEANUP;
-        }
-
         kptr = (*values)[i]->keyvals;
         for (j=0; j < (*values)[i]->cnt; j++) {
             ival_list = (orte_gpr_replica_ival_list_t*)opal_list_remove_first(gptr->ival_list);
@@ -493,9 +494,12 @@ int orte_gpr_replica_get_fn(orte_gpr_addr_mode_t addr_mode,
                 goto CLEANUP;
             }
             kptr[j]->value->type = ival_list->value.type;
-            if (ORTE_SUCCESS != (rc = orte_dss.copy(&((kptr[j]->value)->data), ival_list->value.data, ival_list->value.type))) {
-                ORTE_ERROR_LOG(rc);
-                goto CLEANUP;
+            /* okay to have NULL data */
+            if (NULL != ival_list->value.data) {
+                if (ORTE_SUCCESS != (rc = orte_dss.copy(&((kptr[j]->value)->data), ival_list->value.data, ival_list->value.type))) {
+                    ORTE_ERROR_LOG(rc);
+                    goto CLEANUP;
+                }
             }
             OBJ_RELEASE(ival_list);
         }
@@ -520,10 +524,10 @@ CLEANUP:
 
 int orte_gpr_replica_get_conditional_fn(orte_gpr_addr_mode_t addr_mode,
                             orte_gpr_replica_segment_t *seg,
-                            orte_gpr_replica_itag_t *tokentags, size_t num_tokens,
-                            orte_gpr_replica_itag_t *keytags, size_t num_keys,
-                            size_t num_conditions, orte_gpr_replica_itagval_t **conditions,
-                            size_t *cnt, orte_gpr_value_t ***values)
+                            orte_gpr_replica_itag_t *tokentags, orte_std_cntr_t num_tokens,
+                            orte_gpr_replica_itag_t *keytags, orte_std_cntr_t num_keys,
+                            orte_std_cntr_t num_conditions, orte_gpr_replica_itagval_t **conditions,
+                            orte_std_cntr_t *cnt, orte_gpr_value_t ***values)
 {
     opal_list_t get_list;
     orte_gpr_replica_get_list_t *gptr;
@@ -533,7 +537,8 @@ int orte_gpr_replica_get_conditional_fn(orte_gpr_addr_mode_t addr_mode,
     orte_gpr_keyval_t **kptr;
     orte_gpr_replica_addr_mode_t tokmode, keymode;
     int rc;
-    size_t i, j, k, m, n;
+    orte_std_cntr_t i, j, k, m, n;
+    bool stripped;
 
     OPAL_TRACE(2);
 
@@ -542,14 +547,19 @@ int orte_gpr_replica_get_conditional_fn(orte_gpr_addr_mode_t addr_mode,
     *cnt = 0;
     *values = NULL;
 
-    tokmode = 0x004f & addr_mode;
+    tokmode = ORTE_GPR_REPLICA_TOKMODE(addr_mode);
     if (0x00 == tokmode) {  /* default token addressing mode to AND */
         tokmode = ORTE_GPR_REPLICA_AND;
     }
-    keymode = ((0x4f00 & addr_mode) >> 8) & 0x004f;
+    keymode = ORTE_GPR_REPLICA_KEYMODE(addr_mode);
     if (0x00 == keymode) {  /* default key addressing mode to OR */
         keymode = ORTE_GPR_REPLICA_OR;
     }
+    if (ORTE_GPR_REPLICA_STRIPPED(addr_mode)) {
+        stripped = true;
+    } else {
+        stripped = false;
+    }
 
     /* find all containers that meet search criteria for tokens */
     if (ORTE_SUCCESS != (rc = orte_gpr_replica_find_containers(seg, tokmode,
@@ -597,10 +607,13 @@ int orte_gpr_replica_get_conditional_fn(orte_gpr_addr_mode_t addr_mode,
                         }
                         ival_list->itag = iptr[j]->itag;
                         ival_list->value.type = iptr[j]->value->type;
-                        if (ORTE_SUCCESS != (rc = orte_dss.copy(&((ival_list->value).data), iptr[j]->value->data, iptr[j]->value->type))) {
-                            ORTE_ERROR_LOG(rc);
-                            OBJ_RELEASE(ival_list);
-                            return rc;
+                        /* it is okay to have NULL data */
+                        if (NULL != iptr[j]->value->data) {
+                            if (ORTE_SUCCESS != (rc = orte_dss.copy(&((ival_list->value).data), iptr[j]->value->data, iptr[j]->value->type))) {
+                                ORTE_ERROR_LOG(rc);
+                                OBJ_RELEASE(ival_list);
+                                return rc;
+                            }
                         }
                         opal_list_append(gptr->ival_list, &ival_list->item);
                     }
@@ -632,38 +645,30 @@ MOVEON:
             rc = ORTE_ERROR;
             goto CLEANUP;
         }
-        (*values)[i] = OBJ_NEW(orte_gpr_value_t);
-        if (NULL == (*values)[i]) {
-            ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
-            *cnt = 0;
-            rc = ORTE_ERR_OUT_OF_RESOURCE;
-            goto CLEANUP;
-        }
-        (*values)[i]->addr_mode = addr_mode;
-        (*values)[i]->segment = strdup(seg->name);
-        (*values)[i]->cnt = opal_list_get_size(gptr->ival_list);
-        cptr2 = gptr->cptr;
-        (*values)[i]->num_tokens = cptr2->num_itags;
-        (*values)[i]->tokens = (char **)malloc(cptr2->num_itags * sizeof(char*));
-        if (NULL == (*values)[i]->tokens) {
-            ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
-            rc = ORTE_ERR_OUT_OF_RESOURCE;
-            goto CLEANUP;
-        }
-        for (j=0; j < cptr2->num_itags; j++) {
-            if (ORTE_SUCCESS != (rc = orte_gpr_replica_dict_reverse_lookup(
-                                        &((*values)[i]->tokens[j]), seg, cptr2->itags[j]))) {
+        if (stripped) {
+            if (ORTE_SUCCESS != (rc = orte_gpr_base_create_value(&((*values)[i]), addr_mode, NULL,
+                                                                 (orte_std_cntr_t)opal_list_get_size(gptr->ival_list),
+                                                                 0))) {
                 ORTE_ERROR_LOG(rc);
+                *cnt = 0;
                 goto CLEANUP;
             }
+        } else {
+            cptr2 = gptr->cptr;
+            if (ORTE_SUCCESS != (rc = orte_gpr_base_create_value(&((*values)[i]), addr_mode, seg->name,
+                                                                 (orte_std_cntr_t)opal_list_get_size(gptr->ival_list),
+                                                                 cptr2->num_itags))) {
+                ORTE_ERROR_LOG(rc);
+                *cnt = 0;
+                goto CLEANUP;
+            }
+            for (j=0; j < cptr2->num_itags; j++) {
+                if (ORTE_SUCCESS != (rc = orte_gpr_replica_dict_reverse_lookup(&((*values)[i]->tokens[j]), seg, cptr2->itags[j]))) {
+                    ORTE_ERROR_LOG(rc);
+                    goto CLEANUP;
+                }
+            }
         }
-        (*values)[i]->keyvals = (orte_gpr_keyval_t**)malloc((*values)[i]->cnt * sizeof(orte_gpr_keyval_t*));
-        if (NULL == (*values)[i]->keyvals) {
-            ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
-            rc = ORTE_ERR_OUT_OF_RESOURCE;
-            goto CLEANUP;
-        }
-
         kptr = (*values)[i]->keyvals;
         for (j=0; j < (*values)[i]->cnt; j++) {
             ival_list = (orte_gpr_replica_ival_list_t*)opal_list_remove_first(gptr->ival_list);
@@ -690,10 +695,13 @@ MOVEON:
                 goto CLEANUP;
             }
             kptr[j]->value->type = ival_list->value.type;
-            if (ORTE_SUCCESS != (rc = orte_dss.copy(&((kptr[j]->value)->data), ival_list->value.data, ival_list->value.type))) {
-                ORTE_ERROR_LOG(rc);
-                goto CLEANUP;
+            /* okay to have NULL data */
+            if (NULL != ival_list->value.data) {
+                if (ORTE_SUCCESS != (rc = orte_dss.copy(&((kptr[j]->value)->data), ival_list->value.data, ival_list->value.type))) {
+                    ORTE_ERROR_LOG(rc);
+                    goto CLEANUP;
                 }
+            }
             OBJ_RELEASE(ival_list);
         }
         OBJ_RELEASE(gptr);
@@ -717,8 +725,8 @@ CLEANUP:
 
 int orte_gpr_replica_get_nb_fn(orte_gpr_addr_mode_t addr_mode,
                                 orte_gpr_replica_segment_t *seg,
-                                orte_gpr_replica_itag_t *tokentags, size_t num_tokens,
-                                orte_gpr_replica_itag_t *keytags, size_t num_keys,
+                                orte_gpr_replica_itag_t *tokentags, orte_std_cntr_t num_tokens,
+                                orte_gpr_replica_itag_t *keytags, orte_std_cntr_t num_keys,
                                 orte_gpr_notify_cb_fn_t cbfunc, void *user_tag)
 {
     OPAL_TRACE(2);
