@@ -24,13 +24,15 @@
 #include "orte/dss/dss.h"
 #include "orte/mca/errmgr/errmgr.h"
 
-#include "orte/mca/ns/base/base.h"
+#include "orte/mca/ns/base/ns_private.h"
 
 /*
  * STANDARD RELEASE FUNCTION - WORKS FOR EVERYTHING NON-STRUCTURED
  */
 void orte_ns_base_std_release(orte_data_value_t *value)
 {
+    if (NULL == value) return;
+    
     free(value->data);
     value->data = NULL;
 }
