@@ -2,17 +2,19 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
- * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
+ * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
- * 
+ *
  * Additional copyrights may follow
- * 
+ *
  * $HEADER$
  *
  * These symbols are in a file by themselves to provide nice linker
@@ -23,45 +25,58 @@
  */
 
 #include "orte_config.h"
+#include "orte/orte_constants.h"
 
 #include <fcntl.h>
 #include <errno.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
-#include "orte/orte_constants.h"
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+
 #include "opal/mca/base/mca_base_param.h"
 #include "opal/util/argv.h"
 #include "opal/util/opal_environ.h"
+
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/gpr/gpr.h"
-#include "orte/mca/pls/pls.h"
-#include "orte/mca/pls/poe/pls_poe.h"
 #include "orte/mca/ns/ns.h"
-#include "orte/mca/rmaps/base/base.h"
-#include "orte/mca/rmaps/base/rmaps_base_map.h"
-#include "orte/mca/rmgr/base/base.h"
+#include "orte/mca/rmaps/rmaps.h"
+#include "orte/mca/rmgr/rmgr.h"
 #include "orte/mca/rml/rml.h"
-#include "orte/mca/sds/base/base.h"
-#include "orte/mca/soh/soh.h"
+#include "orte/mca/smr/smr.h"
 #include "orte/util/univ_info.h"
 #include "orte/util/session_dir.h"
 #include "orte/runtime/orte_wait.h"
 
-extern char **environ;
+/* remove for ORTE 2.0 */
+#include "orte/mca/sds/base/base.h"
+
+#include "orte/mca/pls/pls.h"
+#include "orte/mca/pls/poe/pls_poe.h"
 
 /*
  * Local functions
  */
-static int pls_poe_launch(orte_jobid_t jobid);
-static int pls_poe_terminate_job(orte_jobid_t jobid);
+static int pls_poe_launch_job(orte_jobid_t jobid);
+static int pls_poe_terminate_job(orte_jobid_t jobid, struct timeval *timeout, opal_list_t *attrs);
+static int pls_poe_terminate_orteds(orte_jobid_t jobid, struct timeval *timeout, opal_list_t *attrs);
 static int pls_poe_terminate_proc(const orte_process_name_t *name);
+static int pls_poe_signal_job(orte_jobid_t jobid, int32_t signal, opal_list_t *attrs);
+static int pls_poe_signal_proc(const orte_process_name_t *name, int32_t signal);
 static int pls_poe_finalize(void);
+static int pls_poe_cancel_operation(void);
 
-orte_pls_base_module_1_0_0_t orte_pls_poe_module = {
-    pls_poe_launch,
+orte_pls_base_module_t orte_pls_poe_module = {
+    pls_poe_launch_job,
     pls_poe_terminate_job,
+    pls_poe_terminate_orteds,
     pls_poe_terminate_proc,
+    pls_poe_signal_job,
+    pls_poe_signal_proc,
+    pls_poe_cancel_operation,
     pls_poe_finalize
 };
 
@@ -72,7 +87,7 @@ poe_set_handler_default - set signal handler to default
 static void poe_set_handler_default(int sig)
 {
     struct sigaction act;
-                                                                                                      
+
     act.sa_handler = SIG_DFL;
     act.sa_flags = 0;
     sigemptyset(&act.sa_mask);
@@ -87,7 +102,7 @@ poe_argv_append_int - append integer variable to argument variable
 @param min minimum value [IN]
 @param argname argument name [IN]
 */
-static inline int poe_argv_append_int(int *argc, char ***argv, int varname, int min, char *argname) 
+static inline int poe_argv_append_int(int *argc, char ***argv, int varname, int min, char *argname)
 {
     char *tmp_string;
     if(varname >= min) {
@@ -97,7 +112,7 @@ static inline int poe_argv_append_int(int *argc, char ***argv, int varname, int
         free(tmp_string);
     } else {
         return ORTE_ERR_BAD_PARAM;
-    } 
+    }
     return ORTE_SUCCESS;
 }
 
@@ -111,7 +126,7 @@ int pls_poe_launch_interactive_orted(orte_jobid_t jobid)
 {
     opal_list_t nodes, mapping_list;
     opal_list_item_t* item;
-    size_t num_nodes;
+    orte_std_cntr_t num_nodes;
     orte_vpid_t vpid;
     int node_name_index1;
     int node_name_index2;
@@ -145,11 +160,11 @@ int pls_poe_launch_interactive_orted(orte_jobid_t jobid)
     if(ORTE_SUCCESS != rc) {
         goto cleanup;
     }
-                                                                                                       
+
     /*
      * Allocate a range of vpids for the daemons.
      */
-     
+
     num_nodes = opal_list_get_size(&nodes);
     if(num_nodes == 0) {
         return ORTE_ERR_BAD_PARAM;
@@ -158,7 +173,7 @@ int pls_poe_launch_interactive_orted(orte_jobid_t jobid)
     if(ORTE_SUCCESS != rc) {
         goto cleanup;
     }
-     
+
     /* application */
     argv = opal_argv_copy(opal_argv_split(mca_pls_poe_component.orted, ' '));
     argc = opal_argv_count(argv);
@@ -166,7 +181,7 @@ int pls_poe_launch_interactive_orted(orte_jobid_t jobid)
          opal_argv_append(&argc, &argv, "--debug");
     }
     opal_argv_append(&argc, &argv, "--debug-daemons");
-     
+
     opal_argv_append(&argc, &argv, "--no-daemonize");
     opal_argv_append(&argc, &argv, "--bootproxy");
     /* need integer value for command line parameter - NOT hex */
@@ -187,7 +202,7 @@ int pls_poe_launch_interactive_orted(orte_jobid_t jobid)
     opal_argv_append(&argc, &argv, tmp_string);
     free(tmp_string);
 
-                                        
+
     /* setup ns contact info */
     opal_argv_append(&argc, &argv, "--nsreplica");
     if(NULL != orte_process_info.ns_replica_uri) {
@@ -198,7 +213,7 @@ int pls_poe_launch_interactive_orted(orte_jobid_t jobid)
     asprintf(&param, "\"%s\"", uri);
     opal_argv_append(&argc, &argv, param);
     free(uri);
-                                           
+
     /* setup gpr contact info */
     opal_argv_append(&argc, &argv, "--gprreplica");
     if(NULL != orte_process_info.gpr_replica_uri) {
@@ -209,24 +224,24 @@ int pls_poe_launch_interactive_orted(orte_jobid_t jobid)
     asprintf(&param, "\"%s\"", uri);
     opal_argv_append(&argc, &argv, param);
     free(uri);
-     
+
     /*
      * Iterate through each of the nodes and spin
      * up a daemon.
      */
-     
+
     for(item =  opal_list_get_first(&nodes);
         item != opal_list_get_end(&nodes);
         item =  opal_list_get_next(item)) {
         orte_ras_node_t* node = (orte_ras_node_t*)item;
         orte_process_name_t* name;
         pid_t pid;
-     
+
         /* setup node name */
         argv[node_name_index2] = node->node_name;
 
-        fprintf(hfp,"%s\n",node->node_name); 
-     
+        fprintf(hfp,"%s\n",node->node_name);
+
         /* initialize daemons process name */
         rc = orte_ns.create_process_name(&name, node->node_cellid, 0, vpid);
         if(ORTE_SUCCESS != rc) {
@@ -242,17 +257,17 @@ int pls_poe_launch_interactive_orted(orte_jobid_t jobid)
         }
         argv[proc_name_index] = name_string;
         for(i=0;i<argc;i++) {
-           fprintf(cfp,"%s ",argv[i]); 
+           fprintf(cfp,"%s ",argv[i]);
         }
         fprintf(cfp,"\n");
 
         if (mca_pls_poe_component.verbose) {
            opal_output(0, "%s:cmdfile %s\n", __FUNCTION__, opal_argv_join(argv, ' '));
-        }  
+        }
         vpid++;
         free(name);
     }
-   
+
     fclose(cfp);
     fclose(hfp);
 
@@ -283,21 +298,21 @@ int pls_poe_launch_interactive_orted(orte_jobid_t jobid)
 
     if (mca_pls_poe_component.verbose) {
        opal_output(0, "%s:cmdline %s\n", __FUNCTION__, opal_argv_join(argv, ' '));
-    }  
+    }
 
     pid = fork();
     if(pid < 0) {
         rc = ORTE_ERR_OUT_OF_RESOURCE;
         goto cleanup;
     }
-     
+
     /* child */
     if(pid == 0) {
        execv(mca_pls_poe_component.path, argv);
        opal_output(0, "orte_pls_poe: execv failed with errno=%d\n", errno);
        exit(-1);
-    } 
-     
+    }
+
 cleanup:
     while(NULL != (item = opal_list_remove_first(&nodes))) {
         OBJ_RELEASE(item);
@@ -324,33 +339,34 @@ poe_wait_job - call back when POE finish
 */
 static void poe_wait_job(pid_t pid, int status, void* cbdata)
 {
-    opal_list_t map;
-    opal_list_item_t* item;
+    orte_job_map_t *map;
+    opal_list_item_t *item, *item2;
     int rc;
 
     /* query allocation for the job */
-    OBJ_CONSTRUCT(&map, opal_list_t);
-    rc = orte_rmaps_base_get_map(mca_pls_poe_component.jobid,&map);
+    rc = orte_rmaps.get_job_map(&map, mca_pls_poe_component.jobid);
     if(ORTE_SUCCESS != rc) {
         ORTE_ERROR_LOG(rc);
     }
 
-    for(item =  opal_list_get_first(&map);
-        item != opal_list_get_end(&map);
+    for(item =  opal_list_get_first(&map->nodes);
+        item != opal_list_get_end(&map->nodes);
         item =  opal_list_get_next(item)) {
-        orte_rmaps_base_map_t* map = (orte_rmaps_base_map_t*) item;
-        size_t i;
-                
-        for(i = 0 ; i < map->num_procs ; ++i) {
-            orte_session_dir_finalize(&(map->procs[i])->proc_name);
-            rc = orte_soh.set_proc_soh(&(map->procs[i]->proc_name),
+        orte_mapped_node_t* node = (orte_mapped_node_t*) item;
+
+        for (item2 = opal_list_get_first(&node->procs);
+             item2 != opal_list_get_end(&node->procs);
+             item2 = opal_list_get_next(item2)) {
+            orte_mapped_proc_t* proc = (orte_mapped_proc_t*)item2;
+            
+            orte_session_dir_finalize(&(proc->name));
+            rc = orte_smr.set_proc_state(&(proc->name),
                                         ORTE_PROC_STATE_ABORTED, status);
-        }
-        if(ORTE_SUCCESS != rc) {
-            ORTE_ERROR_LOG(rc);
+            if(ORTE_SUCCESS != rc) {
+                ORTE_ERROR_LOG(rc);
+            }
         }
     }
-    OBJ_DESTRUCT(&map);
 }
 
 /**
@@ -365,7 +381,7 @@ poe_create_cmd_file - create POE command file
 static int poe_create_cmd_file(
     FILE *cfp,
     orte_app_context_t* context,
-    orte_rmaps_base_proc_t* proc,
+    orte_mapped_proc_t* proc,
     orte_vpid_t vpid_start,
     orte_vpid_t vpid_range)
 {
@@ -414,7 +430,7 @@ static int poe_create_cmd_file(
     free(uri);
 
     /* push name into environment */
-    orte_ns_nds_env_put(&proc->proc_name, vpid_start, vpid_range, &environ_copy);
+    orte_ns_nds_env_put(&proc->name, vpid_start, vpid_range, &environ_copy);
 
     if (context->argv == NULL) {
         context->argv = malloc(sizeof(char*)*2);
@@ -426,16 +442,16 @@ static int poe_create_cmd_file(
     fprintf(cfp,"%s",mca_pls_poe_component.env);
     while(environ_copy[i]!=NULL) {
         fprintf(cfp," %s",environ_copy[i++]);
-    } 
+    }
     opal_argv_free(environ_copy);
     fprintf(cfp," %s",context->app);
     i=1;
     while(context->argv[i]!=NULL) {
         fprintf(cfp," %s",context->argv[i++]);
-    } 
+    }
 
     /* POE will upset if the file doesn't contain end of line. */
-    fprintf(cfp,"\n"); 
+    fprintf(cfp,"\n");
 
     return ORTE_SUCCESS;
 }
@@ -445,19 +461,19 @@ poe_launch_interactive - launch an interactive job
 @param jobid JOB Identifier [IN]
 @return error number
 */
-static inline int poe_launch_interactive(orte_jobid_t jobid)
+static inline int poe_launch_interactive_job(orte_jobid_t jobid)
 {
-    opal_list_t map, nodes, mapping_list;
-    opal_list_item_t* item;
+    orte_job_map_t *map;
+    opal_list_item_t *item, *item2;
     orte_vpid_t vpid_start, vpid_range;
-    size_t num_nodes, num_procs;
+    orte_std_cntr_t num_nodes, num_procs;
     FILE *hfp, *cfp;
     char** argv;
     int argc;
     int rc, pid;
     sigset_t sigs;
 
-    if( (NULL==(mca_pls_poe_component.cmdfile=tempnam(NULL,NULL))) || 
+    if( (NULL==(mca_pls_poe_component.cmdfile=tempnam(NULL,NULL))) ||
         (NULL==(cfp=fopen(mca_pls_poe_component.cmdfile,"w"))) ) {
         ORTE_ERROR_LOG(ORTE_ERR_OUT_OF_RESOURCE);
         return ORTE_ERR_OUT_OF_RESOURCE;
@@ -465,50 +481,47 @@ static inline int poe_launch_interactive(orte_jobid_t jobid)
 
     mca_pls_poe_component.jobid = jobid;
 
-    OBJ_CONSTRUCT(&nodes, opal_list_t);
-    OBJ_CONSTRUCT(&mapping_list, opal_list_t);
-    rc = orte_rmaps_base_mapped_node_query(&mapping_list, &nodes, jobid);
+    /* get the map for this job */
+    rc = orte_rmaps.get_job_map(&map, jobid);
     if (ORTE_SUCCESS != rc) { ORTE_ERROR_LOG(rc); goto cleanup; }
-     
-    num_nodes = opal_list_get_size(&nodes);
+    
+    num_nodes = opal_list_get_size(&map->nodes);
 
     if(!strncmp(mca_pls_poe_component.resource_allocation,"hostfile",8)) {
 
-        /* Create a tempolary hostlist file if user specify */
+        /* Create a temporary hostlist file if user specify */
 
         if( (NULL==(mca_pls_poe_component.hostfile=tempnam(NULL,NULL))) ||
             (NULL==(hfp=fopen(mca_pls_poe_component.hostfile,"w"))) ) {
             return ORTE_ERR_OUT_OF_RESOURCE;
         }
-        for(item =  opal_list_get_first(&nodes);
-            item != opal_list_get_end(&nodes);
+        for(item =  opal_list_get_first(&map->nodes);
+            item != opal_list_get_end(&map->nodes);
             item =  opal_list_get_next(item)) {
-            orte_ras_node_t* node = (orte_ras_node_t*)item;
-            fprintf(hfp,"%s\n",node->node_name); 
+            orte_mapped_node_t* node = (orte_mapped_node_t*)item;
+            fprintf(hfp,"%s\n",node->nodename);
         }
         fclose(hfp);
     }
 
-    rc = orte_rmgr_base_get_job_slots(jobid, &num_procs);
-    if (ORTE_SUCCESS != rc) { ORTE_ERROR_LOG(rc); goto cleanup; }
-    
-    OBJ_CONSTRUCT(&map, opal_list_t);
-    rc = orte_rmaps_base_get_map(jobid,&map);
+    rc = orte_rmgr.get_vpid_range(jobid, &vpid_start, &vpid_range);
     if (ORTE_SUCCESS != rc) { ORTE_ERROR_LOG(rc); goto cleanup; }
 
-    rc = orte_rmaps_base_get_vpid_range(jobid, &vpid_start, &vpid_range);
-    if (ORTE_SUCCESS != rc) { ORTE_ERROR_LOG(rc); goto cleanup; }
+    /* Create a temporary POE command file */
 
-    /* Create a tempolary POE command file */
-
-    for(item =  opal_list_get_first(&map);
-        item != opal_list_get_end(&map);
+    num_procs = 0;
+    for(item =  opal_list_get_first(&map->nodes);
+        item != opal_list_get_end(&map->nodes);
         item =  opal_list_get_next(item)) {
-        orte_rmaps_base_map_t* map2 = (orte_rmaps_base_map_t*)item;
-        size_t i;
-        for(i=0; i<map2->num_procs; i++) {
-            rc = poe_create_cmd_file(cfp, map2->app, map2->procs[i], vpid_start, vpid_range);
+        orte_mapped_node_t* node = (orte_mapped_node_t*)item;
+
+        for (item2 = opal_list_get_first(&node->procs);
+             item2 != opal_list_get_end(&node->procs);
+             item2 = opal_list_get_next(item2)) {
+            orte_mapped_proc_t* proc = (orte_mapped_proc_t*)item2;
+            rc = poe_create_cmd_file(cfp, map->apps[proc->app_idx], proc, vpid_start, vpid_range);
             if(ORTE_SUCCESS != rc) { ORTE_ERROR_LOG(rc); goto cleanup; }
+            num_procs++;
         }
     }
     fclose(cfp);
@@ -547,7 +560,7 @@ static inline int poe_launch_interactive(orte_jobid_t jobid)
 
     if(mca_pls_poe_component.verbose>10) {
         opal_output(0, "POE cmdline %s\n", opal_argv_join(argv, ' '));
-    }  
+    }
 
     /* Start job with POE */
 
@@ -570,23 +583,11 @@ static inline int poe_launch_interactive(orte_jobid_t jobid)
     } else {
         orte_wait_cb(pid, poe_wait_job, NULL);
     }
-   
+
 
 cleanup:
-    while(NULL != (item = opal_list_remove_first(&map))) {
-        OBJ_RELEASE(item);
-    }
-    OBJ_DESTRUCT(&map);
-    while(NULL != (item = opal_list_remove_first(&nodes))) {
-        OBJ_RELEASE(item);
-    }
-    OBJ_DESTRUCT(&nodes);
-
-    while(NULL != (item = opal_list_remove_first(&mapping_list))) {
-        OBJ_RELEASE(item);
-    }
-    OBJ_DESTRUCT(&mapping_list);
-
+    OBJ_RELEASE(map);
+    
     return rc;
 }
 
@@ -596,15 +597,15 @@ pls_poe_launch - launch a POE job
 @param jobid JOB Identifier [IN]
 @return error number
 */
-static int pls_poe_launch(orte_jobid_t jobid)
+static int pls_poe_launch_job(orte_jobid_t jobid)
 {
-    if(!strncmp(mca_pls_poe_component.class,"interactive",11)) {
-        return poe_launch_interactive(jobid);
+    if(0 == strncmp(mca_pls_poe_component.class,"interactive",11)) {
+        return poe_launch_interactive_job(jobid);
     }
     return ORTE_ERR_NOT_IMPLEMENTED;
 }
 
-static int pls_poe_terminate_job(orte_jobid_t jobid)
+static int pls_poe_terminate_job(orte_jobid_t jobid, struct timeval *timeout, opal_list_t *attrs)
 {
     return ORTE_ERR_NOT_IMPLEMENTED;
 }
@@ -615,8 +616,24 @@ static int pls_poe_terminate_proc(const orte_process_name_t *name)
     return ORTE_ERR_NOT_IMPLEMENTED;
 }
 
+static int pls_poe_terminate_orteds(orte_jobid_t jobid, struct timeval *timeout, opal_list_t *attrs)
+{
+    return ORTE_ERR_NOT_IMPLEMENTED;
+}
+
+static int pls_poe_signal_job(orte_jobid_t jobid, int32_t signal, opal_list_t *attrs)
+{
+    return ORTE_ERR_NOT_IMPLEMENTED;
+}
+
+
+static int pls_poe_signal_proc(const orte_process_name_t *name, int32_t signal)
+{
+    return ORTE_ERR_NOT_IMPLEMENTED;
+}
+
 /**
-pls_poe_finalize - clean up tempolary files
+pls_poe_finalize - clean up temporary files
 @return error number
 */
 static int pls_poe_finalize(void)
