@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -40,9 +40,8 @@ struct orte_ras_bjs_component_t {
 };
 typedef struct orte_ras_bjs_component_t orte_ras_bjs_component_t;
 
-OMPI_COMP_EXPORT extern orte_ras_bjs_component_t mca_ras_bjs_component;
-OMPI_COMP_EXPORT extern orte_ras_base_module_t orte_ras_bjs_module;
-
+ORTE_DECLSPEC extern orte_ras_bjs_component_t mca_ras_bjs_component;
+ORTE_DECLSPEC extern orte_ras_base_module_t orte_ras_bjs_module;
 
 #if defined(c_plusplus) || defined(__cplusplus)
 }
