@@ -34,14 +34,15 @@
 #include "orte/mca/ns/ns.h"
 #include "orte/mca/errmgr/errmgr.h"
 #include "orte/mca/ras/ras.h"
-#include "orte/mca/ras/base/ras_base_node.h"
+#include "orte/mca/ras/base/ras_private.h"
+#include "orte/runtime/runtime_types.h"
+
 #include "orte/mca/rds/rds.h"
-#include "orte/mca/rds/base/base.h"
+#include "orte/mca/rds/base/rds_private.h"
 #include "orte/mca/rds/hostfile/rds_hostfile.h"
 #include "orte/mca/rds/hostfile/rds_hostfile_lex.h"
 
-#include "orte/runtime/runtime_types.h"
-
+static bool orte_rds_hostfile_queried = false;
 static orte_cellid_t local_cellid;
 static bool need_cellid = true;
 static char *cur_hostfile_name = NULL;
@@ -125,6 +126,7 @@ static int orte_rds_hostfile_parse_line(int token, opal_list_t* existing, opal_l
     orte_ras_node_t* node;
     bool update = false;
     bool got_count = false;
+    bool got_max = false;
 
     if (ORTE_RDS_HOSTFILE_STRING == token ||
         ORTE_RDS_HOSTFILE_HOSTNAME == token ||
@@ -260,10 +262,11 @@ static int orte_rds_hostfile_parse_line(int token, opal_list_t* existing, opal_l
                 return ORTE_ERROR;
             }
             /* Only take this update if it puts us >= node_slots */
-            if (((size_t) rc) >= node->node_slots) {
-                if (node->node_slots_max != (size_t)rc) {
+            if (rc >= node->node_slots) {
+                if (node->node_slots_max != rc) {
                     node->node_slots_max = rc;
                     update = true;
+                    got_max = true;
                 }
             } else {
                 opal_show_help("help-rds-hostfile.txt", "rds:max_slots_lt",
@@ -285,7 +288,11 @@ static int orte_rds_hostfile_parse_line(int token, opal_list_t* existing, opal_l
 done:
     if (update) {
         if (!got_count) {
-            ++node->node_slots;
+            if (got_max) {
+                node->node_slots = node->node_slots_max;
+            } else {
+                ++node->node_slots;
+            }
         }
         opal_list_append(updates, &node->super);
     } else {
@@ -311,9 +318,6 @@ static int orte_rds_hostfile_parse(const char *hostfile, opal_list_t* existing,
     orte_rds_hostfile_done = false;
     orte_rds_hostfile_in = fopen(hostfile, "r");
     if (NULL == orte_rds_hostfile_in) {
-        opal_show_help("help-rds-hostfile.txt", "rds:no-hostfile",
-                       true,
-                       cur_hostfile_name);
         rc = ORTE_ERR_NOT_FOUND;
         goto unlock;
     }
@@ -368,7 +372,7 @@ unlock:
  * rds_hostfile_path, and add the nodes to the registry.
  */
 
-static int orte_rds_hostfile_query(void)
+static int orte_rds_hostfile_query(orte_jobid_t job)
 {
     opal_list_t existing;
     opal_list_t updates, rds_updates;
@@ -378,6 +382,19 @@ static int orte_rds_hostfile_query(void)
     orte_ras_node_t *ras_item;
     int rc;
 
+    if (orte_rds_hostfile_queried) {
+        /* if we have already been queried, then
+         * our info is on the registry, so just
+         * return. Note that this restriction
+         * may eventually be lifted - ideally, 
+         * we might check to see if this is a
+         * new file name and go ahead with the
+         * query if so.
+         */
+        return ORTE_SUCCESS;
+    }
+    orte_rds_hostfile_queried = true;
+    
     OBJ_CONSTRUCT(&existing, opal_list_t);
     OBJ_CONSTRUCT(&updates, opal_list_t);
     OBJ_CONSTRUCT(&rds_updates, opal_list_t);
@@ -394,7 +411,9 @@ static int orte_rds_hostfile_query(void)
         if(mca_rds_hostfile_component.default_hostfile) {
             rc = ORTE_SUCCESS;
         } else {
-            opal_output(0, "orte_rds_hostfile: could not open %s\n", mca_rds_hostfile_component.path);
+            opal_show_help("help-rds-hostfile.txt", "rds:no-hostfile",
+                           true,
+                           mca_rds_hostfile_component.path);
         }
         goto cleanup;
     } else if (ORTE_SUCCESS != rc) {
@@ -482,6 +501,18 @@ static int orte_rds_hostfile_query(void)
         if (ORTE_SUCCESS != rc) {
             goto cleanup;
         }
+        
+        /* and now, indicate that ORTE should override any oversubscribed conditions
+         * based on local hardware limits since the user (a) might not have
+         * provided us any info on the #slots for a node, and (b) the user
+         * might have been wrong! If we don't check the number of local physical
+         * processors, then we could be too aggressive on our sched_yield setting
+         * and cause performance problems.
+         */
+        rc = orte_ras_base_set_oversubscribe_override(job);
+        if (ORTE_SUCCESS != rc) {
+            goto cleanup;
+        }
     }
 
 cleanup:
@@ -513,14 +544,7 @@ cleanup:
 }
 
 
-static int orte_rds_hostfile_finalize(void)
-{
-    return ORTE_SUCCESS;
-}
-
-
 orte_rds_base_module_t orte_rds_hostfile_module = {
     orte_rds_hostfile_query,
-    orte_rds_base_store_resource,
-    orte_rds_hostfile_finalize
+    orte_rds_base_store_resource
 };
