@@ -2,7 +2,7 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart, 
@@ -28,9 +28,9 @@ extern "C" {
 #endif
 
 /** Global URM component */
-OMPI_COMP_EXPORT extern orte_rmgr_base_component_t mca_rmgr_cnos_component;
+ORTE_DECLSPEC extern orte_rmgr_base_component_t mca_rmgr_cnos_component;
 /** Global URM module */
-OMPI_COMP_EXPORT extern orte_rmgr_base_module_t orte_rmgr_cnos_module;
+extern orte_rmgr_base_module_t orte_rmgr_cnos_module;
 
 #if defined(c_plusplus) || defined(__cplusplus)
 }
