@@ -2,13 +2,15 @@
  * Copyright (c) 2004-2005 The Trustees of Indiana University and Indiana
  *                         University Research and Technology
  *                         Corporation.  All rights reserved.
- * Copyright (c) 2004-2005 The University of Tennessee and The University
+ * Copyright (c) 2004-2006 The University of Tennessee and The University
  *                         of Tennessee Research Foundation.  All rights
  *                         reserved.
  * Copyright (c) 2004-2005 High Performance Computing Center Stuttgart,
  *                         University of Stuttgart.  All rights reserved.
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
+ * Copyright (c) 2007      Los Alamos National Security, LLC.  All rights
+ *                         reserved. 
  * $COPYRIGHT$
  *
  * Additional copyrights may follow
@@ -57,7 +59,7 @@
 #include "orte/util/proc_info.h"
 #include "orte/util/session_dir.h"
 #include "orte/util/universe_setup_file_io.h"
-#include "orte/mca/soh/soh.h"
+#include "orte/mca/smr/smr.h"
 #include "orte/mca/rml/rml.h"
 #include "orte/mca/rds/rds_types.h"
 #include "orte/mca/ns/ns.h"
@@ -66,8 +68,6 @@
 #include "orte/runtime/runtime.h"
 #include "orte/runtime/orte_setup_hnp.h"
 
-extern char **environ;
-
 /* Local condition variables and mutex
  */
 static opal_mutex_t orte_setup_hnp_mutex;
@@ -97,7 +97,6 @@ static void orte_setup_hnp_wait(pid_t wpid, int status, void *data);
  */
 int orte_setup_hnp(char *target_cluster, char *headnode, char *username)
 {
-#ifndef __WINDOWS__
     char **argv, *param, *uri, *uid, *hn=NULL;
     char *path, *name_string, *orteprobe;
     int argc, rc=ORTE_SUCCESS, id, intparam;
@@ -106,7 +105,7 @@ int orte_setup_hnp(char *target_cluster, char *headnode, char *username)
     orte_cellid_t cellid=ORTE_CELLID_MAX, *cptr;
     orte_jobid_t jobid;
     orte_vpid_t vpid;
-    size_t i, j, k, cnt=0;
+    orte_std_cntr_t i, j, k, cnt=0;
     orte_gpr_value_t **values=NULL, *value;
     orte_gpr_keyval_t **keyvals;
     char *keys[4], *tokens[3], *cellname;
@@ -139,7 +138,7 @@ int orte_setup_hnp(char *target_cluster, char *headnode, char *username)
         keyvals = values[0]->keyvals;
         for (i=0; i < values[0]->cnt; i++) {
             if (0 == strcmp(keyvals[i]->key, ORTE_RDS_FE_NAME)) {
-                hn = strdup(keyvals[i]->value->data);
+                hn = strdup((const char*)keyvals[i]->value->data);
                 continue;
             }
             if (0 == strcmp(keyvals[i]->key, ORTE_RDS_FE_SSH)) {
@@ -185,7 +184,7 @@ int orte_setup_hnp(char *target_cluster, char *headnode, char *username)
             keyvals = values[i]->keyvals;
             for (j=0; j < values[i]->cnt; j++) {
                 if ((0 == strcmp(keyvals[j]->key, ORTE_RDS_FE_NAME)) &&
-                     0 == strcmp(keyvals[j]->value->data, headnode)) {
+                     0 == strcmp((const char*)keyvals[j]->value->data, headnode)) {
                     /* okay, this is the right cell - now need to find
                      * the ssh flag (if provided) and cellid
                      */
@@ -332,7 +331,7 @@ MOVEON:
     OBJ_CONSTRUCT(&orte_setup_hnp_condition, opal_condition_t);
 
     /* get a jobid for the probe */
-    rc = orte_ns.create_jobid(&jobid);
+    rc = orte_ns.create_jobid(&jobid, NULL);
     if (ORTE_SUCCESS != rc) {
         ORTE_ERROR_LOG(rc);
         return rc;
@@ -450,13 +449,14 @@ MOVEON:
     }
 
     /* issue the non-blocking recv to get the probe's findings */
-    rc = orte_rml.recv_buffer_nb(ORTE_RML_NAME_ANY, ORTE_RML_TAG_PROBE,
+    rc = orte_rml.recv_buffer_nb(ORTE_NAME_WILDCARD, ORTE_RML_TAG_PROBE,
                                  0, orte_setup_hnp_recv, NULL);
     if(rc < 0) {
         ORTE_ERROR_LOG(rc);
         goto CLEANUP;
     }
 
+#ifndef __WINDOWS__
     /* fork a child to exec the rsh/ssh session */
     orte_setup_hnp_rc = ORTE_SUCCESS;
     pid = fork();
@@ -486,52 +486,21 @@ MOVEON:
         OPAL_THREAD_UNLOCK(&orte_setup_hnp_mutex);
 
         if (ORTE_SUCCESS == orte_setup_hnp_rc) {
-            /* Remember if we were infrastructre or not */
+            /* Remember if we were infrastructure or not */
             id = mca_base_param_find("orte", NULL, "infrastructure");
             mca_base_param_lookup_int(id, &intparam);
             if ( ((int)true) != intparam) {
                 infrastructure = false;
             }
 
-            /* need to restart the local system so it can connect to the remote daemon.
-             * we only want to clear the run-time itself - we cannot close the OPAL
-             * utilities, though, or we will lose all of our MCA parameters
-             */
-            orte_system_finalize();
-
-            /*
-             * now set the relevant MCA parameters to point us at the remote daemon...
-             */
-            rc = opal_setenv("OMPI_MCA_gpr_replica_uri",
-                             orte_setup_hnp_orted_uri, true, &environ);
-            if (ORTE_SUCCESS != rc) {
-                fprintf(stderr, "orte_setup_hnp: could not set gpr_replica_uri in environ\n");
-                return rc;
-            }
-
-            rc = opal_setenv("OMPI_MCA_ns_replica_uri",
-                             orte_setup_hnp_orted_uri, true, &environ);
-            if (ORTE_SUCCESS != rc) {
-                fprintf(stderr, "orte_setup_hnp: could not set ns_replica_uri in environ\n");
-                return rc;
-            }
-
-            opal_unsetenv("OMPI_MCA_seed", &environ);
-
-            rc = opal_setenv("OMPI_MCA_universe_uri",
-                             orte_setup_hnp_orted_uri, true, &environ);
-            if (ORTE_SUCCESS != rc) {
-                fprintf(stderr, "orte_setup_hnp: could not set universe_uri in environ\n");
-                return rc;
-            }
-
-            /*
-             * ...re-init ourselves...
-             */
-            rc = orte_system_init(infrastructure);
-            if (ORTE_SUCCESS != rc) {
-                ORTE_ERROR_LOG(rc);
-                return rc;
+            /* need to restart the local system so it can connect to the remote daemon. */
+            if (ORTE_SUCCESS != (rc = orte_restart(orte_setup_hnp_cbdata.name, orte_setup_hnp_orted_uri))) {
+               /** can't use ORTE_ERROR_LOG here as it may no longer be valid. Since we may
+                * have gotten part way through the shutdown/restart process, we can't have
+                * any idea of our current state - all we can really do at this point is
+                * abort
+                */
+                fprintf(stderr, "orte_setup_hnp: aborted during restart of local process\n");
             }
 
             /*
@@ -542,22 +511,21 @@ MOVEON:
 
         return orte_setup_hnp_rc;
     }
-
-CLEANUP:
-    return rc;
-
 #else
-    printf ("This function has not been implemented in windows yet, file %s line %d\n", __FILE__, __LINE__);
+    ORTE_ERROR_LOG(ORTE_ERROR);
+    opal_output(0, "This function has not been implemented in windows yet, file %s line %d\n", __FILE__, __LINE__);
     abort();
 #endif
 
+CLEANUP:
+    return rc;
 }
 
 static void orte_setup_hnp_recv(int status, orte_process_name_t* sender,
                                 orte_buffer_t* buffer, orte_rml_tag_t tag,
                                 void* cbdata)
 {
-    size_t n=1;
+    orte_std_cntr_t n=1;
     int rc;
 
     OPAL_THREAD_LOCK(&orte_setup_hnp_mutex);
